#===============================================================
#
#   wordrep.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;


BEGIN
{
my $cols = 80;
if (-t STDOUT)
{
use Term::ReadKey qw(GetTerminalSize);
($cols, undef) = GetTerminalSize();
}
print '=' x ($cols - 5) . "\n";
}




use File::Glob ':glob';      # Override glob built-in




sub wordrep_file($);
sub find_files_recursively(@);
sub recurse_dir($);
sub find_files(@);
sub warning($;@);
sub error($;@);
sub error_exit($;@);
sub fatal($;@);
sub usage();
sub env_say($@);




my $PREFIX = 'WORDREP';
my $PREFIX_ = ' ' x ((length $PREFIX) + 1);

$| = 1;
my $RC = 0;
my @MSG_LINES;

my $IS_WIN32 = ($^O eq 'MSWin32');

my @REL_FILE_SPECS;
my @RECURSE_WILD_SPECS;
my $IS_TEST_MODE = 0;
my $MUST_RECURSE = 0;

my @REP_REFS;


my $NR_FILES_MODIFIED = 0;
my $NR_FILES_WRITTEN = 0;








=optdoc
use glo::genopt;

my @genopts = (
[ '<1>', 'command_or_commandsfile',  'ssm', "", [ "Replace Command or filespec of file containing Replace Commands",
"Replace Command: <delim><find><delim><replace><delim>[w|b|e|a|r][i]",
"- w = find word, b = find begin word, e = find end word",
"- a = find any,  r = find Perl regular expression",
"- i = ignore case of search string",
"- \\s in replace string is converted to single space",
"commandsfile:  Empty lines and lines beginning with '#' are ignored" ] ],
[ '<*>', 'files',	 'sso',	"", "Files to process" ],
[ 't', 'test',	 'bso',	 0, "test only - do not make modifications" ],
[ 'r', 'recurse',	 'bso',	 0, "recurse directory looking for -f= files" ],
[ 'f', 'wild-files', 'sao',	"", "wild-card files to process" ],
);
GENOPT_set_flag_prefix( '-');
GENOPT_set_optdefs( 'wordrep', \@genopts,
'Replace words/strings in files',
"Directory/Files starting with '.' are skipped");
GENOPT_parse();

=cut




{

if (@ARGV == 1 && $ARGV[0] =~ /^--?(h|help)$/)
{
usage();
exit 0;
} else
{
if (@ARGV < 2 || substr($ARGV[0], 0, 1) eq '-')
{
env_say( 1, "Not enough arguments");
usage();
exit 9;
}
}





my @finds;
{
my $find = shift @ARGV;
if (-f $find)
{
env_say( 1, "WordRep File: '$find' found");
open( my $in_fh, '<', $find) or fatal( "Cannot open '$find'", $!);
my @lines = <$in_fh>;
close $in_fh;
foreach my $line (@lines)
{
$line =~ s/\s+$//;
if ($line ne '' && substr($line, 0, 1) ne '#')
{
push @finds, $line;
}
}
} else
{
my $delim = substr( $find, 0, 1);
my $replace;
my $replace_options;
my $command = substr( $find, 1);
($find, $replace, $replace_options) = split( quotemeta $delim, $command);
while (@ARGV && !defined $replace_options)
{
$command .= ' ' . shift( @ARGV);

($find, $replace, $replace_options) = split( quotemeta $delim, $command);
}
error_exit( "No find string specified")
if ($find eq '');
error_exit( "No replace string specified ($find)")
if (!defined $replace_options);
push @finds, $delim . $find . $delim . $replace . $delim . $replace_options;
}
}




my @rel_file_defs;
{
foreach my $arg (@ARGV)
{
if (substr($arg,0,1) eq '-')
{
if ($arg eq '-t' || $arg eq '-t+')
{
$IS_TEST_MODE = 1;
} elsif ($arg eq '-t-')
{
$IS_TEST_MODE = 0;
} elsif ($arg eq '-r' || $arg eq '-r+')
{
$MUST_RECURSE = 1;
} elsif ($arg eq '-r-')
{
$MUST_RECURSE = 0;
} elsif (substr( $arg, 0, 3) eq '-r=')
{
$MUST_RECURSE = 1;
$arg = substr ($arg, 3);

push @RECURSE_WILD_SPECS, split( ',', $arg);
} elsif (substr( $arg, 0, 3) eq '-f=')
{
$arg = substr ($arg, 3);

push @RECURSE_WILD_SPECS, split( ',', $arg);
} else
{
env_say( 1, "Invalid option '$arg'. Allowed '-t', '-r', '-r=<files>', '-f=<files>'");
usage();
exit 9;
}
} else
{
push @rel_file_defs, split( ',', $arg);
}
}
if ($MUST_RECURSE && @RECURSE_WILD_SPECS == 0)
{
env_say( 1, "No recurse (wild-)files specified. Assume *.*");
push @RECURSE_WILD_SPECS, '*.*';
}




if ($IS_WIN32)
{
foreach my $filespec (@rel_file_defs)
{
$filespec =~ tr!\\!/!;
}
}
}




foreach my $find (@finds)
{
env_say( 1, "Command: $find");

my $delim = substr( $find, 0, 1);
my $command = substr( $find, 1);
my ($find, $replace, $replace_options) = split( quotemeta $delim, $command);
error_exit( "- No find string specified")
if ($find eq '');
error_exit( "- No replace string specified ($find)")
if (!defined $replace_options);
if ($replace_options ne '')
{
$replace_options = lc $replace_options;
} else
{
$replace_options = 'w';
}

my @replace_options = split( '', $replace_options);
my $replace_mode;
my $case_mode = '';
my $is_eval_mode = 0;
foreach my $option (@replace_options)
{
if (index( 'wbear', $option) >= 0)
{
if (defined $replace_mode)
{
error_exit( "- Replace mode ('$option') already defined to '$replace_mode'");
} else
{
$replace_mode = $option;
}
} elsif ($option eq 'i')
{
$case_mode = 'i';
} else
{
error_exit( "- Invalid replace option '$option'. Allowed: [w|b|e|a|r] [i]");
}
}
$replace_mode = 'w'
if (!defined $replace_mode);




my $search;
if ($replace_mode eq 'w')			# word
{
$search = '\b' . quotemeta( $find) . '\b';
} elsif ($replace_mode eq 'b')			# begin of word
{
$search = '\b' . quotemeta( $find);
} elsif ($replace_mode eq 'e')			# end of word
{
$search = quotemeta( $find) . '\b';
} elsif ($replace_mode eq 'a')			# all
{
$search = quotemeta $find;
} elsif ($replace_mode eq 'r')			# regexp
{
$search = $find;
if ($replace =~ /\$\d/)
{
$replace = '"' . $replace . '"';
$is_eval_mode = 1;
}
} else
{
error_exit( "- Invalid option '$replace_mode'. Allowed: w, b, e, a, r");
}




$replace =~ s/\\s/ /g;
push @REP_REFS, [$search, $replace, $case_mode, $is_eval_mode];
env_say( 2, "- find=$find ($search), replace=$replace, options=$replace_options");
}




if ($MUST_RECURSE)
{
@REL_FILE_SPECS = find_files_recursively( @rel_file_defs);
} else
{
@REL_FILE_SPECS = find_files( @rel_file_defs);
}





{
my %seen;
@REL_FILE_SPECS = grep { ! $seen{ $_ }++ } @REL_FILE_SPECS;
}
}




{
if (@REL_FILE_SPECS)
{
env_say( 1, "Found " . scalar @REL_FILE_SPECS . " file(s).");
foreach my $file (@REL_FILE_SPECS)
{
wordrep_file( $file);
}
} else
{
warning( "No files found");
}
}




{
env_say( 0, '');
my $nr_files_parsed = @REL_FILE_SPECS;
env_say( 1, "Files parsed/modified/written: $nr_files_parsed / $NR_FILES_MODIFIED / $NR_FILES_WRITTEN");
}

exit $RC;




END
{
if (@MSG_LINES)
{
env_say( 1, "The following warnings/errors were encountered:");
map { env_say( 2, "$_"); } @MSG_LINES;
}

my $rc = $?;
print "\a"
if ($rc > 0 || @MSG_LINES > 0);

env_say( 1, ($rc) ? "Failed ($?)" : "Done");
}




sub wordrep_file($)
{
my ($file) = @_;

if (open( my $in_fh, '<', $file))
{



my @lines = <$in_fh>;
close $in_fh;
my $total_changes = 0;




foreach my $ref (@REP_REFS)
{
my ($search, $replace, $case_mode, $is_eval_mode) = @{$ref};





my $changes;
if ($case_mode eq 'i')
{
$changes = grep /$search/i, @lines;
} else
{
$changes = grep /$search/, @lines;
}
if ($changes > 0)
{
$total_changes += $changes;
env_say( 1, "$file ($changes)");




if ($case_mode eq 'i')
{
foreach my $line (@lines)
{
my $modified;
if ($is_eval_mode)
{
$modified = $line =~ s/$search/$replace/giee;
} else
{
$modified = $line =~ s/$search/$replace/gi;
}
print "        $line"
if ($modified);
}
} else
{
foreach my $line (@lines)
{
my $modified;
if ($is_eval_mode)
{
$modified = $line =~ s/$search/$replace/gee;
} else
{
$modified = $line =~ s/$search/$replace/g;
}
print "        $line"
if ($modified);
}
}
} else
{

}
}
if ($total_changes > 0)
{
$NR_FILES_MODIFIED++;

if ($IS_TEST_MODE)
{
env_say( 1, "Test-mode. File not written");
} else
{
if (open( my $out_fh, '>', $file))
{
print $out_fh @lines;
close $out_fh;
$NR_FILES_WRITTEN++;
} else
{
error( "Unable to write file $file", "- $!");
}
}
}
} else
{
error( "Unable to open file $file", "- $!");
}
}




sub find_files_recursively(@)
{
my @wild_dirs = @_;
my @rel_filespecs;

foreach my $wild_dir_spec (@wild_dirs)
{
$wild_dir_spec =~ s!//!/!g;
$wild_dir_spec =~ s!/$!!g;
}
env_say( 1, "Recursing (@RECURSE_WILD_SPECS) files in (@wild_dirs) ...");
foreach my $wild_dir_spec (@wild_dirs)
{
my @dir_specs;
if ($wild_dir_spec =~ /[\*\?]/)
{
push @dir_specs, grep -d $_, glob( $wild_dir_spec);
} else
{
if (-d $wild_dir_spec)
{
push @dir_specs, $wild_dir_spec;
} elsif (-f _)
{
warning( "FileSpec must be directory: '$wild_dir_spec'");
} else
{
warning( "No such directory '$wild_dir_spec'");
}
}
foreach my $dir_spec (@dir_specs)
{
push @rel_filespecs, recurse_dir( $dir_spec);
}
}

return @rel_filespecs;
}





sub recurse_dir($)
{
my ($dir,
) = @_;
my @rel_filespecs;




foreach my $wild_file (@RECURSE_WILD_SPECS)
{
my $filespec = "$dir/$wild_file";
if ($wild_file =~ /[\*\?]/)
{
push @rel_filespecs, grep ! -d $_, glob( $filespec);
} else
{
if (-f $filespec)
{
push @rel_filespecs, $filespec;
}
}
}




opendir( DIR, "$dir") || fatal( "Cannot opendir '$dir'", $?);
my @entries = readdir( DIR);
closedir( DIR);
foreach my $entry (@entries)
{
next if (substr( $entry, 0, 1) eq '.');	    # Skip directories/files starting with '.'
my $this_dir = "$dir/$entry";
push @rel_filespecs, recurse_dir( $this_dir)
if (-d $this_dir);
}

return @rel_filespecs;
}




sub find_files(@)
{
my @filespecs = @_;
my @rel_filespecs;

env_say( 1, "Files: (@filespecs)");

foreach my $filespec (@filespecs)
{
if ($filespec =~ /[\*\?]/)
{
push @rel_filespecs, glob( $filespec);
} else
{
if (-f $filespec)
{
push @rel_filespecs, $filespec
} elsif (-d _)
{
warning( "FileSpec must be file: '$filespec'");
} else
{
warning( "No such file '$filespec'");
}
}
}

return @rel_filespecs;
}




sub warning($;@)
{
my ($first_line,
@rest_lines,
) = @_;

env_say( 1, "*WARNING* $first_line");
if (@rest_lines)
{
map { env_say( 2, "$_") } @rest_lines;
}
push @MSG_LINES, "WRN: $first_line";
}




sub error($;@)
{
my ($first_line,
@rest_lines,
) = @_;

env_say( 1, "*ERROR* $first_line");
if (@rest_lines)
{
map { env_say( 2, "$_") } @rest_lines;
}
push @MSG_LINES, "ERR: $first_line";
$RC = 2
if ($RC < 2);
}




sub error_exit($;@)
{
my ($first_line,
@rest_lines,
) = @_;

env_say( 1, "*ERROR* $first_line");
if (@rest_lines)
{
map { env_say( 2, "$_") } @rest_lines;
}
push @MSG_LINES, "ERR: $first_line";

exit 2;
}




sub fatal($;@)
{
my ($first_line,
@rest_lines,
) = @_;

env_say( 1, "*FATAL* $first_line");
if (@rest_lines)
{
map { env_say( 2, "$_") } @rest_lines;
}
push @MSG_LINES, "FTL: $first_line";

exit 9;
}




sub usage()
{
print "usage is: wordrep <command> | <commands-file> <files/dirs...> [-t] [-r] [-r=<files>] [-f=<files>]\n";
print "  command = <delim><find><delim><replace><delim>[w|b|e|a|r][i]\n";
print "    w = find word, b = find begin word, e = find end word,\n";
print "    a = find any,  r = find perl regular expresion\n";
print "    i = ignore case of search string\n";
print "    \\s in replace string is converted to single space.\n";
print "    -t == test-only\n";
print "    -r == recurse directory\n";
print "    -r=<files> == recurse direcory using comma-separated wild files\n";
print "    -f=<files> == comma-separated wild files\n";
print "  commands-file: Empty lines and lines beginning with '#' are ignored\n";
print "  wild files: Directories and Files starting with '.' are skipped\n";
}




sub env_say($@)
{
my ($indent,    # 0 = no prefix, 1 = $prefix, 2 = $prefix_ - Negative = prefix_level - 1
@lines_and_or_refs
) = @_;

my @lines = map { (ref $_) ? @{$_} : $_ } @lines_and_or_refs;
if ($indent == 0)
{
map { CORE::say( $_) } @lines;
} else
{
if (abs( $indent) == 1)
{
my $first_line = shift @lines;
CORE::say( "$PREFIX: $first_line");
}
map { CORE::say( "$PREFIX_ $_") } @lines;
}
}


