#===============================================================
#
#   cleanup.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package cleanup;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CLEANUP_small_stuff
CLEANUP_system_bld
CLEANUP_subsys_bld
CLEANUP_comp_bld
CLEANUP_system_aud
CLEANUP_subsys_aud
CLEANUP_comp_aud
CLEANUP_system_make_files
CLEANUP_subsys_make_files
CLEANUP_export
CLEANUP_res
CLEANUP_currencies
CLEANUP_silo_user_directory
CLEANUP_silo_cache_directory
CLEANUP_plugin_cache_directories
CLEANUP_tmp_directory
CLEANUP_usr_files
CLEANUP_obsolete_bld_files
);
}




use glo::env;
use glo::file;
use gbsenv;
use gbsglo;
use target;
use exec;
use sws;
use swb;
use swa;




sub CLEANUP_small_stuff();
sub CLEANUP_system_bld($);
sub CLEANUP_subsys_bld($$);
sub CLEANUP_comp_bld($);
sub CLEANUP_system_aud($$);
sub CLEANUP_subsys_aud($$$);
sub CLEANUP_comp_aud($$);
sub CLEANUP_system_make_files($);
sub CLEANUP_subsys_make_files($);
sub CLEANUP_export($);
sub CLEANUP_res($);
sub CLEANUP_currencies();
sub CLEANUP_silo_user_directory();
sub CLEANUP_silo_cache_directory();
sub CLEANUP_plugin_cache_directories();
sub CLEANUP_tmp_directory();
sub CLEANUP_usr_files();
sub CLEANUP_obsolete_bld_files();

sub get_make_files($$);
sub delete_files($$);
sub delete_contents($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_PLUGIN_ROOT = ENV_getenv_perl_path( 'GBS_PLUGIN_ROOT');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv('GBS_TARGET');
my $GBS_AUDIT = ENV_getenv('GBS_AUDIT');
my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
my @GBS_TOOLS = ENV_getenv( 'GBS_TOOLS');

my @GBS_SCM_SKIPTYPES = ENV_getenv( 'GBS_SCM_SKIPTYPES');




sub CLEANUP_small_stuff()
{
ENV_say( 1, "Cleanup .lck files");
{
my @files;
push @files, map { ENV_glob( "$GBS_ROOT_PATH/dev/$_/.*.lck") } GBSGLO_subsystems_full_gbs();
push @files, "$GBS_ROOT_PATH/res/.*.lck";
delete_files( \@files, 1);
}
}




sub CLEANUP_system_bld($)
{
my ($target) = @_;

ENV_say( 1, "Cleanup for All Builds for Target '$target'...");




my @all_subsys = GBSGLO_subsystems_for_target( $target);
if (@all_subsys)
{
my $nr_subsys = @all_subsys;
my @full_gbs_subsystems;
my @non_gbs_subsystems;
foreach my $subsys (@all_subsys)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
push @full_gbs_subsystems, $subsys;
} else
{
push @non_gbs_subsystems, $subsys;
}
}
my $nr_full_subsys = @full_gbs_subsystems;
my $nr_non_full_subsys = @non_gbs_subsystems;




if (@full_gbs_subsystems)
{
ENV_say( 1, "Cleanup Build of all ($nr_full_subsys/$nr_subsys) Full-GBS SubSystems for Target '$target'");

my @files = map { ENV_glob( "$GBS_ROOT_PATH/dev/$_/comp/*/bld/$target/*") } @full_gbs_subsystems;
delete_files( \@files, 1);
}




if (@non_gbs_subsystems)
{
ENV_say( 1, "Cleanup Build of all ($nr_non_full_subsys/$nr_subsys) Non-GBS SubSystems for Target '$target'");

SWB_set( $target, 0);
foreach my $subsys (@non_gbs_subsystems)
{
ENV_say( 1, "SubSys: $subsys:");
SWS_set( $subsys, 0);
ENV_chdir( "$GBS_ROOT_PATH/dev/$subsys");
EXEC_run( 'gbssub.pl', [ 'cleanup_bld', 0 ]);
}
SWS_set( $GBS_SUBSYS, 0);   # restore
SWB_set( $GBS_TARGET, 0);   # restore
}
} else
{
ENV_say( 2, "No Builds for Target '$target'");
}
}




sub CLEANUP_subsys_bld($$)
{
my ($subsys,
$target) = @_;

if (GBSGLO_subsystem_does_build( $subsys, $target))
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{



ENV_say( 1, "Cleanup BLD of Full-GBS SubSystem '$subsys' for Target '$target'");

my @bld_files = ENV_glob( "$GBS_ROOT_PATH/dev/$subsys/comp/*/bld/$target/*");
delete_files( \@bld_files, 1);
} else
{



ENV_say( 1, "Cleanup BLD of Non-GBS SubSystem '$subsys' for Target '$target'");

SWB_set( $target, 0);
EXEC_run( 'gbssub.pl', [ 'cleanup_bld', 0 ]);
SWB_set( $GBS_TARGET, 0);   # restore
}
} else
{
ENV_say( 1, "SubSystem '$subsys' does generate for Target '$target'");
}
}




sub CLEANUP_comp_bld($)
{
my ($target) = @_;

if (GBSGLO_component_does_build( $GBS_SUBSYS, $GBS_COMPONENT, $target))
{
ENV_say( 1, "Cleanup Build of Component '$GBS_SUBSYS:$GBS_COMPONENT' for Target '$target'");

my @bld_files = ENV_glob( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT/bld/$target/*");
delete_files( \@bld_files, 1);
} else
{
ENV_say( 1, "Component '$GBS_SUBSYS:$GBS_COMPONENT' does not generate for Target '$target'");
}
}




sub CLEANUP_system_aud($$)
{
my ($audit,
$target,
) = @_;

ENV_say( 1, "Cleanup for All Audits for Audit '$audit' and Target '$target'...");




my @all_subsys = GBSGLO_subsystems_for_audit( $audit, $target);
if (@all_subsys)
{
my $nr_subsys = @all_subsys;
my @full_gbs_subsystems;
my @non_gbs_subsystems;
foreach my $subsys (@all_subsys)
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{
push @full_gbs_subsystems, $subsys;
} else
{
push @non_gbs_subsystems, $subsys;
}
}
my $nr_full_subsys = @full_gbs_subsystems;
my $nr_non_full_subsys = @non_gbs_subsystems;




if (@full_gbs_subsystems)
{
ENV_say( 1, "Cleanup Audit of all ($nr_full_subsys/$nr_subsys) Full-GBS SubSystems for Audit '$audit' and Target '$target'");

my @aud_dirs = map { ENV_glob( "$GBS_ROOT_PATH/dev/$_/comp/*/aud/$audit/$target") } @full_gbs_subsystems;
delete_contents( \@aud_dirs, undef, 1);
}




if (@non_gbs_subsystems)
{
ENV_say( 1, "Cleanup Audit of all ($nr_non_full_subsys/$nr_subsys) Non-GBS SubSystems for Audit '$audit' and Target '$target'");

SWA_set( $audit, 0);
SWB_set( $target, 0);
foreach my $subsys (@non_gbs_subsystems)
{
ENV_say( 1, "SubSys: $subsys");
SWS_set( $subsys, 0);
ENV_chdir( "$GBS_ROOT_PATH/dev/$subsys");
EXEC_run( 'gbssub.pl', [ 'cleanup_aud', 0 ]);
}
SWS_set( $GBS_SUBSYS, 0);   # restore
SWB_set( $GBS_TARGET, 0);   # restore
SWA_set( $GBS_AUDIT, 0);    # restore
}
} else
{
ENV_say( 2, "No Audits for Audit '$audit' and Target '$target'");
}
}




sub CLEANUP_subsys_aud($$$)
{
my ($subsys,
$audit,
$target,
) = @_;

if (GBSGLO_subsystem_does_audit( $subsys, $audit, $target))
{
if (GBSGLO_subsystem_is_full_gbs( $subsys))
{



ENV_say( 1, "Cleanup Audit of GBS SubSystem '$subsys' for Audit '$audit' and Target '$target'");

my @aud_dirs = ENV_glob( "$GBS_ROOT_PATH/dev/$subsys/comp/*/aud/$audit/$target");
delete_contents( \@aud_dirs, undef, 1);
} else
{



ENV_say( 1, "Cleanup Audit of non-GBS SubSystem '$subsys' for Audit '$audit' and Target '$target'");

SWA_set( $audit, 0);
SWB_set( $target, 0);
EXEC_run( 'gbssub.pl', [ 'cleanup_aud', 0 ]);
SWB_set( $GBS_TARGET, 0);   # restore
SWA_set( $GBS_AUDIT, 0);    # restore
}
} else
{
ENV_say( 1, "SubSystem '$subsys' does Audit for Audit '$audit' and Target '$target'");
}
}




sub CLEANUP_comp_aud($$)
{
my ($audit,
$target,
) = @_;

if (GBSGLO_component_does_audit( $GBS_SUBSYS, $GBS_COMPONENT, $audit, $target))
{
ENV_say( 1, "Cleanup Audit $audit of Component '$GBS_SUBSYS:$GBS_COMPONENT' for Audit '$audit' and Target '$target'");

my @aud_dirs = ENV_glob( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$GBS_COMPONENT/aud/$audit/$target");
delete_contents( \@aud_dirs, undef, 1);
} else
{
ENV_say( 1, "Component '$GBS_SUBSYS:$GBS_COMPONENT' does not Audit for Audit '$audit' and Target '$target'");
}
}




sub CLEANUP_system_make_files($)
{
my ($target) = @_;

ENV_say( 1, "Cleanup Make files of all SubSystems of $GBS_ROOT_PATH for Target '$target'");

my @make_files = map { get_make_files( $_, $target) } GBSGLO_subsystems_full_gbs();
delete_files( \@make_files, 1);
}




sub CLEANUP_subsys_make_files($)
{
my ($target) = @_;

ENV_say( 1, "Cleanup Make files of SubSystem $GBS_SUBSYS for Target '$target'");

my @make_files = get_make_files( $GBS_SUBSYS, $target);
delete_files( \@make_files, 1);
}




sub get_make_files($$)
{
my ($subsys,
$target,
) = @_;

return ( ENV_glob( "$GBS_ROOT_PATH/dev/$subsys/build/$target.mk",
"$GBS_ROOT_PATH/dev/$subsys/comp/*/bld/$target.mk",
"$GBS_ROOT_PATH/dev/$subsys/comp/*/bld/$target.ref",
)
);
}




sub CLEANUP_export($)
{
my ($subsys) = @_;	# '*' == ALL

ENV_say( 1, "Cleanup EXPORT Directory for SubSystem: $subsys");

my @dirs;
if ($subsys eq '*')
{
@dirs = map { "$GBS_ROOT_PATH/dev/$_/export" } GBSGLO_subsystems_full_gbs();
} else
{
@dirs = ("$GBS_ROOT_PATH/dev/$subsys/export");
}

delete_contents( \@dirs, undef, 1);
}




sub CLEANUP_res($)
{
my ($subsys) = @_;	# '*' == ALL



my @dirs;
if ($subsys eq '*')
{
@dirs = map { "$GBS_ROOT_PATH/res/$_" } GBSGLO_subsystems();
} else
{
@dirs = ("$GBS_ROOT_PATH/res/$subsys");
}

delete_contents( \@dirs, undef, 1);
}




sub CLEANUP_currencies()
{

my @files;

if ($GBS_ROOT_PATH eq '')
{
ENV_say( 1, "No Root: Cleanup User currencies");



my $gbs_base_dir = ENV_getenv_perl_path( 'GBS_BASE_PATH');
push @files, "$gbs_base_dir/.gbsrc";
} else
{



push @files, "$GBS_ROOT_PATH/dev/.gbsrc";




push @files, map { "$GBS_ROOT_PATH/dev/$_/comp/.gbsrc" } GBSGLO_subsystems_full_gbs();
}




delete_files( \@files, 1);
}




sub CLEANUP_silo_user_directory()
{

ENV_say( 2, "'doxygen' found. May take a long time...")
if (-e "$GBS_ROOT_PATH/silo/doxygen");

my @del_refs;




delete_contents( [ "$GBS_ROOT_PATH/silo" ], [ "^\.gbs\$" ], 1 );
}




sub CLEANUP_silo_cache_directory()
{


my @del_refs;




my @cache_dirs = ENV_glob( "$GBS_ROOT_PATH/silo/.gbs/cache/*");
delete_contents( \@cache_dirs, undef, 1);
}




sub CLEANUP_plugin_cache_directories()
{


my @del_refs;




my $gbs_plugin_path = ENV_canonicalize_path( "$GBS_PLUGIN_ROOT/..");
my @cache_dirs = ( "$gbs_plugin_path/html", ENV_glob( "$GBS_PLUGIN_ROOT/*") );
delete_contents( \@cache_dirs, undef, 1);
}




sub CLEANUP_tmp_directory()
{


my @del_refs;




my @targets_re = map { "^$_\$" } @GBS_ALL_TARGETS, @GBS_AUDITS, @GBS_TOOLS;
delete_contents( [ "$GBS_ROOT_PATH/tmp" ], \@targets_re, 1);




my @target_dirs = map { "$GBS_ROOT_PATH/tmp/$_" } @GBS_ALL_TARGETS;
delete_contents( \@target_dirs, undef, 0);
}




sub CLEANUP_usr_files()
{


my @files;
push @files, ENV_glob( "$GBS_ROOT_PATH/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/build/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/build/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/comp/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/comp/*/opt/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/dev/*/comp/*/opt/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sys/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sys/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sysaudit/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sysaudit/*/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sysbuild/*.usr");
push @files, ENV_glob( "$GBS_ROOT_PATH/sysbuild/*/*.usr");




my $gbs_tmp_dir = "$GBS_ROOT_PATH/tmp";
my $total = 0;
my $nr_moved = 0;
foreach my $filespec (@files)
{
$total++;
my $short_spec = $filespec;
$short_spec =~ s!$GBS_ROOT_PATH/!!;
ENV_say( 1, "  .../$short_spec");
my $temp_name = $short_spec;
$temp_name =~ s!/!_!g;
if (rename( $filespec, "$gbs_tmp_dir/$temp_name"))
{
$nr_moved++;
} else
{
ENV_sig( W => "Failed to rename $short_spec to tmp_dir/$temp_name", "- $!");
}
}
ENV_say( 1, "  $nr_moved/$total .usr files move to TMP directory");
}




sub CLEANUP_obsolete_bld_files()
{

my @files;
ENV_say( 1, "Searching...");

foreach my $target (@GBS_ALL_TARGETS)
{


my @src_types = TARGET_get_src_types( $target);
my %bld_out_types;
my %bld_out_files;
foreach my $src_type (@src_types)
{

$bld_out_types{$src_type} = [ TARGET_get_src_items( $target, $src_type, 'OUT_TYPES'),
TARGET_get_src_items( $target, $src_type, 'OPT_OUT_TYPES'),
'.map' ];
$bld_out_files{$src_type} = [ TARGET_get_src_items( $target, $src_type, 'OUT_FILES') ];
}

foreach my $subsys (GBSGLO_subsystems_for_target( $target))
{

foreach my $comp (GBSGLO_components_for_target( $subsys, $target))
{

my $target_path = "$GBS_ROOT_PATH/dev/$subsys/comp/$comp/bld/$target";
my @bld_files = GBSGLO_glob( "$target_path/*");
my %bld_files = map { ENV_split_spec_f( $_) => 1 } @bld_files;

my %src_types_found;
foreach my $src_file (GBSGLO_glob( "$GBS_ROOT_PATH/dev/$subsys/comp/$comp/src/*"))
{
my ($name, $src_type) = ENV_split_spec_nt( $src_file);
$src_types_found{$src_type} = 1;
foreach my $bld_type (@{$bld_out_types{$src_type}})
{
delete( $bld_files{"$name$bld_type"});
}
}

foreach my $src_type (keys( %src_types_found))
{
foreach my $bld_file (@{$bld_out_files{$src_type}})
{
delete( $bld_files{$bld_file});
}
}

my @files_to_delete = keys( %bld_files);
map { ENV_say( 1, "==> $subsys:$comp:$target:$_") } @files_to_delete;
push @files, map { "$target_path/$_" } @files_to_delete;
}
}
}




delete_files( \@files, 1);
}







sub delete_files($$)
{
my ($filespecs_ref,
$must_print_result,
) = @_;

my ($nr_delete, $total) = (0,0);
foreach my $filespec (@{$filespecs_ref})
{
my $file = ENV_split_spec_f( $filespec);
next if (grep( $file =~ $_, @GBS_SCM_SKIPTYPES));
if (-e $filespec)
{
$total++;
if (unlink( $filespec))
{
$nr_delete++;
} else
{
my $err_msg = $!;
if (!$err_msg)
{
if (!-f $filespec)
{
$err_msg = 'Not a File';
} else
{
$err_msg = 'Unknown reason';
}
}
ENV_sig( E => "Unable to delete file '$filespec'", "- $err_msg");
}
}
}

if ($must_print_result)
{
ENV_say( 1, "  $nr_delete/$total Files deleted");
}
}




sub delete_contents($$$)
{
my ($dirs_ref,
$extra_ignore_re_ref,
$must_print_result,
) = @_;

my @skiptypes_re = @GBS_SCM_SKIPTYPES;
push @skiptypes_re, @{$extra_ignore_re_ref}
if (defined $extra_ignore_re_ref);

my @counts = (0,0,0);	# ($nr_dirs_deleted, $nr_files_deleted, $total_scanned)
foreach my $dir (@{$dirs_ref})
{



if (-d $dir)
{
my @this_counts = FILE_del_tree( $dir, \@skiptypes_re, 0);
map { $counts[$_] += $this_counts[$_] } 0..$#this_counts;
}
}
if ($must_print_result)
{
my ($nr_dirs_deleted, $nr_files_deleted, $total_scanned) = @counts;
my $nr_delete = $nr_dirs_deleted + $nr_files_deleted;
ENV_say( 1, "  $nr_delete/$total_scanned Files and/or Dirs deleted");
}
}

1;

