#===============================================================
#
#   fcreate.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package fcreate;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
FCREATE_get_any
FCREATE_any
FCREATE_gen_type
FCREATE_sca
FCREATE_export
FCREATE_ssprop
FCREATE_att_file
FCREATE_att_ref_file
FCREATE_scope
FCREATE_owners
FCREATE_system
FCREATE_steps
FCREATE_switch
FCREATE_sys
FCREATE_subsys_script
FCREATE_makefile
FCREATE_gbsall
);
}




use glo::env;
use glo::slurp;
use glo::spit;
use glo::file;
use gbsenv;
use gbsglo;
use fspec;




sub FCREATE_get_any($);
sub FCREATE_any($$$);
sub FCREATE_gen_type($$$$$);
sub FCREATE_sca($$$$$);
sub FCREATE_export($$$$);
sub FCREATE_ssprop($$);
sub FCREATE_att_file($$$$);
sub FCREATE_att_ref_file($$$$);
sub FCREATE_scope($$$$$);
sub FCREATE_owners($$);
sub FCREATE_system($$);
sub FCREATE_steps($$$);
sub FCREATE_switch($$$);
sub FCREATE_sys($$$);
sub FCREATE_subsys_script($$$$);
sub FCREATE_makefile($$);
sub FCREATE_gbsall($$);

sub create_gbs_file($$$@);
sub create_non_template_gbs_file($$%);
sub create_empty_file($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_SYSBUILD_PATH = "$GBS_ROOT_PATH/sysbuild";
my $GBS_SCRIPTS_TEMPLATE_PATH = "$GBS_SCRIPTS_PATH/template";
my $GBS_SYSBUILD_TEMPLATES_PATH = "$GBS_SYSBUILD_PATH/templates";
my $GBS_TEMPLATES_PATH = ENV_getenv_perl_path( 'GBS_TEMPLATES_PATH');	# User-defined

my $COMMAND_FILE_EXT = ENV_shell_filetype();




sub FCREATE_get_any($)
{
my ($filetypes_or_ref,    # [ $primary_filetype, $sec_filetype ] secondary = from TARGET.GBS
) = @_;
my @anyfile_specs;

my @any_types = (ref $filetypes_or_ref) ? @{$filetypes_or_ref} : $filetypes_or_ref;





my @template_paths;
push @template_paths, $GBS_TEMPLATES_PATH
if ($GBS_TEMPLATES_PATH ne '');
push @template_paths, ($GBS_SYSBUILD_TEMPLATES_PATH, $GBS_SCRIPTS_TEMPLATE_PATH);


my %files_found;

foreach my $path (@template_paths)
{
my @all_files = SLURP_dir_files( $path, 0);
my @any_files;
foreach my $type (@any_types)
{
my $q_type = quotemeta $type;
push @any_files, grep /^any.*$q_type$/, @all_files;
last if @any_files;	    # no need to search secondary files if at least one primary found
}





foreach my $file (@any_files)
{
if (!exists $files_found{$file})
{
push @anyfile_specs, "$path/$file";
$files_found{$file} = 1;
}
}
}


return (wantarray) ? @anyfile_specs : $anyfile_specs[0];
}




sub FCREATE_any($$$)
{
my ($filespec,
$anyfile_spec_or_comment_ref,	# $anyfile_spec or [ $begin_comment, $end_comment ]
$subsys_component_ref,		# [ $subsys, $component ]
) = @_;



my ($file_name, $filetype) = ENV_split_spec_nt( $filespec);
my $file = "$file_name$filetype";
my ($subsys, $component) = @{$subsys_component_ref};
my $system = ENV_getenv( 'GBS_SYSTEM_NAME');

if (ref $anyfile_spec_or_comment_ref)
{



my ($begin_com, $end_com) = @{$anyfile_spec_or_comment_ref};

my $begin_com_l = length( $begin_com);
my $end_com_l = length( $end_com);
my $max_line_length = 72;
my $border_length = $max_line_length - $begin_com_l - $end_com_l;
my $left_com = $begin_com . ' ' x (4 - $begin_com_l);
my $left_com_l = length( $left_com);
my $file_text = "File: $file";
my $file_text_l = length( $file_text);
my @lines;
push @lines, $begin_com . '=' x $border_length . $end_com;
push @lines, $left_com  . $file_text . ' ' x ($max_line_length - $left_com_l - $file_text_l - $end_com_l) . $end_com;
push @lines, $begin_com . '=' x $border_length . $end_com;
push @lines, '';
push @lines, $begin_com . ' ===EOF=== ' . $end_com;
foreach my $line (@lines)
{
$line =~ s/\s*$/\n/;	# remove trailing white and add nl
}

SPIT_file( $filespec, \@lines);
} elsif ($anyfile_spec_or_comment_ref ne '')
{



my $anyfile_spec = $anyfile_spec_or_comment_ref;

my $include_guard = uc $file;
$include_guard =~ s/\./_/g;

FILE_copy_template( $anyfile_spec, $filespec,
SYSTEM_NAME => $system,
SUBSYS => $subsys,
COMPONENT => $component,
FILE => $file,
FILE_NAME => $file_name,
FILE_TYPE => $filetype,
UC_FILE => uc $file,
UC_FILE_NAME => uc $file_name,
UC_FILE_TYPE => uc $filetype,
INCLUDE_GUARD => $include_guard,
);
} else
{
SPIT_file( $filespec);
}

return $filespec;
}








sub FCREATE_gen_type($$$$$)
{
my ($file,		    # incs_<type>.gbs, incs_<type>.usr, etc. see above
$root_path,
$subsys,
$component,
$target) = @_;
my $filespec;

my $title;
if ($subsys)
{
$title = $subsys.'::';
if ($component)
{
$title .= $component;
} else
{
$title .= 'GEN';
}
} else
{
$title = 'SYSGEN';
}
if ($target)
{
$title .= ":$target";
}

$filespec = FSPEC_gen_type( $file, $root_path, $subsys, $component, $target);
create_gbs_file( $filespec, undef, undef,
TITLE=>$title);

return $filespec;
}






sub FCREATE_sca($$$$$)
{
my ($file,		    # sca_<target>.gbs, sca_ALL.gbs, sca_<target>.usr, etc. see above
$root_path,
$subsys,	    # may be ''
$audit,
$target_or_name,    # may also be <uppercase_name>
) = @_;
my $filespec;

my $title;
if ($subsys)
{
$title = $subsys . '::AUDIT';
} else
{
$title = 'SYSAUDIT';
}
$title .= ":$audit";
$title .= "[$target_or_name]";

$filespec = FSPEC_sca( $file, $root_path, $subsys, $audit, $target_or_name);
create_gbs_file( $filespec, undef, undef,
TITLE=>$title);

return $filespec;
}




sub FCREATE_ssprop($$)
{
my ($root_path,
$subsys,
) = @_;
my $filespec;

$filespec = FSPEC_ssprop( $root_path, $subsys);
create_empty_file( $filespec);

return $filespec;
}




sub FCREATE_export($$$$)
{
my ($file,	    	#export.gbs or export.usr
$root_path,
$subsys,
$component,	# can be '' in case of build/export.gbs or undef for non-gbs subsys
) = @_;
my $filespec;

my $title = $subsys;
$title .= '::' . $component if ($component);
$filespec = FSPEC_export( $file, $root_path, $subsys, $component);
create_gbs_file( $filespec, undef, undef,
TITLE=>$title);

return $filespec;
}




sub FCREATE_att_file($$$$)
{
my ($file,		# e.g. target.gbs, audit.usr, tool.gbs, etc
$root_path,
$item_name,
$plugin_name,
) = @_;
my $filespec;

(my $dir, $file) = FSPEC_att_file( $file, $root_path, $item_name);
$filespec = "$dir/$file";

create_gbs_file( $filespec, undef, undef,
ATT_NAME	=> $item_name,
PLUGIN	=> $plugin_name,
);

return $filespec;
}




sub FCREATE_att_ref_file($$$$)
{
my ($file,		# e.g. target.gbs, audit.usr, tool.gbs, etc
$root_path,
$item_name,
$plugin_name,
) = @_;
my $filespec;

(my $dir, $file) = FSPEC_att_file( $file, $root_path, $item_name);
$filespec = "$dir/$file";

my $file_name = ENV_split_spec_n( $file);
my $filename_gbs = "$file_name.gbs";

my $template_file = "${file_name}_ref.gbs";
create_gbs_file( $filespec, $template_file, undef,
ATT_NAME	=> $item_name,
PLUGIN	=> $plugin_name,
INCLUDE	=> $filename_gbs,
);

return $filespec;
}




sub FCREATE_scope($$$$$)
{
my ($file,		# scope.gbs or scope.usr
$root_path,
$subsys,
$component,
$lines_ref) = @_;
my $filespec;

$filespec = FSPEC_scope( $file, $root_path, $subsys, $component);
create_gbs_file( $filespec, undef, $lines_ref,
TITLE=>"$subsys\:\:$component");

return $filespec;
}




sub FCREATE_owners($$)
{
my ($file,		    #  with or without .gbs or .usr
$root_path,
) = @_;
my $filespec;

$filespec = FSPEC_owners( $file, $root_path);
create_empty_file( $filespec);

return $filespec;
}




sub FCREATE_system($$)
{
my ($file,		    #  with or without .gbs or .usr
$root_path,
) = @_;
my $filespec;

$filespec = FSPEC_system( $file, $root_path);
create_empty_file( $filespec);

return $filespec;
}




sub FCREATE_steps($$$)
{
my ($file,		# steps.gbs or steps.usr
$root_path,
$system_name) = @_;
my $filespec;

$filespec = FSPEC_steps( $file, $root_path);

my @lines;
foreach my $subsys (GBSGLO_subsystems())
{
push @lines,
"STEP $subsys",
"     EXEC       = SELECT",
'';
}
create_gbs_file( $filespec, undef, \@lines,
SYSTEM_NAME=>$system_name);

return $filespec;
}




sub FCREATE_switch($$$)
{
my ($file,			# switch.gbs or switch.gbs.bat or switch.gbs.sh or

$root_path,
$system_name) = @_;
my $filespec;

my $site = ENV_getenv( 'GBS_SITE');
$site = 'EIN' if ($site eq '');

$filespec = FSPEC_switch( $file, $root_path);
create_gbs_file( $filespec, undef, undef,
SYSTEM_NAME=>$system_name, SITE=>$site);

return $filespec;
}




sub FCREATE_sys($$$)
{
my ($file,		# sys.gbs or sys.usr
$root_path,
$system_name) = @_;
my $filespec;

$filespec = FSPEC_sys( $file, $root_path);
create_gbs_file( $filespec, undef, undef,
SYSTEM_NAME=>$system_name);

return $filespec;
}




sub FCREATE_subsys_script($$$$)
{
my ($file_name,	# gbssub with or without .sh or .bat
$root_path,
$subsys,
$ss_type,	# MAKE, MSVS or OTHER
) = @_;
my $filespec;

$filespec = FSPEC_subsys_script( $file_name, $root_path, $subsys);
my $filetype = ENV_split_spec_t( $filespec);
create_gbs_file( $filespec, lc "gbssub_$ss_type$filetype", undef,
SUBSYS => $subsys);

return $filespec;
}




sub FCREATE_makefile($$)
{
my ($root_path,
$subsys,
) = @_;
my $filespec;

$filespec = FSPEC_makefile( $root_path, $subsys);
my $template_spec = "$GBS_SCRIPTS_TEMPLATE_PATH/Makefile.mk";
ENV_whisper( 1, "Creating 'Makefile.mk' file from template 'Makefile.ml'");
FILE_copy_template( $template_spec, $filespec,
SUBSYS => $subsys);

return $filespec;
}




sub FCREATE_gbsall($$)
{
my ($filetype,	    # full_name+type or .sh or .bat or ''
$in_install,	    # In GBS_SCRIPTS_ROOT else in GBS_BASE_PATH
) = @_;
my $filespec;

$filespec = FSPEC_gbsall( $filetype, $in_install);
my $location = ($in_install) ? 'Install' : 'User';
create_gbs_file( $filespec, undef, undef,
LOCATION => $location);

return $filespec;
}




sub create_gbs_file($$$@)
{
my ($filespec,	    # path/name.gbs or .usr
$template_file,	    # may be undef (same as filespec)
$lines_ref,	    # may be undef
@translate) = @_;


my $file = ENV_split_spec_f( $filespec);
if (!-e $filespec || -z $filespec)
{
my $filename_gbs ;
if (defined $template_file)
{
$filename_gbs = $template_file;
} else
{
$filename_gbs = $file;
$filename_gbs =~ s/usr$/gbs/;
}
my $template_spec = "$GBS_SCRIPTS_TEMPLATE_PATH/$filename_gbs";
if (-f $template_spec)
{
ENV_whisper( 1, "Creating $file file from template $filename_gbs");
FILE_copy_template( $template_spec, $filespec, @translate,
FILE=>$file, _LINES_=>$lines_ref);
} else
{
create_non_template_gbs_file( $filespec, $lines_ref, @translate);
}
chmod 0755, $filespec
if (substr( $filespec, -3, 3) eq '.sh');	# set file to 'x'
} else
{
ENV_say( 1, "$file file already exists - not created");
}
}




sub create_non_template_gbs_file($$%)
{
my ($filespec,
$lines_ref,
%translate) = @_;

my ($dir, $file) = ENV_split_spec_df( $filespec);
if (!-d $dir)
{
ENV_whisper( 1, "Creating $dir...");
ENV_mkdir( $dir);
}
my $nr_lines = (defined $lines_ref) ? @{$lines_ref} : 0;

if ($nr_lines > 0)
{
ENV_whisper( 1, "Creating $file file ($nr_lines)");
} else
{
ENV_whisper( 1, "Creating empty $file file");
}
my @lines;
push @lines, "#========================================================";
push @lines, "#   [$translate{TITLE}] $file";
push @lines, "#========================================================";
push @lines, @{$lines_ref}
if ($nr_lines > 0);
push @lines, '';
push @lines, "###EOF###";

SPIT_file_nl( $filespec, @lines);
}




sub create_empty_file($)
{
my ($filespec) = @_;

my ($dir, $file) = ENV_split_spec_df( $filespec);
if (!-f $filespec)
{
ENV_whisper( 1, "Creating empty $file file");
SPIT_file( $filespec);
chmod 0755, $filespec
if (substr( $filespec, -3, 3) eq '.sh');	# set file to 'x'
} else
{
ENV_say( 1, "$file file already exists - not created");
}
}

1;

