#===============================================================
#
#   gbsbgglo.pm
#
#===============================================================
#   This src_files is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsbgglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSBGGLO_init
GBSBGGLO_main
GBSBGGLO_set
GBSBGGLO_exec
GBSBGGLO_handle_rc
);
}




use glo::env;
use glo::time;
use glo::slurp;
use gbsenv;
use gbsfilecom;
use fail;




sub GBSBGGLO_init($$);
sub GBSBGGLO_main($$);
sub GBSBGGLO_set($$);
sub GBSBGGLO_exec($$$@);
sub GBSBGGLO_handle_rc($$$$);

sub do_banner(@);
sub do_echo(@);
sub do_glkb($$$$@);
sub do_sca($$$@);
sub do_scasum($$);
sub banner($$@);
sub print_fail_msg($);




my $GBS_IGNORE_ERRORS = $ENV{GBS_IGNORE_ERRORS};

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = $ENV{GBS_SUBSYS};
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');




my $BATCH_NAME = '';	# To be used in logging. E.g.: gbsgen, gbsgen_post, gbsaudit, gbsaudit_post




my $FAIL_COUNT = 0;

my $CWD;	    # filled from $GBSBGJOB_SCRIPT_SPEC src_files




my %FUNCTIONS = (

BANNER  => [ \&do_banner,	    1, ],	# $first_line, @lines
ECHO    => [ \&do_echo,	    1, ],	# @lines
SET     => [ \&GBSBGGLO_set,    2, ],	# $name, $value
EXEC    => [ \&GBSBGGLO_exec,   7, ],	# $src_files, $must_print_commands, $out_files, @command_data_lines
GLKB    => [ \&do_glkb,	    8, ],	# $src_files, $must_print_commands, $glkb_args, $out_files, @command_data_lines
SCA	    => [ \&do_sca,	    3, ],	# $src_files, $unix_style_search, $out_files, @command_data_lines
SCASUM  => [ \&do_scasum,       2, ],	# $src_files, $sum_filespec
);




sub GBSBGGLO_init($$)
{
my ($batch_name,
$component,	    # may be '-'
) = @_;




$BATCH_NAME = uc $batch_name;
{
my ($os, $host) = ENV_uname();
ENV_whisper( 1, "$BATCH_NAME: Running on $host ($os)");
}




if ($component eq '' || $component eq '-')
{
$GBS_COMPONENT = '';

$ENV{GBS_COMPONENT} = '';
$ENV{GBS_COMPONENT_PATH} = '';

$CWD = ENV_chdir( "$GBS_ROOT_PATH/dev/$GBS_SUBSYS");
} else
{
my $new_component_path = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp/$component";
$GBS_COMPONENT = $component;
banner( '*', "Component: $GBS_COMPONENT");
$ENV{GBS_COMPONENT} = $GBS_COMPONENT;
$ENV{GBS_COMPONENT_PATH} = $new_component_path;

$CWD = ENV_chdir( "$new_component_path/src");
}
}




sub GBSBGGLO_main($$)
{
my ($gbsbg_script_spec,
$func_ref,		# [ $key, $function ]
) = @_;
my $rc = 0;




{
my ($key, $function) = @{$func_ref};
ENV_sig( F => "Invalid function-key '$key'")
if (!exists $FUNCTIONS{$key});
$FUNCTIONS{$key}->[0] = $function;
}




my @lines = SLURP_file( $gbsbg_script_spec);
while (@lines)
{
my $command = shift @lines;

my ($function, $min_nr_lines) = @{$FUNCTIONS{$command}};
my $nr_lines = shift @lines;
ENV_sig( F => "$command: nr_lines ($nr_lines) < min_nr_lines ($min_nr_lines)")
if ($nr_lines < $min_nr_lines);




my @sub_lines;
while ($nr_lines > 0)
{
my $sub_line = shift @lines;
push @sub_lines, $sub_line;
$nr_lines--;
}




my $this_rc = &$function( @sub_lines);
$rc = $this_rc if ($this_rc > $rc);
if ($this_rc != 0)
{
print_fail_msg( scalar @lines);
last if (!$GBS_IGNORE_ERRORS);
}
}

return $rc;
}




END
{
my $rc = $?;

my $rc_txt;
if ($rc == 0)
{
$rc_txt = 'Completed';
} else
{
$rc_txt = "Unsuccessful ($rc)";
$rc_txt .= " (Count: $FAIL_COUNT)"
if ($FAIL_COUNT > 0);
}
ENV_say( 1, "$BATCH_NAME: $rc_txt");
}




sub do_banner(@)
{
my ($first_line,
@lines
) = @_;

banner( '*', $first_line, @lines);

return 0;
}




sub do_echo(@)
{
my (@lines) = @_;

ENV_say( 1, @lines);

return 0;
}




sub GBSBGGLO_set($$)
{
my ($name,
$value,
) = @_;
$value = '' if (!defined $value);


$value = ' '
if ($value eq '');
$ENV{$name} = $value;

return 0;
}




sub GBSBGGLO_exec($$$@)
{
my ($src_files_or_ref,
$must_print_commands,	# scripts should print their own command(s). '-' == undef
$out_files,		# List
@command_data_lines	# ($command_type, $command_line, $ok_values, $exec_condition), ...

) = @_;
my $rc = 0;

my $src_files_ref = (ref $src_files_or_ref) ? $src_files_or_ref : [ $src_files_or_ref ];




my $nr_command_data_lines = @command_data_lines;
my @command_data_refs;
for (my $i = 0; $i < $nr_command_data_lines; $i += 4)
{
my ($command_type, $command_line, $ok_values, $exec_condition) = @command_data_lines[$i..$i + 3];
push @command_data_refs, [ $command_type, $command_line, $ok_values, $exec_condition ];
}




$rc = GBSFILECOM_exec( $must_print_commands, \@command_data_refs);
if ($rc == 0)
{
ENV_whisper( 0, '-');
} else
{
GBSBGGLO_handle_rc( $rc, $src_files_ref, 'exec', $out_files);
}

return $rc;
}




sub GBSBGGLO_handle_rc($$$$)
{
my ($rc,
$src_files_ref,
$sub_command,
$out_files,
) = @_;


if ($rc > 0)
{
ENV_say( 1, "$BATCH_NAME: ### Failed ($rc) ***");
FAIL_log( $sub_command, $rc, $GBS_SUBSYS, $GBS_COMPONENT, "@{$src_files_ref}");
$FAIL_COUNT++;




if ($out_files ne '')
{
foreach my $out_file (split( ' ', $out_files))
{

if (-f $out_file)
{
ENV_whisper( 1, "  Deleting '$out_file'...");
unlink $out_file;
}
}
}
}
ENV_whisper( 0, '-');

return $rc
}




sub do_glkb($$$$@)
{
my ($src_file,
$must_print_commands,
$glkb_args,	    # "$gen_type $glkb_type $glkb_file $out_filespec"
$out_files,	    # List
@command_line,
) = @_;


ENV_sig( F => 'GLKB_func not defined');


}





sub do_sca($$$@)
{
my ($src_files,
$unix_style_search,	# bool
$out_files,		# List
@command_data_lines
) = @_;


ENV_sig( F => 'SCA_func not defined');


}





sub do_scasum($$)
{
my ($src_files,
$sum_filespec,
) = @_;


ENV_sig( F => 'SCASUM_func not defined');


}




sub banner($$@)
{
my ($pre_char,
$first_line,
@other_lines
) = @_;

my $pre = $pre_char x 3;
my $time_stamp = TIME_time2num();
my $time_stamp_indent = ' ' x length( $time_stamp);
ENV_say( 1, $pre);
ENV_say( 1, "$pre $time_stamp - $first_line");
map { ENV_say( 1, "$pre $time_stamp_indent - $_") } @other_lines;
ENV_say( 1, $pre);
}




sub print_fail_msg($)
{
my ($nr_remaining_lines) = @_;

my ($title, $sub_line);

$title = $BATCH_NAME;

if ($nr_remaining_lines > 0)
{
if ($GBS_IGNORE_ERRORS)
{
$sub_line = "Continuing with remaining commands";
} else
{
$sub_line = "Remaining commands will be skipped";
}
} else
{
$sub_line = "No Remaining commands";
}
banner( '#', "$title: ### FAILED ###",
$sub_line);
}

1;


