#===============================================================
#
#   gbsbgjob.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsbgjob;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSBGJOB_banner
GBSBGJOB_echo
GBSBGJOB_set
GBSBGJOB_exec_short
GBSBGJOB_exec
GBSBGJOB_glkb
GBSBGJOB_sca
GBSBGJOB_scasum
GBSBGJOB_batch_init
GBSBGJOB_batch_run
GBSBGJOB_batch_queue
GBSBGJOB_batch_wait
GBSBGJOB_batch_finish
);
}




use glo::env;
use glo::spit;
use gbsenv;
use exec;
use fail;




sub GBSBGJOB_banner(@);
sub GBSBGJOB_echo(@);
sub GBSBGJOB_set($$);
sub GBSBGJOB_exec_short($$$$);
sub GBSBGJOB_exec($$$@);
sub GBSBGJOB_glkb($$$$@);
sub GBSBGJOB_sca($$$@);
sub GBSBGJOB_scasum($$);
sub GBSBGJOB_batch_init($);
sub GBSBGJOB_batch_run($$$);
sub GBSBGJOB_batch_queue($$$);
sub GBSBGJOB_batch_wait();
sub GBSBGJOB_batch_finish();

sub get_command_data_lines($);
sub set_batch_type($);
sub create_gbsbg_lines($@);








my @COMMAND_FILE_TYPES = (ENV_is_win32()) ? qw(.bat) : qw(.sh);

my %BATCH_PROGS = (

exec    => 'gbsbgexec.pl',
sca	    => 'gbsbgsca.pl',
scasum  => 'gbsbgscasum.pl',
);




my $BATCH_TYPE;	    # exec, sca, scasum

my @QUEUED_FILES;




sub GBSBGJOB_banner(@)
{
my @lines = @_;

return create_gbsbg_lines( BANNER => @lines);
}




sub GBSBGJOB_echo(@)
{
my (@lines) = @_;

return create_gbsbg_lines( ECHO => @lines);
}




sub GBSBGJOB_set($$)
{
my ($name,
$value,
) = @_;


return create_gbsbg_lines( SET => $name, $value);
}




sub GBSBGJOB_exec_short($$$$)
{
my ($file,			# for failure logging purposes
$must_print_commands,	# undef = ok
$out_files_ref,		# undef = ok
$command_ref,
) = @_;

$must_print_commands = '-'
if (!defined $must_print_commands);
my $command_type = 'S';
my @command_data_lines = get_command_data_lines( [ [ $command_type, ENV_command_line( $command_ref), 0, '-' ] ]);




set_batch_type( 'exec');

return create_gbsbg_lines( EXEC => $file, $must_print_commands, $out_files_ref, @command_data_lines );
}




sub GBSBGJOB_exec($$$@)
{
my ($file,			# for failure logging purposes
$must_print_commands,	# undef = ok
$out_files_ref,		# undef = ok
@command_data_refs,     # [ $command_type, $command_line, $ok_values, $exec_condition ]

) = @_;

$must_print_commands = '-'
if (!defined $must_print_commands);
my @command_data_lines = get_command_data_lines( \@command_data_refs);


set_batch_type( 'exec');

return create_gbsbg_lines( EXEC => $file, $must_print_commands, $out_files_ref, @command_data_lines );
}




sub GBSBGJOB_glkb($$$$@)
{
my ($file,			# for failure logging purposes
$must_print_commands,	# undef = OK
$glkb_args_ref,		# [ $gen_type, $glkb_type, $glkb_file, $out_filespec ]
$out_files_ref,		# undef = OK
@command_data_refs,     # [ $command_type, $command_line, $ok_values, $exec_condition ]

) = @_;

$must_print_commands = '-'
if (!defined $must_print_commands);
my @command_data_lines = get_command_data_lines( \@command_data_refs);


set_batch_type( 'exec');

return create_gbsbg_lines( GLKB => $file, $must_print_commands, $glkb_args_ref, $out_files_ref, @command_data_lines);
}




sub GBSBGJOB_sca($$$@)
{
my ($src_files,		# List
$unix_style_search,	# bool
$out_files_ref,		# undef = ok
@command_data_refs,     # [ $command_type, $command_line, $ok_values, $exec_condition ]

) = @_;

my @command_data_lines = get_command_data_lines( \@command_data_refs);


set_batch_type( 'sca');

return create_gbsbg_lines( SCA => $src_files, $unix_style_search, $out_files_ref, @command_data_lines );
}




sub GBSBGJOB_scasum($$)
{
my ($file,			# for failure logging purposes
$sum_filespec,
) = @_;

set_batch_type( 'scasum');

return create_gbsbg_lines( SCASUM => $file, $sum_filespec);
}




sub GBSBGJOB_batch_init($)
{
my ($max_parallel_jobs) = @_;









$BATCH_TYPE = '';

EXEC_batch_init( $max_parallel_jobs);
}




sub GBSBGJOB_batch_run($$$)
{
my ($batch_name,	# gbsgen_post, gbsaudit_post
$component,	# '-' == none
$gbsbg_lines_ref,
) = @_;
my $rc;

ENV_sig( F => "BATCH_type was not initialised", @{$gbsbg_lines_ref})
if ($BATCH_TYPE eq '');


my $gbsbg_file = SPIT_tmp_file_nl( "gbsbg_batch_$batch_name", $gbsbg_lines_ref);

my $batch_prog = $BATCH_PROGS{$BATCH_TYPE};	    # e.g.: 'gbsbgexec.pl'
$rc = EXEC_batch_run( ENV_command_ref( $batch_prog, [ $batch_name, $component, $gbsbg_file ]));
$BATCH_TYPE = '';

unlink $gbsbg_file;

return $rc;
}




sub GBSBGJOB_batch_queue($$$)
{
my ($batch_name,	# gbsgen gbs_audit
$component,	# '-' == none
$gbsbg_lines_ref,
) = @_;
my $queued;		    # 1 = queued, 0 == not queued because of failed steps

ENV_sig( F => "BATCH_type was not initialised", @{$gbsbg_lines_ref})
if ($BATCH_TYPE eq '');

my $gbsbg_file = SPIT_tmp_file_nl( "gbsbg_batch_$batch_name", $gbsbg_lines_ref);
push @QUEUED_FILES, $gbsbg_file;

my $batch_prog = $BATCH_PROGS{$BATCH_TYPE};	    # e.g.: 'gbsbgexec.pl'
$queued = EXEC_batch_queue( ENV_command_ref( $batch_prog, [ $batch_name, $component, $gbsbg_file ]));
$BATCH_TYPE = '';

return $queued;
}




sub GBSBGJOB_batch_wait()
{
my $rc;

$rc = EXEC_batch_wait();

unlink @QUEUED_FILES;
@QUEUED_FILES = ();

return $rc;
}




sub GBSBGJOB_batch_finish()
{
if (GBSENV_get_exec_level() == 0)	# may have be called by gbsmake->gbsgen!
{



my @lines = FAIL_read();
if (@lines)
{
ENV_say( 1, "Failed:");
map { print( "###  $_") } @lines;	# We canot use ENV_say because the lines are already formatted
unlink FAIL_filespec()
if (!GBSENV_mode_is_background());
}
}


set_batch_type( undef);
}




sub get_command_data_lines($)
{
my ($command_data_refs_ref) = @_;
my @command_data_lines;

foreach my $ref (@{$command_data_refs_ref})
{
push @command_data_lines, @{$ref};





}

return @command_data_lines;
}




sub set_batch_type($)
{
my ($type) = @_;	 # exec, sca, scasum or undef

ENV_sig( F => "GBSBGJOB_batch_init was not called")
if (!defined $BATCH_TYPE);

if ($BATCH_TYPE eq '')
{
$BATCH_TYPE = $type;
} elsif ($BATCH_TYPE eq $type)
{

} else
{
ENV_sig( F => "BATCH_TYPE mismatch ($BATCH_TYPE <- $type)");

}
}




sub create_gbsbg_lines($@)
{
my ($name,
@line_items
) = @_;
my @lines;

my $nr_line_items = scalar @line_items;
if ($nr_line_items > 0)
{
push @lines, ( $name, $nr_line_items );
foreach my $line_item (@line_items)
{
if (defined $line_item)
{
if (ref $line_item)
{
$line_item = "@{$line_item}";   # ref
} else
{

}
} else
{
$line_item = '';		    # undef
}

push @lines, $line_item;
}
}

return @lines;
}

1;


