#===============================================================
#
#   gbscmd.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbscmd;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSCMD_get_gbs_command
GBSCMD_get_gbs_command_ref
GBSCMD_get_base_command
GBSCMD_get_batch_command
GBSCMD_get_submit_command
GBSCMD_get_subwin_command
GBSCMD_get_os_command
GBSCMD_get_view_file_command
GBSCMD_get_browse_file_command
GBSCMD_get_edit_file_command
GBSCMD_set_xtitle
GBSCMD_bg_notifier
GBSCMD_scripts_path_changed
GBSCMD_get_level
);
}




use glo::env;
use glo::shell;
use gbsenv;




sub GBSCMD_get_gbs_command($@);
sub GBSCMD_get_gbs_command_ref($@);
sub GBSCMD_get_base_command($@);
sub GBSCMD_get_batch_command($);
sub GBSCMD_get_submit_command($$$);
sub GBSCMD_get_subwin_command($$);
sub GBSCMD_get_os_command($);
sub GBSCMD_get_view_file_command($);
sub GBSCMD_get_browse_file_command($);
sub GBSCMD_get_edit_file_command($$);
sub GBSCMD_set_xtitle($);
sub GBSCMD_bg_notifier();
sub GBSCMD_scripts_path_changed();
sub GBSCMD_get_level($);

sub get_env_command_ref($$@);
sub get_target_env($);
sub substitute_args($$$$);




my $IS_WIN32 = ENV_is_win32();
my $IS_LUNIX = !$IS_WIN32;

my %ENV_COMMAND_REFS;	    # Do not inilialize!


my $GBS_XTITLE; 	    # Do not initialize!

my %TARGET_ENVS;





my $DEFAULT_WIN32_EDITOR  = ENV_getenv( 'WINDIR') . '\notepad.exe %1%';
my $DEFAULT_LINUX_EDITOR  = (ENV_is_linux()) ? 'gedit $1' : 'vi $1';
my $DEFAULT_LUNIX_BROWSER = 'firefox $1';
my %GEN_DEFS = (

GBS_VIEWER	=> [ undef,    $DEFAULT_WIN32_EDITOR, 'less $1' ],
GBS_BROWSER	=> [ undef,    'start %1%',           $DEFAULT_LUNIX_BROWSER ],
GBS_EDITOR	=> [ 'EDITOR', $DEFAULT_WIN32_EDITOR, $DEFAULT_LINUX_EDITOR ],
);
my %COMMANDS_REQ_SHELL = (
less    => 1,
more    => 1,
);

my $OS_GBS_PERL_CMD = ENV_getenv( 'GBS_PERL_CMD');
my $OS_GBS_WPERL_CMD = ENV_getenv( 'GBS_WPERL_CMD');
my $OS_GBS_SCRIPTS_PATH = ENV_getenv( 'GBS_SCRIPTS_PATH');
my $SCRIPT_FILETYPE = ENV_shell_filetype();
my %GBS_COMMAND_REFS = (



gbs		    => [ 's', 0, 1 , 4 ],
gbsaudit	    => [ 's', 0, 1 , 3 ],
gbsbg	    => [ 'p', 0, 0 , 1 ],
gbsbldcheck	    => [ 'p', 0, 0 , 1 ],
gbsedit	    => [ 'p', 0, 0 , 4 ],
gbsexit	    => [ 's', 0, 1 , 0 ],
gbsexport	    => [ 'p', 0, 0 , 4 ],
gbsgen	    => [ 'p', 0, 0 , 3 ],
gbshelp	    => [ 'p', 0, 0 , 0 ],
gbslocate	    => [ 's', 0, 1 , 4 ],
gbsmaint	    => [ 's', 0, 1 , 0 ],
gbsmake	    => [ 'p', 0, 0 , 3 ],
gbsmakemake	    => [ 'p', 0, 0 , 2 ],
gbsman	    => [ 'd', 0, 0 , 0, [ more => "$OS_GBS_SCRIPTS_PATH/gbsman.txt" ] ],
gbsnew	    => [ 'p', 0, 1 , 4 ],			# special case!
gbsscm	    => [ 'p', 0, 0 , 4 ],
gbssetup	    => [ 's', 0, 1 , 0 ],
gbsshow	    => [ 'p', 0, 0 , 4 ],
gbssilo	    => [ 'p', 0, 0 , 1 ],
gbsstart	    => [ 's', 0, 1 , 4 ],
gbsstart_here   => [ 's', 0, 1 , 4 ],
gbsstats	    => [ 'p', 0, 0 , 1 ],
gbsswc	    => [ 's', 0, 1 , 4 ],
gbsswr	    => [ 's', 0, 1 , 4 ],
gbssws	    => [ 's', 0, 1 , 4 ],
gbsswt	    => [ 's', 0, 1 , 4 ],
gbsswa	    => [ 's', 0, 1 , 4 ],
gbssysall	    => [ 'p', 0, 0 , 2 ],
gbssysaudit	    => [ 'p', 0, 0 , 2 ],
gbssysgen	    => [ 'p', 0, 0 , 2 ],
gbssysmake	    => [ 'p', 0, 0 , 2 ],
gbssystool	    => [ 'p', 0, 0 , 2 ],
gbsuninstall    => [ 'p', 0, 1 , 0 ],
gbswhich	    => [ 'p', 0, 0 , 1 ],
gbsxref	    => [ 'p', 1, 0 , 0 ],
gbsxref_tkx	    => [ 'p', 1, 0 , 0 ],
notify_tkx	    => [ 'p', 1, 0 , 0 ],
);

my $OS_GBS_BASE_PATH = ENV_getenv( 'GBS_BASE_PATH');
my %GBS_BASE_COMMANDS = (

define	    => 1,
gbsinit	    => 1,
gbssetup	    => 1,
gbsuninstall    => 1,
gbsstart	    => 1,
gbsstart_here   => 1,
);

my %OS_COMMANDS = (

echo	=> 1,
make	=> 0,
more	=> 1,
tail	=> 0,
);




sub GBSCMD_get_gbs_command($@)
{
my ($name,
@args,
) = @_;
my $command_line;

my $command_ref = GBSCMD_get_gbs_command_ref( $name, @args);
$command_line = ENV_command_line( $command_ref);


return $command_line;
}




sub GBSCMD_get_gbs_command_ref($@)
{
my ($name,
@args,
) = @_;
my $command_ref;

my $ref = $GBS_COMMAND_REFS{$name};

ENV_sig( F => "No such GBS command '$name'")
if (! defined $ref);

my ($command_type, $needs_gui, $must_be_sourced) = @{$ref};


my ($type,
$prefix,    # (perl | call | start | '')
$os_exec,
);
my @new_args;
if ($command_type eq 'p')		# perl
{
$type = '.pl';
$prefix = 'perl';
$os_exec = "$OS_GBS_SCRIPTS_PATH/$name.pl";
} elsif ($command_type eq 's')	# script
{
$type = $SCRIPT_FILETYPE;
$prefix = ($IS_LUNIX && $must_be_sourced) ? 'source' : '';
$os_exec = "$OS_GBS_SCRIPTS_PATH/$name$SCRIPT_FILETYPE";
} elsif ($command_type eq 'd')	# defined
{
my ($command, @these_args) = @{$ref->[4]};	# $special_command_ref
$type = '';
$os_exec = $command;
push @new_args, @these_args;
} else
{
ENV_sig( F => "Impossible command_type '$command_type'");
}
push @new_args, @args;

if ($needs_gui)
{
if (!GBSENV_gui_available())
{
ENV_sig( E => "*** Cannot startup $name ***",
"- Perl Tkx Package is not available");
}
}
$os_exec =~ tr!\\!/!;
$command_ref = ENV_new_command_ref( $type, $prefix, $os_exec, [ @new_args ]);


return $command_ref;
}




sub GBSCMD_get_base_command($@)
{
my ($name,
@args,
) = @_;
my $command;

ENV_sig( F => "no such GBS command '$name'")
if (! exists $GBS_BASE_COMMANDS{$name});

$command = "$OS_GBS_BASE_PATH/$name$SCRIPT_FILETYPE";
$command = "source $command"
if ($IS_LUNIX);

$command =~ tr!\\!/!;
$command = "$command @args"
if (@args);

return $command;
}








sub GBSCMD_get_batch_command($)
{
my ($command_ref,		    # ADT
) = @_;
my $full_command;

$full_command = ENV_command_line( $command_ref);


my $gbs_batch = get_target_env( 'GBS_BATCH');
if ($gbs_batch ne '')
{
$full_command = substitute_args( $gbs_batch, '1', $full_command, 1);
}

return $full_command;
}








sub GBSCMD_get_submit_command($$$)
{
my ($command,
$command_name,
$os_log_file) = @_;
my $submit_cmd;

my $gbs_submit = get_target_env( 'GBS_SUBMIT');
if ($gbs_submit eq '')
{
$submit_cmd = $command;
} else
{
$submit_cmd = substitute_args( $gbs_submit, '1', $command, 1);
$submit_cmd = substitute_args( $submit_cmd, '2', $command_name, 0);
$submit_cmd = substitute_args( $submit_cmd, '3', $os_log_file, 0);
}



return $submit_cmd;
}








sub GBSCMD_get_subwin_command($$)
{
my ($command,
$command_name) = @_;
my $subwin_cmd;

my $gbs_subwin = get_target_env( 'GBS_SUBWIN');
if ($gbs_subwin eq '')
{
$subwin_cmd = $command;
} else
{
$subwin_cmd = substitute_args( $gbs_subwin, '1', $command, 1);
$subwin_cmd = substitute_args( $subwin_cmd, '2', $command_name, 0);
}



return $subwin_cmd;
}




sub GBSCMD_get_os_command($)
{
my ($unix_builtin_command) = @_;	# e.g.: make more echo tail
my $os_command;

$os_command = get_target_env( 'GBS_' . uc $unix_builtin_command);
if ($os_command eq '')
{
my $is_same = $OS_COMMANDS{$unix_builtin_command};
if (defined $is_same)
{
if ($IS_WIN32)
{
$os_command = ($is_same) ? $unix_builtin_command : "$ENV{GBS_SCRIPTS_PATH}/win32utils/$unix_builtin_command.exe";
} else
{
$os_command = $unix_builtin_command;
}
} else
{
ENV_sig( F => "Unknown os command '$unix_builtin_command'");
}
}

$os_command =~ s!/!\\!g
if ($IS_WIN32);

return $os_command;
}






sub GBSCMD_get_view_file_command($)
{
my ($filespec,
) = @_;


$filespec = ENV_enquote( $filespec);

return get_env_command_ref( GBS_VIEWER => $filespec);
}





sub GBSCMD_get_browse_file_command($)
{
my ($filespec,
) = @_;


$filespec = ENV_enquote( $filespec);

return get_env_command_ref( GBS_BROWSER => $filespec);
}





sub GBSCMD_get_edit_file_command($$)
{
my ($filespec,
$editor_opt,
) = @_;


$filespec = ENV_enquote( $filespec);

my @opts = (defined $editor_opt && $editor_opt ne '') ? ($editor_opt) : ();

return get_env_command_ref( GBS_EDITOR => $filespec, @opts);
}




sub get_env_command_ref($$@)
{
my ($env_name,  # GBS_VIEWER, GBS_BROWSER, GBS_EDITOR
$filespec,
@options,
) = @_;
my $command_ref;




my $env_command_ref = $ENV_COMMAND_REFS{$env_name};
my ($os_command, $args_ref);
if (defined $env_command_ref)
{
($os_command, $args_ref) = @{$env_command_ref};
} else
{
my $command_line = ENV_getenv( $env_name);
if ($command_line eq '')
{



my ($sec_env_name, $win_default, $unix_default) = @{$GEN_DEFS{$env_name}};

$command_line = ENV_getenv( $sec_env_name)
if (defined $sec_env_name);
$command_line = ($IS_WIN32) ? $win_default : $unix_default
if ($command_line eq '');
}

($os_command, $args_ref) = ENV_command_split( $command_line);
$os_command =~ tr!/!\\!
if ($IS_WIN32);
}




$filespec =~ s!/!\\!g
if ($IS_WIN32);
my @args = substitute_args( $args_ref, '1', $filespec, 1);
push @args, @options;


$command_ref = ENV_command_ref( $os_command, \@args);
return $command_ref;
}





sub GBSCMD_set_xtitle($)
{
my ($title) = @_;
my $command;

$title =~ s!/!\\!g
if ($IS_WIN32);

$GBS_XTITLE = ENV_getenv( 'GBS_XTITLE') if (!defined $GBS_XTITLE);
if ($GBS_XTITLE eq '')
{
$command = SHELL_xtitle( $title);
} else
{
$command = substitute_args( $GBS_XTITLE, '1', $title, 1);
}


return $command;
}





sub GBSCMD_bg_notifier()
{
my $gbs_bg_notifier = get_target_env( 'GBS_BG_NOTIFIER');

return $gbs_bg_notifier;
}




sub get_target_env($)
{
my ($base_env) = @_;
my $env_value;

my $env_name;
my $gbs_target = uc ENV_getenv( 'GBS_TARGET');
my $gbs_audit = uc ENV_getenv( 'GBS_AUDIT');

my $key = join( '_', $base_env, $gbs_target, $gbs_audit);
my $ref = $TARGET_ENVS{$key};
if (defined $ref)
{
($env_name, $env_value) = @{$ref};

} else
{
my @envs;
if ($gbs_audit ne '')
{
push @envs, "${base_env}_${gbs_target}_${gbs_audit}";
push @envs, "${base_env}_${gbs_audit}";
} else
{
push @envs, "${base_env}_${gbs_target}";
}
push @envs, "${base_env}";

$env_name = '';
foreach my $env (@envs)
{

$env_value = ENV_getenv( $env);
if ($env_value ne '')
{
$env_name = $env;
last;
}
}
if ($env_name ne '')
{
ENV_say( 1, "$env_name EnvVar defined");
$TARGET_ENVS{$key} = [ $env_name, $env_value ];
}
}


return $env_value;
}




sub substitute_args($$$$)
{
my ($strings_ref,	    # $string == OK
$name,
$value,
$is_mandatory,
) = @_;
my @strings;    # or $string

@strings = (ref $strings_ref) ? @{$strings_ref} : ($strings_ref);

my $token_name = ($IS_WIN32) ? "\%$name\%" : "\$$name";
my $token = quotemeta $token_name;
my $count = 0;

foreach my $string (@strings)
{
$count += $string =~ s/$token/$value/g;
}

if ($count == 0 && $is_mandatory)
{
ENV_sig( EE => "Argument '$token_name' required in string '@strings'");
}

return (wantarray) ? @strings : $strings[0];
}




sub GBSCMD_scripts_path_changed()
{
$OS_GBS_SCRIPTS_PATH = ENV_getenv( 'GBS_SCRIPTS_PATH');
}





sub GBSCMD_get_level($)
{
my ($command,	# e.g.: gbsgen
) = @_;
my $level;	#   $level: 0 = 0, 1 = root, 2 = subsys 3 = component 4 = any

my $ref = $GBS_COMMAND_REFS{$command};

if (defined $ref)
{
$level = $ref->[3];
} else
{
ENV_sig( F => "Unknowmn GBS command '$command'");
}

return $level;
}

1;


