#===============================================================
#
#   gbsedit.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSEDIT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::list;
use glo::ask;
use glo::genopt;
use glo::scm;
use gbsenv;
use gbsglo;
use swrglo;
use gbsscm;
use plugin;
use plg;
use target;
use validate;
use files;
use fcreate;
use fspec;
use run;




sub analyse_file();
sub check_file_location();
sub create_file_if_necessary();
sub do_the_edit();
sub pre_select_plugin();
sub pre_bld_type_gbs_file();
sub gen_type_gbs_file();
sub pre_type_gbs_file();
sub pre_select_sca();
sub pre_export_gbs_file();
sub pre_gbssub_file();
sub pre_makefile_file();
sub pre_gbsall_file();
sub pre_unknown_gbs_file();
sub pre_any_file();
sub validate_sub_comp_subsys($);
sub validate_sub_path($);
sub validate_system();
sub validate_subsys();
sub validate_component();
sub validate_target();
sub ask_level($@);
sub spec_any();
sub create_any();
sub get_gbs_gen_type($);
sub is_src_type($);
sub is_inc_type($);




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = ENV_getenv_perl_path( 'GBS_ROOT_PATH');
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_TOOL = ENV_getenv( 'GBS_TOOL');
my $GBS_SSTYPE = ENV_getenv( 'GBS_SSTYPE');	# GBS MSVS make Other

my $CWD_IS_IN_CURRENT_GBS_PATH = 0;
my $CWD_IS_IN_A_GBS_PATH = 0;
my @SUBDIR_COMP_SUBSYS;

my $SUBSYS;
my $FULL_GBS;
my $COMPONENT;
my $SUBDIR;
my $TARGET;
my $AUDIT;
my $ATT_NAME;			# audit, target or tool name
my $PLUGIN;
my $SPECIFIED_SUBSYS = 0;
my $SPECIFIED_COMPONENT = 0;
my $FILE_SPEC;			# FULL filespec

my $SUB_PATH;
my $FILE_PATH;
my $FILE_NAME;
my $FILE_TYPE;
my $FILE_NAME_AND_TYPE;

my $FILE_TYPE_GBS = '';		# .gbs .usr

my $GENERIC_FILE_NAME;
my $GENERIC_FILE_NAME_VAR_PART;	    # without the leading '_'

my $IN_INSTALL;		# bool: used for gbsall

my $FILE_CREATE_FUNC;	# defaults to ENV_touch_file




my @SUBDIR_NAMES = qw( src inc loc bld dat sav);

my @GENERIC_FILE_NAMES_REFS = (



[ qr/^audit(\.gbs|\.usr)$/,			0,  'audit',	],
[ qr/^bld_flags(_.*|)(\.gbs|\.usr)$/,	2,  'bld_flags',],  # $src_type
[ qr/^bld_incs(_.*|)(\.gbs|\.usr)$/,	2,  'bld_incs',	],  # $src_type
[ qr/^export(\.gbs|\.usr)$/,		0,  'export',	],
[ qr/^flags(_.*|)(\.gbs|\.usr)$/,		2,  'flags',    ],  # $src_type
[ qr/^gbsall(\.bat|\.sh|)$/,		1,  'gbsall',	],  # batch-file
[ qr/^gbssub(\.bat|\.sh|)$/,		1,  'gbssub',	],  # batch-file
[ qr/^incs(_.*|)(\.gbs|\.usr)$/,		2,  'incs',  	],  # $src_type
[ qr/^owners(\.gbs|\.usr)$/,		0,  'owners',	],
[ qr/^Makefile\.mk$/,			-1, 'makefile',	],
[ qr/^sca(_.*|)(\.gbs|\.usr)$/,		2,  'sca',	],  # $target or <uppercase_name>
[ qr/^scope(\.gbs|\.usr)$/,			0,  'scope',	],
[ qr/^ssprop(\.gbs|\.usr)$/,		0,  'ssprop',	],  # No .usr file!
[ qr/^steps(\.gbs|\.usr)$/,			0,  'steps',	],
[ qr/^switch(\.gbs|.usr|)(\.bat|\.sh|)$/,	3,  'switch',	],  # batch-file
[ qr/^sys(\.gbs|\.usr)$/,			0,  'sys',   	],
[ qr/^system(\.gbs|\.usr)$/,		0,  'system',	],
[ qr/^target(\.gbs|\.usr)$/,		0,  'target',	],
[ qr/^tool(\.gbs|\.usr)$/,			0,  'tool',	],
[ qr/.*/,					-1, 'any',	],
[ qr/^.*(\.gbs|\.usr)$/,			-1, 'unkgbs',	],
);

my @ALL_GBS_FILES = map { $_->[2] if ($_->[1] >= 0) } @GENERIC_FILE_NAMES_REFS;

my %FILE_DEFS = (





audit   => [
0, 1, 0, 0,	    \&pre_select_plugin,
sub { FSPEC_att_file(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME) },
sub { FCREATE_att_file( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME, $PLUGIN) } ],
bld_flags   => [
0, 1, 0, 0,	    \&pre_bld_type_gbs_file,
sub { FSPEC_gen_type(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, '', '', $TARGET) },
sub { FCREATE_gen_type( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, '', '', $TARGET) } ],
bld_incs   => [
0, 1, 0, 0,	    \&pre_bld_type_gbs_file,
sub { FSPEC_gen_type(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, '', '', $TARGET) },
sub { FCREATE_gen_type( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, '', '', $TARGET) } ],
flags   => [
0, 1, 0, 0,	    \&pre_type_gbs_file,
sub { FSPEC_gen_type(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, $TARGET) },
sub { FCREATE_gen_type( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, $TARGET) } ],
incs    => [
0, 1, 0, 0,	    \&pre_type_gbs_file,
sub { FSPEC_gen_type(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, $TARGET) },
sub { FCREATE_gen_type( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, $TARGET) } ],
export  => [
3, 2, 1, 0,	    \&pre_export_gbs_file,
sub { FSPEC_export(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT) },
sub { FCREATE_export( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT) } ],
gbsall  => [
0, 0, 0, 0,	    \&pre_gbsall_file,
sub { FSPEC_gbsall(   $FILE_NAME_AND_TYPE, $IN_INSTALL) },
sub { FCREATE_gbsall( $FILE_NAME_AND_TYPE, $IN_INSTALL) } ],
gbssub  => [
1, 2, 0, 0,	    \&pre_gbssub_file,
sub { FSPEC_subsys_script(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS) },
sub { FCREATE_subsys_script( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $GBS_SSTYPE) } ],
makefile  => [
0, 2, 0, 0,	    \&pre_makefile_file,
sub { FSPEC_makefile(   $GBS_ROOT_PATH, $SUBSYS) },
sub { FCREATE_makefile( $GBS_ROOT_PATH, $SUBSYS) } ],
owners  => [
0, 1, 0, 1,	    undef,
sub { FSPEC_owners(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) },
sub { FCREATE_owners( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) } ],
sca    => [
0, 1, 0, 0,	    \&pre_select_sca,
sub { FSPEC_sca(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $AUDIT, $TARGET) },
sub { FCREATE_sca( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $AUDIT, $TARGET) } ],
scope   => [
3, 3, 0, 0,	    undef,
sub { FSPEC_scope(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT) },
sub { FCREATE_scope( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, undef) } ],
ssprop  => [
1, 2, 0, 2,	    undef,
sub { FSPEC_ssprop(   $GBS_ROOT_PATH, $SUBSYS) },
sub { FCREATE_ssprop( $GBS_ROOT_PATH, $SUBSYS) } ],
steps   => [
0, 1, 0, 0,	    undef,
sub { FSPEC_steps(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) },
sub { FCREATE_steps( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $GBS_SYSTEM_NAME)} ],
switch  => [
0, 1, 0, 0,	    undef,
sub { FSPEC_switch(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) },
sub { FCREATE_switch( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $GBS_SYSTEM_NAME) } ],
sys	    => [
0, 1, 0, 0,	    undef,
sub { FSPEC_sys(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) },
sub { FCREATE_sys( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $GBS_SYSTEM_NAME) } ],
system  => [
0, 1, 0, 1,	    undef,
sub { FSPEC_system(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) },
sub { FCREATE_system( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH) } ],
target  => [
0, 1, 0, 0,	    \&pre_select_plugin,
sub { FSPEC_att_file(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME) },
sub { FCREATE_att_file( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME, $PLUGIN) } ],
tool    => [
0, 1, 0, 0,	    \&pre_select_plugin,
sub { FSPEC_att_file(   $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME) },
sub { FCREATE_att_file( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME, $PLUGIN) } ],
unkgbs  => [
0, 0, 0, 0,	    \&pre_unknown_gbs_file,
sub { spec_any()   },
sub { create_any() } ],
any  => [
0, 0, 1, 0,	    \&pre_any_file,
sub { spec_any() },
sub { create_any() } ],
);

my $FILE_DEF_REF;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',  'file',	     'ssm', "", '[[[subsys:]component:]subdir:[<subpath>/]]file'],
[ '<2>',  'editor_opt',  'sso', "", 'Editor option. e.g. Line-nr to jump to'],
[ 'nco',  'no_checkout', 'bso', 0 , '.usr files will always be forced to no_checkout'],
);
GENOPT_set_optdefs( 'gbsedit', \@genopts,
[ 'Create/edit GBS-files and user-files. GBS-files end with .gbs or .usr',
'The switch.gbs, gbssub and gbsall batch files are also considered GBS_files',
'Source, Include and glkbt-files are considered user-files',
'Relevant templates will be used with file-creation' ],
undef);
GENOPT_parse();
my $FILE = GENOPT_get( 'file');
my $EDITOR_OPT = GENOPT_get( 'editor_opt');
my $NO_CHECKOUT = GENOPT_get( 'no_checkout');

my $MUST_CHECKOUT = !$NO_CHECKOUT;




{
ENV_say( 1, "No current GBS system.")
if ($GBS_ROOT_PATH eq '');

if ($GBS_ROOT_PATH ne '' && ENV_is_in_path( $CWD, $GBS_ROOT_PATH))
{

$CWD_IS_IN_CURRENT_GBS_PATH = 1;
$CWD_IS_IN_A_GBS_PATH = 1;
} else
{
if (SWRGLO_find_root_path( $CWD) ne '')
{

$CWD_IS_IN_A_GBS_PATH = 1;
} else
{

}
}
}


$FILE = ENV_perl_canon_paths( $FILE);
if (-e $FILE)
{



$FILE_SPEC = ENV_abs_path( $CWD, $FILE);
if ($GBS_ROOT_PATH eq '' || !ENV_is_in_path( $FILE_SPEC, $GBS_ROOT_PATH))
{
ENV_say( 1, "File '$FILE' is not in located in current GBS System ($GBS_SYSTEM_NAME).");
if ($MUST_CHECKOUT)
{
$MUST_CHECKOUT = 0;
ENV_say( 2, "CheckOut disabled.");
}
}
do_the_edit();
} else
{



if ($FILE =~ /:/)
{
ENV_sig( EE => "The [[[subsys:]component:]subdir:]file notation is only possible in a GBS System")
if ($GBS_ROOT_PATH eq '');

(my $file, @SUBDIR_COMP_SUBSYS) = reverse( split( ':', $FILE));
ENV_sig( EE => "Syntax is [[[<subsys>:][<component>:]][<subdir>:][<subpath>/]]<file>")
if (@SUBDIR_COMP_SUBSYS > 3);

($SUB_PATH, $FILE_NAME, $FILE_TYPE) = ENV_split_spec_dnt( $file);
$FILE_PATH = '';
} else
{
$SUB_PATH = '';
($FILE_PATH, $FILE_NAME, $FILE_TYPE) = ENV_split_spec_dnt( $FILE);
}




analyse_file();




if ($CWD_IS_IN_CURRENT_GBS_PATH)
{
VALIDATE_gbs();
} elsif ($CWD_IS_IN_A_GBS_PATH)
{
ENV_say( 1, "CWD is not in current GBS system.");
} else
{
ENV_say( 1, "CWD is not in a GBS system.");
}




check_file_location();
create_file_if_necessary();
do_the_edit();
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}





sub analyse_file()
{
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";




my $found_ref;
my $arg1;
my $arg2;
foreach my $ref (@GENERIC_FILE_NAMES_REFS)
{
if ($FILE_NAME_AND_TYPE =~ $ref->[0])		# $re
{
$found_ref = $ref;
$arg1 = $1;
$arg2 = $2;
last;
}
}




if (defined $found_ref)
{
my (undef, $type, $generic_name) = @{$found_ref};


if ($type == 0)	    # scope.gbs scope.usr
{
$FILE_TYPE_GBS = $arg1;
} elsif ($type == 1)    # gbssub.bat gbssub.sh
{
my $file_type = $arg1;
if ($file_type eq '')
{
$FILE_TYPE = ENV_shell_filetype();
$FILE_NAME = $FILE_NAME_AND_TYPE;
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";
ENV_say( 1, "Assumed $FILE_NAME_AND_TYPE");
}

} elsif ($type == 2)    # flags(_*).gbs flags(_*).usr
{
$GENERIC_FILE_NAME_VAR_PART = $arg1;
$GENERIC_FILE_NAME_VAR_PART =~ s/^_//;	# remove leading '_' (if present)
$FILE_TYPE_GBS = $arg2;
} elsif ($type == 3)    # switch.gbs.bat switch.gbs.sh switch.usr.bat switch.usr.sh
{
my $file_type_gbs = $arg1;
my $file_type = $arg2;
if ($file_type_gbs eq '')
{
$FILE_TYPE_GBS = '.gbs';
$FILE_NAME = "$FILE_NAME$FILE_TYPE_GBS";
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";
} else
{
$FILE_TYPE_GBS = $file_type_gbs;
}
if ($file_type eq '')
{
$FILE_TYPE = ENV_shell_filetype();
$FILE_NAME = $FILE_NAME_AND_TYPE;
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";
}
ENV_say( 1, "Assumed $FILE_NAME_AND_TYPE")
if ($file_type_gbs eq '' || $file_type eq '');
} elsif ($type == -1)    # *.gbs, *.usr *.*
{
ENV_say( 1, "Non GBS file with type $FILE_TYPE")
if ($generic_name eq 'unkgbs');
} else
{
ENV_sig( F => "Impossible type ($type)");
}
$GENERIC_FILE_NAME = $generic_name;
$FILE_DEF_REF = $FILE_DEFS{$GENERIC_FILE_NAME};
} else
{
ENV_sig( F => 'No Pattern match');
}

if (0)
{
CORE::say( "FILE_NAME=$FILE_NAME");
CORE::say( "FILE_TYPE=$FILE_TYPE");
CORE::say( "FILE_NAME_AND_TYPE=$FILE_NAME_AND_TYPE");
CORE::say( "GENERIC_FILE_NAME=$GENERIC_FILE_NAME");
CORE::say( "FILE_TYPE_GBS=$FILE_TYPE_GBS");
CORE::say( "GENERIC_FILE_NAME_VAR_PART=$GENERIC_FILE_NAME_VAR_PART")
if (defined $GENERIC_FILE_NAME_VAR_PART);
}

ENV_exit()
if ($FILE_TYPE eq '' && ASK_YNQ( "No file-type specified ($FILE_NAME). Continue?", 'N', 1) eq 'N');


}




sub check_file_location()
{



my ($subsys_comp, $minimum_level, $sub_path_allowed, $special_file, $pre_func, $fspec_func, $create_func) = @{$FILE_DEF_REF};




validate_sub_comp_subsys( $subsys_comp);

if ($minimum_level == 1)		    # system
{
validate_system();
} elsif ($minimum_level == 2)	    # subsys
{
validate_subsys();
} elsif ($minimum_level == 3)	    # component
{
validate_component();
}

ENV_sig( EE => "SubPath '$SUB_PATH' not allowed for this kind of GBS file")
if ($SUB_PATH ne '' && !$sub_path_allowed);

if ($FILE_TYPE_GBS eq '.gbs')
{
VALIDATE_administrator();
} elsif ($FILE_TYPE_GBS eq '.usr')
{
$MUST_CHECKOUT = 0;
}




if ($special_file > 0)
{
if ($special_file == 2)
{
ENV_sig( EE => "Cannot create/edit .usr of $FILE_NAME")
if ( $FILE_TYPE eq '.usr');
}
ENV_sig( W => "***",
"** Special GBS file ($FILE_NAME_AND_TYPE) You are on dangerous ground!",
"***");
ENV_exit()
if (ASK_YNQ('Edit anyway?', 'N', 1) eq 'N');
}




if (defined $pre_func)
{
&$pre_func();
}




$FILE_SPEC = &$fspec_func();		# get the filespec




$FILE_CREATE_FUNC = $create_func;
}




sub create_file_if_necessary()
{
my $must_create = 0;

if (!-f $FILE_SPEC)
{
ENV_say( 1, "File '$FILE_SPEC' does not exist");
ENV_exit()
if (ASK_YNQ( 'Create?', 'Y', 1) eq 'N');
ENV_say( 1, "Creating '$FILE_SPEC'...");
$must_create = 1;
} elsif (-z $FILE_SPEC)
{
ENV_say( 1, "File '$FILE_SPEC' exists but has zero size");
if (ASK_YNQ( 'Re-Create?', 'Y', 1) eq 'Y')
{
ENV_say( 1, "Re-Creating '$FILE_SPEC'...");
$must_create = 1;
}
}

if ($must_create)
{



if (defined $FILE_CREATE_FUNC)
{
&$FILE_CREATE_FUNC();
} else
{
ENV_touch_file( $FILE_SPEC);
}
}
}




sub do_the_edit()
{
if ($MUST_CHECKOUT)
{
GBSSCM_preset( 0);
GBSSCM_connect();
SCM_assert_co_file( $FILE_SPEC);
}

ENV_say( 1, "Editing '$FILE_SPEC'...");
RUN_editor( $FILE_SPEC, $EDITOR_OPT);
}




sub pre_select_plugin()
{



my $att_type = $GENERIC_FILE_NAME;
my $att_type_uc = uc $att_type;		# E.g.: TARGET
my $att_type_fuc = ucfirst $att_type;		# E.g.: Target
my @all_atts = ENV_getenv( "GBS_ALL_${att_type_uc}S");
ENV_sig( EE => "No ${att_type_fuc}s defined")
if (!@all_atts);

my $default_att = ENV_getenv( "GBS_$att_type_uc");
$default_att = $all_atts[0]
if ($default_att eq '' && @all_atts == 1);
$ATT_NAME = ASK_value_from_menu( "Select $att_type_fuc", \$default_att, undef, [ @all_atts ]);

my $filespec = FSPEC_att_file( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $ATT_NAME);
if (! -e $filespec)
{
$PLUGIN = '';
if ($ATT_NAME eq $default_att)
{
$PLUGIN = ENV_getenv( "GBS_${att_type_uc}_PLUGIN");
}
if ($PLUGIN eq '')
{



my @all_plugins = PLG_get_plugin_names( $att_type, 1);	# $get_all_platforms = 1
my @all_att_plugins = PLUGIN_get_all_plugin_names( PLUGIN_att2plugin( $att_type));
@all_plugins = LIST_unique( [ @all_plugins, @all_att_plugins ]);
push @all_plugins, '<Define Own>';
$PLUGIN = ASK_value_from_menu( "Select $att_type_fuc Plugin", -1, undef, [ @all_plugins ]);
if ($PLUGIN eq '<Define Own>')
{
$PLUGIN = ASK_dir( "Enter $att_type_fuc Plugin name", '', '', 0, -1);
}
}
}
}




sub pre_bld_type_gbs_file()
{



gen_type_gbs_file();
}




sub gen_type_gbs_file()
{






validate_target();

if ($GENERIC_FILE_NAME_VAR_PART eq '')
{



ENV_say( 1, "Format of $GENERIC_FILE_NAME file name is '${GENERIC_FILE_NAME}_<src_type>'");
my @src_types = TARGET_get_src_types( $GBS_TARGET);
my $index = ASK_index_from_menu( 'Select src type', -1, undef, [ @src_types ]);
my $src_type = substr( $src_types[ $index ], 1);	# Remove the '.'
$FILE_NAME = "${GENERIC_FILE_NAME}_$src_type";
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";
$GENERIC_FILE_NAME_VAR_PART = $src_type;
} else
{



my $src_type = ".$GENERIC_FILE_NAME_VAR_PART";
if (!is_src_type( $src_type))
{
ENV_sig( W => "Unknown src type '$src_type' in this context ($TARGET)");
ENV_exit()
if (ASK_YNQ( 'Continue anyway?', 'Y', 1) eq 'N');
}
}
}




sub pre_type_gbs_file()
{



gen_type_gbs_file();

my $default = -1;

my @item_refs;

push @item_refs,
[ "sysbuild/$TARGET",	"$GBS_ROOT_PATH/sysbuild/$TARGET",	    [ '', '', $TARGET ] ]
if ($TARGET);
push @item_refs,
[ "sysbuild",		"$GBS_ROOT_PATH/sysbuild",		    [ '', '', '' ] ];
push @item_refs,
[ "build/$TARGET",	"$GBS_ROOT_PATH/dev/$SUBSYS/build/$TARGET", [ $SUBSYS, '', $TARGET ] ]
if ($TARGET);
push @item_refs,
[ "build",		"$GBS_ROOT_PATH/dev/$SUBSYS/build",	    [ $SUBSYS, '', '' ] ];

if ($FULL_GBS && $COMPONENT ne '')
{
push @item_refs,
[ "opt/$TARGET",	"$GBS_ROOT_PATH/dev/$SUBSYS/comp/$COMPONENT/opt/$TARGET",   [ $SUBSYS, $COMPONENT, $TARGET ] ]
if ($TARGET);
push @item_refs,
[ "opt",		"$GBS_ROOT_PATH/dev/$SUBSYS/comp/$COMPONENT/opt",	    [ $SUBSYS, $COMPONENT, '' ] ];
}

ENV_debug( 1, scalar @item_refs . " Items");
ENV_say( 1, "Currencies: $SUBSYS : $COMPONENT : $TARGET");
my $index = ask_level( $default, @item_refs);
($SUBSYS, $COMPONENT, $TARGET) = @{$item_refs[$index]->[2]};	# $user_data


if ($MUST_CHECKOUT)
{




my $filespec = FSPEC_gen_type( $FILE_NAME_AND_TYPE, $GBS_ROOT_PATH, $SUBSYS, $COMPONENT, $TARGET);
GBSSCM_preset( 0);
GBSSCM_connect();

my $dir = ENV_split_spec_d( $filespec);
my $dir_state = SCM_get_states( $dir);

SCM_add_dirs( $dir, \&FILES_get_ignore_spec, 0, 0)	    # $pre_validated, $verbose
if ($dir_state < 0);	# not_scm, not_exist
}
}




sub pre_gbsall_file()
{



if ($MUST_CHECKOUT)
{
$MUST_CHECKOUT = 0;
ENV_say( 1, "CheckOut disabled.");
}




my @item_refs = (

[ 'In User dir    (GBS_BASE_PATH)',	ENV_getenv( 'GBS_BASE_PATH') ],
[ 'In Install dir (GBS_SCRIPTS_ROOT)',	ENV_getenv( 'GBS_SCRIPTS_ROOT') ]
);
$IN_INSTALL = (ask_level( -1, @item_refs) == 0) ? 0 : 1;
}




sub pre_gbssub_file()
{



ENV_sig( EE => "No current SubSys")
if ($SUBSYS eq '');
ENV_sig( EE => "This file is only allowed for NON Full GBS SubSystems")
if ($FULL_GBS);
}




sub pre_makefile_file()
{



ENV_sig( EE => "No current SubSys")
if ($SUBSYS eq '');
ENV_sig( EE => "This file is only allowed for 'make' and 'Other' SubSystem Types ($GBS_SSTYPE)")
if ($GBS_SSTYPE ne 'make' && $GBS_SSTYPE ne 'Other');
}




sub pre_select_sca()
{






my @all_targets = ENV_getenv( 'GBS_ALL_TARGETS');
ENV_sig( EE => 'No Targets defined')
if (!@all_targets);
my @all_audits = ENV_getenv( 'GBS_ALL_AUDITS');
ENV_sig( EE => "No Audits defined")
if (!@all_audits);




my $target = $GENERIC_FILE_NAME_VAR_PART;
if ($target eq '')
{



ENV_say( 1, "Format of sca file name is 'sca_<target>' or 'sca_<uppercasename>'");
my $index = ASK_index_from_menu( 'Specify Fomat type', 0, undef, [ 'Target', 'UpperCase name' ]);
if ($index == 0)
{
my $default_target = $GBS_TARGET;
$default_target = $all_targets[0]
if ($default_target eq '' && @all_targets == 1);
$index = ASK_index_from_menu( 'Select Target', \$default_target, undef, [ @all_targets ]);
$target = $all_targets[ $index ];
} else
{
$target = ASK_file( 'Enter Uppercase name', 'ALL', '', 0, -1);	# $default, $path, $must_exist, $length
if ($target ne uc $target)
{
ENV_sig( W => "Name '$target' contains lowercase charaters. Converted to uppercase");
$target = uc $target;
}
}
$FILE_NAME = "sca_$target";
$FILE_NAME_AND_TYPE = "$FILE_NAME$FILE_TYPE";
$GENERIC_FILE_NAME_VAR_PART = $target;
} else
{



if (!grep( $_ eq $target, @all_targets) && $target ne uc $target)
{
ENV_say( 1, "Available Targets: @all_targets");
ENV_sig( EE => "The name part ($target) of $FILE_NAME_AND_TYPE must be either a Target name or UpperCase");
}
}




my $default_audit = $GBS_AUDIT;
$default_audit = $all_audits[0]
if ($default_audit eq '' && @all_audits == 1);
$AUDIT = ASK_value_from_menu( "Select Audit", \$default_audit, undef, [ @all_audits ]);




my $target_select = ($target eq 'ALL') ? undef : $target;
my @audit_subsystems = GBSGLO_subsystems_for_audit( $AUDIT, $target_select);

my @item_refs = ( [ "sysaudit/$AUDIT", "$GBS_ROOT_PATH/sysaudit/$AUDIT", '' ]);

foreach my $subsys (@audit_subsystems)
{
push @item_refs, [ "dev/$subsys/audit/$AUDIT", "$GBS_ROOT_PATH/dev/$subsys/audit/$AUDIT", $subsys ];
}
my $index = ask_level( 0, @item_refs);

$SUBSYS = $item_refs[$index]->[2];	# $user_data

$TARGET = $target;
}




sub pre_export_gbs_file()
{








if ($FULL_GBS)
{
if ($SPECIFIED_COMPONENT)
{
validate_sub_path( undef);
} else
{
if ($SUB_PATH ne '')
{
validate_sub_path( 'build');
$COMPONENT = '';
} else
{
if ($COMPONENT ne '')
{
my $se = (-e "$GBS_ROOT_PATH/dev/$SUBSYS/build/$FILE_NAME_AND_TYPE") ? '*' : '';
my $ce = (-e "$GBS_ROOT_PATH/dev/$SUBSYS/comp/$COMPONENT/$FILE_NAME_AND_TYPE") ? '*' : '';
my @list = ( "SubSys$ce ($SUBSYS)",
"Component$ce ($COMPONENT)");
my $cur_entry = @list - 1;  # last entry

my $index = ASK_index_from_menu( 'Select Level of export.gbs', $cur_entry, undef, [ @list ]);
$COMPONENT = ''
if ($index == 0);
}
}
}
} else
{
$COMPONENT = undef;
}
}




sub pre_unknown_gbs_file()
{



my $nr_gbs_files = @ALL_GBS_FILES;
my $nr_per_line = $nr_gbs_files / 2;
ENV_sig( W => "Unknown GBS ($FILE_TYPE) file ($FILE_NAME_AND_TYPE)",
"Valid GBS files:",
"- @ALL_GBS_FILES[0..$nr_per_line]",
"- @ALL_GBS_FILES[$nr_per_line..$#ALL_GBS_FILES]");
ENV_exit()
if (ASK_YNQ( 'Edit anyway?', 'N', 1) eq 'N');

pre_any_file();
}




sub pre_any_file()
{






my $path;
if ($SUBDIR ne '')
{
validate_component();
$path = "$GBS_ROOT_PATH/dev/$SUBSYS/comp/$COMPONENT/$SUBDIR";
} else
{
$path = $CWD;
ENV_say( 1, "Currently in: $path");
if ($CWD_IS_IN_CURRENT_GBS_PATH)
{
my $this_dir = ENV_parent_dir( $path, -1);
if (grep( $_ eq $this_dir, @SUBDIR_NAMES))
{
$SUBDIR = $this_dir;
}
}
}

if ($SUBDIR eq 'src')
{
if (!is_src_type( $FILE_TYPE))
{
ENV_sig( W => "Unknown src type '$FILE_TYPE' in this context ($TARGET)");
ENV_exit()
if (ASK_YNQ( 'Continue anyway?', 'Y', 1) eq 'N');
}
ENV_sig( EE => "Cannot specify SubPath ($SUB_PATH) in SubDir $SUBDIR")
if ($SUB_PATH ne '');
} elsif ($SUBDIR eq 'loc'|| $SUBDIR eq 'inc')
{
if (!is_inc_type( $FILE_TYPE))
{
ENV_sig( W => "Unknown $SUBDIR type '$FILE_TYPE' in this context ($TARGET)");
ENV_exit()
if (ASK_YNQ( 'Continue anyway?', 'Y', 1) eq 'N');
}
ENV_sig( EE => "Cannot specify SubPath ($SUB_PATH) in SubDir $SUBDIR")
if ($SUB_PATH ne '');
} else
{
$path .= "/$SUB_PATH"
if ($SUB_PATH ne '');
ENV_sig( EE => "Path '$path' does not exist")
if (!-d $path);
}

$FILE_PATH = $path;
}




sub validate_sub_comp_subsys($)
{
my ($subsys_comp) = @_;	    # 0 = none, 1 = subsys, 2 = component, 3 = subsys+comp

if (@SUBDIR_COMP_SUBSYS > 0)
{
if ($subsys_comp == 0)		# any
{
($SUBDIR, $COMPONENT, $SUBSYS) = @SUBDIR_COMP_SUBSYS;
} elsif ($subsys_comp == 1)	# subsys only
{
if (@SUBDIR_COMP_SUBSYS == 1)
{
$SUBSYS = $SUBDIR_COMP_SUBSYS[0];
} else
{
ENV_sig( EE => "Can only specify SubSys: for GBS file $FILE_NAME_AND_TYPE");
}
} else # $subsys_comp == 2 or $subsys_comp == 3
{
if (@SUBDIR_COMP_SUBSYS == 1)
{
$SUBSYS = $SUBDIR_COMP_SUBSYS[0];
} elsif (@SUBDIR_COMP_SUBSYS == 2)
{
($COMPONENT, $SUBSYS) = @SUBDIR_COMP_SUBSYS;
} else
{
ENV_sig( EE => "can only specify SubSys: and/or Component: for GBS file $FILE_NAME_AND_TYPE");
}
}
}




if ($SUBSYS)
{
$SPECIFIED_SUBSYS = 1;
$SUBSYS = $GBS_SUBSYS
if ($SUBSYS eq '.');
ENV_sig( EE => "No such SubSystem '$SUBSYS'")
if (! -e "$GBS_ROOT_PATH/dev/$SUBSYS");
} else
{
$SUBSYS = $GBS_SUBSYS;
}
if ($SUBSYS)
{
$FULL_GBS = GBSGLO_subsystem_is_full_gbs( $SUBSYS);
}




if ($COMPONENT)
{
$SPECIFIED_COMPONENT = 1;
$COMPONENT = $GBS_COMPONENT
if ($COMPONENT eq '.');
ENV_sig( EE => "No such Component '$COMPONENT' in SubSystem '$SUBSYS'")
if (! -e "$GBS_ROOT_PATH/dev/$SUBSYS/comp/$COMPONENT");
} else
{
$COMPONENT = $GBS_COMPONENT;
}



if ($SUBDIR)
{
ENV_sig( EE => "SubDir '$SUBDIR' must be one of (@SUBDIR_NAMES)")
if (!grep( $SUBDIR eq $_, @SUBDIR_NAMES));
} else
{
$SUBDIR = '';
}




$TARGET = $GBS_TARGET;
$AUDIT = $GBS_AUDIT;




if ($SUBDIR ne '')
{
ENV_debug( 1, "$SUBSYS:$COMPONENT:$SUBDIR");
} elsif ($COMPONENT ne '')
{
ENV_debug( 1, "$SUBSYS:$COMPONENT");
} elsif ($SUBSYS ne '')
{
ENV_debug( 1, "$SUBSYS");
}
}




sub validate_sub_path($)
{
my ($alloweds_ref) = @_;

if ($SUB_PATH ne '')
{
if (defined $alloweds_ref)
{
$alloweds_ref = [ $alloweds_ref ] if (!ref $alloweds_ref);
ENV_sig( EE => "'$FILE': SubPath must be one of (@{$alloweds_ref})")
if (!grep( $_ eq $SUB_PATH, @{$alloweds_ref}));
} else
{
ENV_sig( EE => "'$FILE': SubPath cannot be used in this context");
}
}
}




sub validate_system()
{
ENV_sig( EE => "No current GBS System")
if ($GBS_ROOT_PATH eq '');
}




sub validate_subsys()
{
validate_system();
ENV_sig( EE => "No current SubSys")
if ($SUBSYS eq '');
}




sub validate_component()
{
validate_subsys();
ENV_sig( EE => "No current Component")
if ($COMPONENT eq '');
}




sub validate_target()
{
ENV_sig( EE => "No current Target")
if ($TARGET eq '');
}




sub ask_level($@)
{
my ($default,	# 0 == first
@item_refs	# [ $item_name, $path, $user_data ]
) = @_;


if ($default == -1)
{
foreach my $i (0..$#item_refs)
{
if ($CWD eq $item_refs[$i]->[1])	# $path
{
$default = $i;
last;
}
}
}

foreach my $i (0..$#item_refs)
{
my $ref = $item_refs[$i];
my ($item_name, $path) = @{$ref};

if (-f "$path/$FILE_NAME_AND_TYPE")
{
$ref->[0] .= ' *';	# $item_name
$default = $i
if ($default == -1);
}
}

return ASK_index_from_menu( "Select Level for '$FILE_NAME_AND_TYPE'", $default, undef, [ @item_refs ]);
}




sub spec_any()
{
my $filespec;

if ($FILE_PATH eq '')
{
$filespec = "$CWD/$FILE_NAME_AND_TYPE";
} else
{
$filespec = "$FILE_PATH/$FILE_NAME_AND_TYPE";
}

return $filespec;
}




sub create_any()
{

my @any_filetypes = ($FILE_TYPE);
if ($SUBDIR eq 'src')
{
my $src_filetype = '.' . get_gbs_gen_type( $FILE_TYPE);
push @any_filetypes, $src_filetype
unless ($src_filetype eq $FILE_TYPE);
} elsif ($SUBDIR eq 'inc' || $SUBDIR eq 'loc')
{



foreach my $src_type (TARGET_get_src_types( $TARGET))
{
my @inc_types = TARGET_get_src_items( $TARGET, $src_type, 'INC_TYPES');
if (grep $_ eq $FILE_TYPE, @inc_types)
{
push @any_filetypes, $src_type;
my $src_filetype = '.' . get_gbs_gen_type( $src_type);
push @any_filetypes, $src_filetype;
last;
}
}
}
my (@anyfile_specs) = FCREATE_get_any( \@any_filetypes);

my $anyfile_spec_or_comment_ref;
if (@anyfile_specs)
{



my $anyfile_spec;
if (@anyfile_specs == 1)
{
$anyfile_spec = $anyfile_specs[0];
} else
{
ENV_say( 1, "Multiple Templates found:");

my @menu_items;
foreach my $spec (@anyfile_specs)
{
my $file = ENV_split_spec_f( $spec);
$file =~ s/any_//;
push @menu_items, [ $file, $spec ];
}
$anyfile_spec = ASK_value_from_menu( "  Select Template:", 0, undef, [ @menu_items ]);
}
ENV_say( 1, "- Using $anyfile_spec");
$anyfile_spec_or_comment_ref = $anyfile_spec;
} else
{



ENV_say( 1, "No Template File found");
my $begin_com = ASK_text( '- Enter Begin Comments character(s)', '#', [ 0 ,3 ]);
if ($begin_com ne '')
{
my $end_com   = ASK_text( '- Enter   End Comments character(s)', '',  [ 0, 3 ]);
$anyfile_spec_or_comment_ref = [ $begin_com, $end_com ];
} else
{
$anyfile_spec_or_comment_ref = '';
ENV_say( 1, "Creating empty file...");
}
}


return FCREATE_any( $FILE_SPEC, $anyfile_spec_or_comment_ref, [ $SUBSYS, $COMPONENT ]);
}




sub get_gbs_gen_type($)
{
my ($src) = @_;
my $gbs_src_type;

if (is_src_type( $src))
{
$gbs_src_type = TARGET_get_src_items( $TARGET, $src, 'TYPE')->[0];	# $type e.g.: gen, asm, c, glb, ...
} else
{
$gbs_src_type = '';
}

return $gbs_src_type;
}




sub is_src_type($)
{
my ($type) = @_;
my $is_src_type = 0;

VALIDATE_target();	    # Error-exit if current component does not generate for the current Target

if (-f FSPEC_att_file( 'target', $GBS_ROOT_PATH, $TARGET) &&
TARGET_is_src_type( $TARGET, $type))
{
$is_src_type = 1;
}

return $is_src_type;
}




sub is_inc_type($)
{
my ($type) = @_;
my $is_inc_type = 0;

VALIDATE_target();	    # Error-exit if current component does not generate for the current Target

if (-f FSPEC_att_file( 'target', $GBS_ROOT_PATH, $TARGET))
{
my @inc_types = TARGET_get_inc_types( $TARGET);
if (grep $_ eq $type, @inc_types)
{
$is_inc_type = 1;
}
}
return $is_inc_type;
}

