#===============================================================
#
#   gbshtml.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbshtml;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSHTML_init
GBSHTML_doc_start
GBSHTML_doc_top
GBSHTML_doc_title
GBSHTML_h
GBSHTML_get_toc
GBSHTML_doc_bottom
GBSHTML_doc_end
);
}




use glo::env;
use glo::html;
use glo::flock;
use glo::build;
use gbsres;
use gbsdb;
use gbsenv;




sub GBSHTML_init($);
sub GBSHTML_doc_start($$$$$);
sub GBSHTML_doc_top($@);
sub GBSHTML_doc_title($$);
sub GBSHTML_h($$$$);
sub GBSHTML_get_toc($);
sub GBSHTML_doc_bottom();
sub GBSHTML_doc_end();

sub insert_img($);
sub banner(@);
sub indent($@);




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $FILE_ORIGIN = '';	# 'internal', 'plugin', 'application'

my $USE_CACHE = 0;

my $GBS_SCRIPTS_DOC_PATH = "$GBS_SCRIPTS_PATH/doc";
my $HTML_PATH;
my $HTML_IMAGES_PATH;	    # $HTML_PATH/images
my $HTML_SCRIPTS_PATH;	    # $HTML_PATH/scripts

my $FILENAME;
my $HEAD_TITLE;
my $SUB_TITLE;
my $BODY_CLASS;
my @H_COUNTS;
my @TOC_REFS;

my @FILE_ITEMS_REFS;	# [ $FILENAME, $HEAD_TITLE, $SUB_TITLE, $BODY_CLASS, [ @H_COUNTS ], [ @TOC_REFS ]  ]





my $GBS_HEAD_COMMENTS = <<HERE1
<!--==============================================================
#   This file is a part of the GBS (Generic Build Support) package
#   created by Randy Marques of Randy Marques Consultancy, Netherlands
#   For information: www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#==============================================================-->
HERE1
;

my $OTHER_HEAD_COMMENTS = <<HERE2
<!--==============================================================
#   This file was generated by the GBS (Generic Build Support) package
#   created by Randy Marques of Randy Marques Consultancy, Netherlands
#   For information: www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#==============================================================-->
HERE2
;





my %FILES = (
'gbs.css'	    => 1,	    # Keep for backward compatibility
'gbs3.css'	    => 1,
'gbsprint.css'  => 1,
'favicon.ico'   => 1,
);

my %SCRIPTS = (
'sorttable.js'  => 1,
);

my %IMGS =  (

up_small	 => [ 'index_up_3.gif',		'level_1',    'Up Chapter Button',	'Prev Chapter' ],   # 14x10
down_small	 => [ 'index_down_3.gif',	'level_1',    'Down Chapter Button',	'Next Chapter' ],
blank_small  => [ 'blank.gif',		'level_1',    'Blank' ],
up_medium	 => [ 'index_up_2.gif',		'level_1',    'Up Section Button',	'Prev Section' ],   # 16x18
down_medium  => [ 'index_down_2.gif',	'level_1',    'Down Section Button',	'Next Section' ],
blank_medium => [ 'blank.gif',		'level_1',    'Blank' ],
up_big	 => [ 'index_up_1.gif',		'level_1',    'Top Document Button',	'Top Document' ],   # 24x24
down_big	 => [ 'index_down_1.gif',	'level_1',    'Bottom Doc. Button',	'Bottom Document' ],
blank_big	 => [ 'blank.gif',		'level_1',    'Blank' ],
gbslogo	 => [ 'gbslogo.gif',		'gbslogo',    'GBS Logo' ],		# 233x145
gbslogo_small=> [ 'gbslogo_small.gif',	'gbslogo_s',  'GBS Logo' ],		# 100x62
rmclogo_wide => [ 'Banner_Wide_240x31.png', 'rmclogo_w',  'RMC Logo' ],		# 240x31
bullet	 => [ 'greysmallbullet.gif',	'bullet_s',   'Bullet', ]		# 5x5
);




sub GBSHTML_init($)
{
my ($file_origin,	# 'internal', 'plugin', 'application'
) = @_;

ENV_sig( F => "GBSHTML_init($file_origin) already initialised for '$FILE_ORIGIN'")
if ($FILE_ORIGIN ne '' && $file_origin ne $FILE_ORIGIN);

if ($file_origin ne $FILE_ORIGIN)
{
$FILE_ORIGIN = $file_origin;

if ($FILE_ORIGIN eq 'plugin')
{
$USE_CACHE = 1;
$HTML_PATH = ENV_canonicalize_path( "$GBS_ROOT_PATH/../.gbs/html");
mkdir $HTML_PATH;
} elsif ($FILE_ORIGIN eq 'application')
{
$USE_CACHE = 1;
$HTML_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache/html";
} elsif ($FILE_ORIGIN eq 'internal')
{
$USE_CACHE = 0;
$HTML_PATH = "$GBS_SCRIPTS_DOC_PATH";
} else
{
ENV_sig( F => "Invalid FILE_ORIGIN ($FILE_ORIGIN)");
}
$HTML_IMAGES_PATH = "$HTML_PATH/images";
$HTML_SCRIPTS_PATH = "$HTML_PATH/scripts";

if ($USE_CACHE)
{
HTML_map( GBSDB_maps_get());

my $lock = FLOCK_ex_wait( "$HTML_PATH/.html.lck");

mkdir $HTML_IMAGES_PATH;
mkdir $HTML_SCRIPTS_PATH;
my @file_to_copy_refs;





foreach my $file (keys %FILES)
{
my $from_filespec = "$GBS_SCRIPTS_DOC_PATH/$file";
my $to_filespec = "$HTML_PATH/$file";
push @file_to_copy_refs, [ $from_filespec, $to_filespec ];
}




foreach my $script (keys %SCRIPTS)
{
my $from_filespec = "$GBS_SCRIPTS_DOC_PATH/scripts/$script";
my $to_filespec = "$HTML_SCRIPTS_PATH/$script";
push @file_to_copy_refs, [ $from_filespec, $to_filespec ];
}




foreach my $ref (values( %IMGS))
{
my $url = $ref->[0];		    # $url
my $from_filespec = "$GBS_SCRIPTS_DOC_PATH/images/$url";
my $to_filespec = "$HTML_IMAGES_PATH/$url";
push @file_to_copy_refs, [ $from_filespec, $to_filespec ];
}




{
my $copy_count = 0;
foreach my $ref (@file_to_copy_refs)
{
my ($from_filespec, $to_filespec) = @{$ref};
if (ENV_file_is_newer( $from_filespec, $to_filespec))
{

ENV_copy_file( $from_filespec, $to_filespec, 'F');
ENV_chmod( 'u+w', $to_filespec);
$copy_count++;
}
}
ENV_say( 1, "$copy_count file(s) copies to $HTML_PATH")
if ($copy_count > 0);
}

FLOCK_unlock( $lock);
}

foreach my $file (keys %FILES)
{
$FILES{$file} = "$HTML_PATH/$file";
}


foreach my $script (keys %SCRIPTS)
{
$SCRIPTS{$script} = "$HTML_SCRIPTS_PATH/$script";
}

}
}





sub GBSHTML_doc_start($$$$$)
{
my ($filespec,	    # name of the html-file to be created (is not created yet)
$head_title,
$want_scripts,
$body_class,	    # e.g.: toc doc. If undef: No body
$target_frame_name, # Used for frame TOC
) = @_;
my @lines;


ENV_sig( F => 'GBSHTML not initialised!')
if ($FILE_ORIGIN eq '');




push @FILE_ITEMS_REFS, [ $FILENAME, $HEAD_TITLE, $SUB_TITLE, $BODY_CLASS, [ @H_COUNTS ], [ @TOC_REFS ] ];




$FILENAME = ENV_split_spec_f( $filespec);
$HEAD_TITLE = $head_title;
$SUB_TITLE = '';
$BODY_CLASS = $body_class;
@H_COUNTS = (0);
@TOC_REFS = ();





my $css_filespec = $FILES{ 'gbs3.css'};
my $cssprint_filespec = $FILES{ 'gbsprint.css'};
my $sortable_spec = ($want_scripts) ? $SCRIPTS{ 'sorttable.js'} : '';
my $favicon_filespec = $FILES{ 'favicon.ico'};

push @lines,
HTML_doc_s( $filespec, $HEAD_TITLE,
($USE_CACHE) ? $OTHER_HEAD_COMMENTS : $GBS_HEAD_COMMENTS,
css => $css_filespec,
cssprint => $cssprint_filespec,
script => $sortable_spec,
favicon => $favicon_filespec,
target_frame_name => $target_frame_name,
description => "Generic Build Support (GBS). Build Automation system for Windows and Linux, developed by Randy Marques of Randy Marques Consultancy",
keywords => "GBS, Generic Build Support, embedded, software, consultancy, configuration management, build automation, programming, make, c, c++, qac, pclint, c++test, tiobe, quality",
);
if (defined $body_class)
{
push @lines, '';
push @lines, HTML_body_s( { class => $BODY_CLASS});
}

return @lines;
}




sub GBSHTML_doc_title($$)
{
my ($title,		# May be undef in which case the head_title will be used
$sub_title,
) = @_;
my @lines;

$HEAD_TITLE = $title
if (defined $title);
$SUB_TITLE = $sub_title;

push @lines, HTML_anchor( "${FILENAME}_PageTop", '');
push @lines, banner( 'Title');
push @lines, HTML_div( {class => 'title' },
indent( 1, HTML_link( 'http://www.genericbuildsupport.com', insert_img( 'gbslogo_small'))),
indent( 1, HTML_link( { class => "home novisit", target => "_top" }, 'index.html', 'HOME')),
indent( 1, HTML_text( $HEAD_TITLE), HTML_br()),
indent( 1, HTML_text( $SUB_TITLE) .
indent( 2, HTML_llink( "${FILENAME}_PageBottom", insert_img( 'down_big'))
)),
);

return @lines;
}





sub GBSHTML_doc_top($@)
{
my ($title_or_ref,	    # [ $title, $t_right ]	$title may be undef in which case the head_title will be used
@item_refs	    # [ $label, $value, $right ]...
) = @_;
my @lines;

my ($title, $t_right);
($title, $t_right) = (ref $title_or_ref) ?  @{$title_or_ref} : ($title_or_ref, '');
$HEAD_TITLE = $title
if (defined $title);
if (@item_refs == 0)
{
if ($t_right eq '')
{
$t_right = HTML_llink( "${FILENAME}_PageBottom", insert_img( 'down_big'));	# Nav
} else
{
@item_refs = ( [ '','','' ] );
}
}




push @lines, HTML_anchor( "${FILENAME}_PageTop", '');
push @lines, banner( 'Title');
my $lic = BUILD_get_lic();




push @lines,
HTML_table_s( id => 'Top', class => 'header', width => '100%');




push @lines,
HTML_trtd( {rowspan => 2}, HTML_link( 'http://www.genericbuildsupport.com', insert_img( 'gbslogo_small'))),
HTML_td( {class => 'gbs-head', width => '100%'}, GBSRES_title_html()),
HTML_td( {class => 'align-right'}, HTML_link( 'http://www.randymarques.com', HTML_bold( GBSRES_company_html()))),
HTML_trtd( {class => 'license', colspan => 2}, "Licensed to: $lic");




if ($t_right eq '')
{
push @lines,
HTML_trtd( {class => 'heading', colspan => 3}, $HEAD_TITLE);
} else
{
push @lines,
HTML_trtd( {class => 'heading', colspan => 2}, $HEAD_TITLE),
HTML_td( {class => 'align-right'}, $t_right);
}





if (@item_refs)
{
$item_refs[$#item_refs]->[2] .= HTML_llink( "${FILENAME}_PageBottom", insert_img( 'down_big'));
foreach my $item_ref (@item_refs)
{
my ( $label, $value, $right ) = @{$item_ref};

if ($value eq '' && $right eq '')
{
push @lines, HTML_trtd( {class => 'subheading', colspan => 3}, $label);
} elsif ($value eq '')
{
push @lines, HTML_trtd( {class => 'subheading', colspan => 2}, $label);
push @lines,   HTML_td( {class => 'subheading'}, $right);
} elsif ($right eq '')
{
push @lines, HTML_trtd( {class => 'subheading'}, $label);
push @lines,   HTML_td( {class => 'subheading', colspan => 2}, ": $value");
} else
{
push @lines, HTML_trtd( {class => 'subheading'}, $label);
push @lines,   HTML_td( {class => 'subheading'}, ": $value");
push @lines,   HTML_td( {class => 'align-right'}, $right);
}
}
}




push @lines,
HTML_table_e();

return @lines;
}




sub GBSHTML_h($$$$)
{
my ($h,		    # h number 1..9
$attrs_ref,	    # HASH, may be undef
$name,
$next_h,	    # Only for h1 and h2
) = @_;
my @lines;

$attrs_ref = {}
if (!defined $attrs_ref);

if ($h <= 2 )	# We only provide <h1> and <h2> with up and down arrows
{
my $h_index = $h - 1;
$H_COUNTS[$h_index]++;
@H_COUNTS = @H_COUNTS[0..$h_index];
my $nr_counts = @H_COUNTS;




my $anchor_prefix = "${FILENAME}_HEAD_";
my $local_url = $anchor_prefix . join( '_', @H_COUNTS);
push @lines, HTML_anchor( $local_url, '');
push @TOC_REFS, [ $h, $name, $local_url ];

my $ext_name = $name;






$ext_name .= indent( 1, HTML_llink( "${FILENAME}_PageBottom", insert_img( 'down_big')));
$ext_name .= indent( 1, HTML_llink( "${FILENAME}_PageTop", insert_img( 'up_big')));




foreach my $i (0..$nr_counts-1)
{
my $size = qw( medium small )[$i];





if (defined $next_h && $next_h > $i)
{
my @counts = @H_COUNTS[0..$i];
$counts[$i]++;
my $next_link = $anchor_prefix . join( '_', @counts);
$ext_name .= indent( 1, HTML_llink( $next_link, insert_img( "down_$size")));
} else
{
$ext_name .= indent( 1, insert_img( "blank_$size"));
}




my $this_count = $H_COUNTS[$i];
if ($this_count > 1)
{
my @counts = @H_COUNTS[0..$i];
$counts[$i]--;
my $prev_link = $anchor_prefix . join( '_', @counts);
$ext_name .= indent( 1, HTML_llink( $prev_link, insert_img( "up_$size")));
} else
{
$ext_name .= indent( 1, insert_img( "blank_$size"));
}
}




if ($h == 1 && $H_COUNTS[0] == 1)
{

$attrs_ref->{class} = 'first';

}
push @lines, HTML_h( $h, $attrs_ref, $ext_name);
} else
{



push @lines, HTML_h( $h, $attrs_ref, $name);
}


return @lines;
}




sub GBSHTML_get_toc($)
{
my ($deepest_level,		# 1 or 2 or default (0) == 2
) = @_;
my @lines;





if (@TOC_REFS == 1)
{
shift @TOC_REFS
if ($TOC_REFS[0]->[0] == 1 && $TOC_REFS[0]->[1] =~ /content/i);
} elsif (@TOC_REFS >= 2)
{
shift @TOC_REFS
if ($TOC_REFS[0]->[0] == 1 && $TOC_REFS[1]->[0] == 1 && $TOC_REFS[0]->[1] =~ /content/i);
}

if ($deepest_level == 1)
{



foreach my $ref (@TOC_REFS)
{
my ($level, $name, $url) = @{$ref};
$name =~ s/:$//;
push @lines, HTML_llink( $url, $name) . HTML_br()
if ($level == 1);
}
} else
{



my @level_2;
foreach my $ref (@TOC_REFS, [ 1, undef, undef ])
{
my ($level, $name, $url) = @{$ref};
$name =~ s/:$//
if (defined $name);
if ($level == 1)
{
if (@level_2)
{
push @lines, HTML_div( { class=>'indent' }, join( '| ', @level_2), HTML_br());
@level_2 = ();
}
push @lines, HTML_llink( $url, HTML_bold( $name)) . HTML_br()
if (defined $name);
} else
{
$name =~ s/ /&nbsp;/g;
push @level_2, HTML_llink( $url, $name);
}
}
}

return @lines;
}




sub GBSHTML_doc_bottom()
{
my @lines;

push @lines, HTML_br();
if (defined $H_COUNTS[0])
{
my $last_head = $H_COUNTS[0] + 1;
my $last_anchor = "${FILENAME}_HEAD_$last_head";
push @lines, HTML_anchor( $last_anchor, '');
}
push @lines, HTML_anchor( "${FILENAME}_PageBottom", '');
my $title = ($SUB_TITLE eq '') ? $HEAD_TITLE : "$HEAD_TITLE - $SUB_TITLE";
push @lines, HTML_div( { class => 'doc_footer' },
indent( 1, $title),
indent( 1, insert_img( 'blank_big')),					    # Nav
indent( 1, HTML_llink( "${FILENAME}_PageTop", insert_img( 'up_big')))	    # Nav
);
push @lines, HTML_div( { class => 'copyright' },
indent( 1, 'Copyright &copy; Randy Marques of Randy Marques Consultancy - All rights reserved'));



return @lines;
}




sub GBSHTML_doc_end()
{
my @lines;

push @lines, HTML_body_e()
if (defined $BODY_CLASS);
push @lines, HTML_doc_e();




($FILENAME, $HEAD_TITLE, $SUB_TITLE, $BODY_CLASS, my ($h_counts_ref, $toc_refs )) = @{pop @FILE_ITEMS_REFS};
@H_COUNTS = @{$h_counts_ref};
@TOC_REFS = @{$toc_refs};

return @lines;
}




sub insert_img($)
{
my ($name) = @_;

my ($url, $class, $alt, $title) = @{$IMGS{$name}};
$url = "$HTML_IMAGES_PATH/$url";

my $html = HTML_img( $url, class => $class, alt => $alt, title => $title);

return $html;
}




sub banner(@)
{
my (@texts
) = @_;
my @lines;

push @lines, '<!--=========================================';
map { push @lines, "==      $_" } @texts;
push @lines, '==========================================-->';

return @lines;
}




sub indent($@)
{
my ($level,
@texts
) = @_;

return "\n" . ('  ' x $level) . "@texts";
}

1;
