#==============================================================
#
#   gbsnew.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSNEW @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::list;
use glo::spit;
use glo::ask;
use glo::slurp;
use glo::genopt;
use glo::build;
use glo::format;
use glo::scm;
use glo::file;
use gbsenv;
use gbsglo;
use gbsscm;
use gbsdb;
use roots;
use rootsglo;
use fcreate;
use files;
use system;
use validate;
use pluginmaint;
use profile;
use ssprop;




sub main_new_system($);
sub create_or_update_system($);
sub ask_new_system_data($);
sub validate_user();
sub main_new_other_system($);
sub main_new_subsys($);
sub check_subsys($$$);
sub ask_new_subsys_data($);
sub main_new_component($);
sub check_component($$$);
sub main_new_target($);
sub main_remove_target($);
sub main_new_audit($);
sub main_remove_audit($);
sub select_via_menu();




my $CWD = ENV_cwd();

my $GBS_ROOT_PATH = ENV_getenv_perl_path( 'GBS_ROOT_PATH');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my @GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');

my @FUNCTION_REFS = (
[ 'exit',		undef			],
[ 'new_root',	\&main_new_system	],
[ 'new_other_root',	\&main_new_other_system ],
[ 'new_subsys',	\&main_new_subsys	],
[ 'new_component',	\&main_new_component	],
[ 'new_target',	\&main_new_target	],
[ 'rem_target',	\&main_remove_target	],
[ 'new_audit',	\&main_new_audit	],
[ 'rem_audit',	\&main_remove_audit	],
);
my %FUNCTION_REFS = map { $_->[0] => $_ } @FUNCTION_REFS;










$| = 1;
my $RC = 0;
my $RESULT = '';    # The (first) item created: $new_root_path, $new_subsys, etc or ''
GBSENV_init();




my @genopts = (
[ '<1>',	'function', 'ssosnew_root,new_other_root,new_subsys,new_component,new_target,rem_target,new_audit,rem_audit',  '', 'Function to be executed' ],
[ '<2>',	'func_arg', 'sso',                               '', 'Function dependent argument' ],
);
my @genconflicts = (
);
GENOPT_set_optdefs( 'gbsnew', \@genopts,
'Root/System, SubSystem, Component and Target Create/Remove/etc',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $FUNCTION = GENOPT_get( 'function');
my $FUNC_ARG = GENOPT_get( 'func_arg');




VALIDATE_gbs();




{
if ($FUNCTION eq '')
{
select_via_menu();
} else
{
my $func = $FUNCTION_REFS{$FUNCTION}->[1];
$RESULT = &$func( $FUNC_ARG);
}
}




{




my @lines;
push @lines, "$RESULT\n";
foreach my $env_name (ENV_get_changed_envs())
{
my $value = ENV_getenv( $env_name);
push @lines, "$env_name=$value\n";
}

my $filespec = ENV_getenv( 'GBS_NEW_FILE');
if ($filespec)
{
SPIT_file( $filespec, \@lines);
}
}

ENV_exit( $RC);




END
{
ENV_print_end_msg( 0);
}




sub main_new_system($)
{
my ($spec_new_root_path) = @_;		# perl_format
my $new_root_path = '';

ENV_say( 1, "New Root ($spec_new_root_path)");

validate_user();




if (ROOTS_get_nr() > 0)
{
ENV_say( 1, "Currently in Roots-List:");
ROOTSGLO_print_root_cols();
} else
{
ENV_say( 1, "No Roots in Roots-List");
}
ENV_say( 0, '');

my $current_root_parent_path = '';
my $last_root_parent_path = '';
my @menu_refs = (
[ '', '' ]		    # <none> entry
);


if ($spec_new_root_path eq '')
{



$current_root_parent_path = ($GBS_ROOT_PATH eq '') ? '' : ENV_parent_path( $GBS_ROOT_PATH);

if ($current_root_parent_path ne '')
{
ENV_say( 1, "Current Root Parent is '$current_root_parent_path'");	# will be default
push @menu_refs, [ "Create a Root in the Current Parent Path ($current_root_parent_path)", 'ADD_CURPAR' ];
} else
{
my $last_root_path = GBSDB_last_root_get();
if ($last_root_path ne '')
{
$last_root_parent_path = ENV_parent_path( $last_root_path);
ENV_say( 1, "Last Used Root Parent is '$last_root_parent_path'");	# will be default
push @menu_refs, [ "Create a Root in the Last Used Parent Path ($last_root_parent_path)", 'ADD_LASTPAR' ];
} else
{

my $gbswa_root = ENV_get_user_path( 0) . '/GBSWA';	# ~/GBSWA or MyDocuments/GBSWA
push @menu_refs, [ "Create in User's GBSWA ($gbswa_root)", 'USE_GBSWA' ];
}
}
if ($CWD !~ /^$GBS_ROOT_PATH/ || $GBS_ROOT_PATH eq '')
{
ENV_say( 1, "Currenty (CWD) in '$CWD'");
push @menu_refs, [ 'Use the Current Path (CWD) as Root Path', 'USE_ROOT' ];
push @menu_refs, [ 'Use the Current Path (CWD) as Parent Path', 'USE_PAR' ];
}
push @menu_refs, [ 'Explicitly specify a Root Path', 'SPEC_ROOT' ];
} else
{



$spec_new_root_path = ENV_abs_path( undef, $spec_new_root_path);
my $exists = (-d $spec_new_root_path) ? 'Exist' : 'NOT Exists';
ENV_say( 1, "Specified '$spec_new_root_path' ($exists)");
push @menu_refs, [ 'Use the specified Path as Root Path', 'USE_SPEC_ROOT' ];
push @menu_refs, [ 'Use the specified Path as Parent Path', 'USE_SPEC_PAR' ];
}

my ($select_value, @more_values) = ASK_value_from_menu( 'Select Root Create action:', 1 , undef, [ @menu_refs ]);


if ($select_value ne '')
{
my $parent_path = '';
my $root_dir = '';
my $root_path = '';	# $parent_path/$root_dir
if ($select_value eq 'ADD_CURPAR')
{
$parent_path = $current_root_parent_path;
} elsif ($select_value eq 'ADD_LASTPAR')
{
$parent_path = $last_root_parent_path;
} elsif ($select_value eq 'USE_ROOT')
{
$root_path = $CWD;
} elsif ($select_value eq 'USE_GBSWA')
{
$parent_path = ENV_get_user_path( 0) . '/GBSWA';	# ~/GBSWA or MyDocuments/GBSWA
if (! -d $parent_path)
{
ENV_mkdir( $parent_path);
ENV_say( 1, " Created $parent_path");
}
} elsif ($select_value eq 'USE_PAR')
{
$parent_path = $CWD;
} elsif ($select_value eq 'SPEC_ROOT')
{

} elsif ($select_value eq 'USE_SPEC_ROOT')
{
$root_path = $spec_new_root_path;
} elsif ($select_value eq 'USE_SPEC_PAR')
{
$parent_path = $spec_new_root_path;
} else
{
ENV_sig( F => "Impossible $select_value");
}
if ($root_path ne '')
{
$parent_path = ENV_parent_path( $root_path);
$root_dir = ENV_parent_dir( $root_path, -1);
}





if ($parent_path eq '')
{



$root_path = ASK_path( "Enter New Root Path", $CWD, 0, -2);
} elsif ($root_dir eq '')
{



ENV_say( 1, "Root Parent is '$parent_path'");
my @dirs = SLURP_dir_dirs( $parent_path, 0);
if (@dirs > 0)
{
ENV_say( 0, "And contains:");
ENV_say( 0, FORMAT_cols( 0, 2, '  ', 2, \@dirs));
}
$root_dir = ASK_dir( 'Enter New Root Dir', '', $parent_path, 0, -2);
$root_path = "$parent_path/$root_dir";
} else
{

ENV_say( 0, "OK");
}
ENV_say( 0, '');
ENV_say( 1, "Selected path:",
"$root_path");

if (-e $root_path)
{



my @root_files = SLURP_dir_all( $root_path, 0);
if (@root_files)
{
ENV_say( 1, "This Root already exists and contains data:",
"@root_files");

if (ASK_YNQ( 'Continue and re-create this Root/System?', 'N', 1) eq 'Y')
{
$new_root_path = ENV_perl_path_nospace( $root_path);
}
} else
{
$new_root_path = ENV_perl_path_nospace( $root_path);
}
} else
{
if (ASK_YNQ( "Root does not exist. Create?", 'Y', 1) eq 'Y')
{
$new_root_path = FILE_mkpath( $root_path);	# Perl path nospace
}
}




if ($new_root_path ne '')
{

create_or_update_system( $new_root_path);
}
}

ENV_sig( W => "No new Root/System created or updated")
if ($new_root_path eq '');

return $new_root_path;
}




sub create_or_update_system($)
{
my ($new_root_path,
) = @_;


ENV_chdir( $new_root_path);





my ( $system_name, @scms_items ) = ask_new_system_data( $new_root_path);




SCM_preset( $new_root_path, [ @scms_items ]);
GBSSCM_setenv( [ @scms_items ]);
GBSSCM_connect();




ENV_say( 1, "Validating Root: $new_root_path");
my $again;
do
{
$again = 0;
my $state = SCM_get_states( $new_root_path);




if ($state == -1)
{
ENV_sig( W => "Root directory not under SCM control");
$again = 1;
} elsif ($state == 0)
{
SCM_checkout( $new_root_path);
}
if ($again)
{
ENV_say( 1, "You now have the chance to fix things. (like Checkout WA)",
"Then enter Y to Retry. Enter N or Q to Quit");
if (ASK_YNQ( "Retry?", 'Y', 1) eq 'Y')
{
GBSSCM_preset( 0);	    # reset scm
} else
{
ENV_exit();
}
}
} while ($again);




ENV_say( 1, "Creating System ...");
FILES_reset( $new_root_path);
my @new_dirs = FILES_mandatory_dirs( $new_root_path, 'SYSTEM');
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 0);	    # $pre_validated, $verbose
ENV_whisper( 1, "Setting ignores...");
SCM_set_ignore( $new_root_path, FILES_get_ignore_spec( $new_root_path), 0);	# do not recurse

my @new_files;




push @new_files, FCREATE_switch( 'switch.gbs', $new_root_path, $system_name);




push @new_files, FCREATE_sys( 'sys.gbs', $new_root_path, $system_name);
SCM_add_txt_files( \@new_files, 0, 0);		# $prevalidated, $verbose




ENV_whisper( 1, "Creating system file");
my $this_gbs_version = BUILD_get_version();
my $gbs_build = ENV_getenv( 'GBS_BUILD');   # VVVYYMMDD

SYSTEM_create( $new_root_path);
SYSTEM_put(	$new_root_path,		build_start	=> $gbs_build);
SYSTEM_put(	$new_root_path,		system_name	=> $system_name);
SYSTEM_put_os( $new_root_path, '.', scms		=> $scms_items[0]);	    # $new_scms,
SYSTEM_put_os( $new_root_path, '.', scms_repository => $scms_items[1]);	    # $new_scms_repository,
SYSTEM_put(	$new_root_path,		scms_data	=> $scms_items[2]);	    # $new_scms_data,
SYSTEM_put(	$new_root_path,		root_version    => $this_gbs_version);
SYSTEM_put_os( $new_root_path, '.', targets		=> []);
SYSTEM_write();   # Includes backup and SCM_checkout

ENV_say( 1, "Root/System '$new_root_path' created");
$GBS_ROOT_PATH = $new_root_path;





ENV_whisper( 1, "Setting GBS Administrator privilege...");
my @gbs_admin = ENV_getenv( 'GBS_ADMINISTRATOR');
push @gbs_admin, $system_name;
ENV_setenv( GBS_ADMINISTRATOR => [ sort @gbs_admin ]);
PROFILE_open( undef);
PROFILE_add( GBS_ADMINISTRATOR => $system_name);
PROFILE_close( 1);	# $must_write
ENV_say( 1, "You are now also GBS Administrator for this Root/System");

}





sub ask_new_system_data($)
{
my ($new_root_path) = @_;
my ($system_name, $scms, $scms_repository, $scms_data);




($system_name) = $new_root_path =~ m!.*[/\\](.+)!;	 #last directory
$system_name = ucfirst $system_name;
$scms = '';

do
{



$system_name = ASK_word( "Enter New System-Name", $system_name, -1);




($scms, $scms_repository, $scms_data) = SCM_select( $scms, $new_root_path, $scms_repository, $scms_data);




ENV_say( 1, "Summary for new Root:");
ENV_say( 0, "  Root           : $new_root_path",
"  System_name    : $system_name",
"  SCMS           : $scms");
ENV_say( 0, "  SCMS-repository: $scms_repository") if (defined $scms_repository);
ENV_say( 0, "  SCMS-data      : $scms_data") if (defined $scms_data);
} while (ASK_YNQ( 'OK?', 'Y', 1) eq 'N');

$scms_data = '' if (!defined $scms_data);
$scms_repository = '' if (!defined $scms_repository);
return ( $system_name, $scms, $scms_repository, $scms_data );
}




sub validate_user()
{
if (0)
{



ENV_say( 1, "==========================================================",
"== GBS is consultancy-ware",
"== The creation of a new Root/System is restricted by the",
"== licence agreement to consultants appointed by",
"== Randy Marques Consultancy",
"==========================================================");




my $digest = 'MaGP3Dn/Ix.uY';
my $ok = ASK_password( 'Enter GBS Consultancy password', $digest);
ENV_sig( EE => 'Invalid Password')
if (!$ok);
}
}




sub main_new_other_system($)
{
my ($spec_new_root_path) = @_;		# perl_format
my $new_root_path = $spec_new_root_path;

ENV_say( 1, "Adjust existing Root for this Platform ($spec_new_root_path)");


ENV_chdir( $new_root_path);





ENV_say( 1, "Platform...");
my $system_name = SYSTEM_get( $new_root_path, 'system_name');
my $new_scms = SYSTEM_get_os( $new_root_path, '.', 'scms');
my $new_scms_repository = SYSTEM_get_os( $new_root_path, '.', 'scms_repository');
my $new_scms_data = SYSTEM_get( $new_root_path, 'scms_data');
ENV_say( 1, "Current SCMS      : $new_scms",
"Current Repository: $new_scms_repository");




my $must_write_repos = 0;
if ($new_scms_repository eq '')
{
my @repos;
foreach my $osname (SYSTEM_get_osnames( $new_root_path, 'scms_repository'))
{
my $repos = SYSTEM_get_os( $new_root_path, $osname, 'scms_repository');
push @repos, $repos
if ($repos);
}
if (@repos)
{
foreach my $repos (@repos)
{
ENV_say( 1, "SCM REPOSITORY: $repos");
if (ASK_YNQ( 'OK?', 'Y', 1) eq 'Y')
{
$new_scms_repository = $repos;
last;
}
}
$new_scms_repository = ASK_text( 'Enter Repository', '', 0)
if ($new_scms_repository eq '');
} else
{
$new_scms_repository = '';
}
$must_write_repos = 1;
}




my @scms_items = ($new_scms, $new_scms_repository, $new_scms_data);
SCM_preset( $new_root_path, [ @scms_items ]);
GBSSCM_setenv( [ @scms_items ]);
GBSSCM_connect();




if ($must_write_repos)
{
SYSTEM_put_os( $new_root_path, '.', scms_repository => $new_scms_repository);
SYSTEM_write();	   # Includes backup and SCM_checkout
}




my @new_files;
push @new_files, FCREATE_switch( 'switch.gbs', $new_root_path, $system_name);
SCM_add_txt_files( \@new_files, 0, 0);	    		# $prevalidated, $verbose
ENV_say( 1, "Root '$new_root_path' adjusted for this Platform");


return $new_root_path;
}




sub main_new_subsys($)
{
my ($spec_new_subsys) = @_;
my $new_subsys;

ENV_say( 1, "New SubSys: $spec_new_subsys");

VALIDATE_root();
VALIDATE_administrator();




my $gbs_dev_path = "$GBS_ROOT_PATH/dev";
my @subsystems = GBSGLO_subsystems();
ENV_say( 1, "Current SubSystems:");
if (@subsystems)
{
ENV_say( 0, FORMAT_indent( 0, 2, [ 0, "@subsystems" ]));
} else
{
ENV_say( 0, '  None');
}

$new_subsys = ASK_dir( "Enter New SubSystem-name", $spec_new_subsys, $gbs_dev_path, 0, 0,
[ \&check_subsys, [ @subsystems ] ]);
$spec_new_subsys = '';

if ($new_subsys ne "")
{




my ($subsys_type, $import_dir, $export_dir, $res_sub_dir,
$targets_ref, $audits_ref, $extra_dirs) = ask_new_subsys_data( $new_subsys);

ENV_say( 1, "Creating SubSystem '$new_subsys'...");
GBSSCM_preset( 0);
GBSSCM_connect();

my $gbs_subsys_path = "$gbs_dev_path/$new_subsys";
ENV_mkdir( $gbs_subsys_path);





my $ssprop_filespec = SSPROP_set_type( $GBS_ROOT_PATH, $new_subsys, $subsys_type);




my @new_dirs;
push @new_dirs, $gbs_subsys_path;	    # already created
push @new_dirs, FILES_mandatory_dirs( $gbs_subsys_path, 'SUBSYS');
push @new_dirs, map { "$gbs_subsys_path/build/$_" } @{$targets_ref};
push @new_dirs, map { "$gbs_subsys_path/audit/$_" } @{$audits_ref};
push @new_dirs, "$gbs_subsys_path/import"
if ($import_dir);
push @new_dirs, "$gbs_subsys_path/export"
if ($export_dir);
push @new_dirs, "$GBS_ROOT_PATH/res/$new_subsys"
if ($res_sub_dir);

if ($subsys_type ne 'GBS')
{
push @new_dirs, "$gbs_subsys_path/app";
if ($subsys_type eq 'make')
{
if ($extra_dirs)
{
push @new_dirs, map { "$gbs_subsys_path/app/$_" } qw( src inc bld );
push @new_dirs, map { "$gbs_subsys_path/app/bld/$_" } @{$targets_ref};
}
}
}
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 0);	    # $pre_validated, $verbose




my @new_files;
push @new_files, $ssprop_filespec;	    # already created
if ($subsys_type ne 'GBS')
{



push @new_files, FCREATE_export( 'export.gbs', $GBS_ROOT_PATH, $new_subsys, undef);
push @new_files, FCREATE_subsys_script( 'gbssub', $GBS_ROOT_PATH, $new_subsys, $subsys_type);
if ($subsys_type eq 'make')
{
push @new_files, FCREATE_makefile( $GBS_ROOT_PATH, $new_subsys);
}
}
SCM_add_txt_files( \@new_files, 0, 0);			# $prevalidated, $verbose




{
my @gbs_all_subsystems = LIST_unique( [ sort ( ENV_getenv( 'GBS_ALL_SUBSYSTEMS'), $new_subsys ) ]);
ENV_setenv( GBS_ALL_SUBSYSTEMS => \@gbs_all_subsystems);

}

ENV_say( 1, "SubSystem '$new_subsys' created");
$GBS_SUBSYS = $new_subsys;
} else
{
ENV_sig( W => "No new SubSystem created");
}

return $new_subsys;
}




sub ask_new_subsys_data($)
{
my ($new_subsys) = @_;
my ($subsys_type, $import_dir, $export_dir, $res_sub_dir, @targets, @audits, $extra_dirs);

my @subsystems = GBSGLO_subsystems();




my @subsys_type_refs = SSPROP_get_all_types();

$subsys_type = 'GBS';	    	# GBS MSVS make Other
$import_dir = 'N';
$export_dir = 'Y';
$res_sub_dir = (@subsystems) ? 'Y' : 'N';
@targets = ENV_getenv( 'GBS_TARGETS');
@audits = ENV_getenv( 'GBS_AUDITS');
$extra_dirs = 'N';

do
{



$subsys_type = ASK_value_from_menu( 'SubSystem Type', \$subsys_type, undef, [ @subsys_type_refs ]);




$import_dir = ASK_YN( "Import directory required?", $import_dir)
if ($subsys_type eq 'GBS');




$export_dir = ASK_YN( "Export directory required?", $export_dir);




$res_sub_dir = ASK_YN( "Res SubSystem directory required?", $res_sub_dir);




{
my @all_targets = ENV_getenv( 'GBS_ALL_TARGETS');
@targets = ASK_values_from_list( "Targets for SubSystem '$new_subsys'", "@targets", \@all_targets, 0);
@audits = ASK_values_from_list( "Audits for SubSystem '$new_subsys'",   "@audits", \@audits, 0)
if (@audits);
}

if ($subsys_type eq 'make')
{



$extra_dirs = ASK_YN( "Create App Sub directories (app/src, app/inc, app/bld/<target>, ..)?", $extra_dirs);
}




ENV_say( 1, "Summary for new SubSys '$new_subsys':");
ENV_say( 0, "  Type        : $subsys_type");
ENV_say( 0, "  Import-dir  : $import_dir") if ($subsys_type eq 'GBS');
ENV_say( 0, "  Export-dir  : $export_dir");
ENV_say( 0, "  Res-Sub-dir : $res_sub_dir");
ENV_say( 0, "  Target(s)   : @targets");
ENV_say( 0, "  Audit(s)    : @audits");
ENV_say( 0, "  App-Sub-dirs: $extra_dirs")
if ($subsys_type eq 'make');
} while (ASK_YNQ( 'OK?', 'Y', 1) eq 'N');

$import_dir = ($import_dir eq 'Y' && $subsys_type eq 'GBS');
$export_dir = ($export_dir eq 'Y');
$res_sub_dir = ($res_sub_dir eq 'Y');
$extra_dirs = ($extra_dirs eq 'Y');
return ($subsys_type, $import_dir, $export_dir, $res_sub_dir, [ @targets ], [ @audits ], $extra_dirs);
}




sub check_subsys($$$)
{
my ($values_ref,
$default_ref,
$existing_subsys_ref,
) = @_;
my $error_txt = '';

my $subsys = $values_ref->[0];
if (grep( $subsys eq $_, @{$existing_subsys_ref}))
{
$error_txt = "SubSystem '$subsys' already exists";
}

return $error_txt;
}




sub main_new_component($)
{
my ($spec_new_component) = @_;
my $new_component;

ENV_say( 1, "New Component: $spec_new_component");

VALIDATE_root();
VALIDATE_subsys_full_gbs();
VALIDATE_administrator();

GBSSCM_preset( 0);

my $gbs_subsys_path = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS";
my $gbs_comp_path = "$gbs_subsys_path/comp";

my @current_comps = GBSGLO_components( $GBS_SUBSYS);
ENV_say( 1, "Current Components:");
if (@current_comps)
{
ENV_say( 0, FORMAT_indent( 0, 2, [ 0, "@current_comps" ]));
} else
{
ENV_say( 0, '  None');
}
my @all_targets = GBSGLO_subsystem_targets( $GBS_SUBSYS);
my @all_audits = GBSGLO_subsystem_audits( $GBS_SUBSYS);
my $default_targets = join( ',', @all_targets);
my $default_audits = join( ',', @all_audits);

my @comp_refs;

my $comp;
do
{



$comp = ASK_dir( "Enter New Component-name", $spec_new_component, $gbs_comp_path, 0, 0,
[ \&check_component, [ @current_comps ] ]);
$spec_new_component = '';
if ($comp ne "")
{



my @targets;
my @scope_comps;
my @audits;




@targets = ASK_values_from_list( "  Select Targets for Component '$comp'",
$default_targets, \@all_targets, 0);
$default_targets = join( ',', @targets);




if (@current_comps)
{
@scope_comps = ASK_values_from_list( "  Select Components in scope for Component '$comp'",
"", [ @current_comps ], 0);

}




if (@all_audits)
{
@audits = ASK_values_from_list( "  Select Audits for Component '$comp'",
$default_audits, \@all_audits, 0);
$default_audits = join( ',', @audits);
}

push @current_comps, $comp;
push @comp_refs, [ $comp, [ @targets ], [ @scope_comps ], [ @audits ] ];
}
} while ($comp ne '' && ASK_YNQ( "Add another Component?", 'N', 1) eq 'Y');

if (@comp_refs)
{
GBSSCM_connect();
foreach my $ref (@comp_refs)
{
my ($comp_name, $targets_ref, $scopes_ref, $audits_ref) = @{$ref};
ENV_say( 1, "Creating Component: $comp_name");




my $component_path = "$gbs_comp_path/$comp_name";
my @new_dirs;
push @new_dirs, $component_path;
push @new_dirs, FILES_mandatory_dirs( $component_path, 'COMPONENT');
push @new_dirs, map { "$component_path/bld/$_" } @{$targets_ref};
push @new_dirs, map { "$component_path/aud/$_" } @{$audits_ref};
foreach my $target (@{$targets_ref})
{
push @new_dirs, map { "$component_path/aud/$_/$target" } @{$audits_ref};
}
SCM_add_dirs( \@new_dirs, \&FILES_get_ignore_spec, 0, 0);	    # $pre_validated, $verbose




my $scope_file = FCREATE_scope( 'scope.gbs', $GBS_ROOT_PATH, $GBS_SUBSYS,
$comp_name, $scopes_ref);
SCM_add_txt_files( $scope_file, 0, 0);		# $prevalidated, $verbose
}
ENV_say( 1, "". @comp_refs . " Component(s) created");




$new_component = $comp_refs[0]->[0];
$GBS_COMPONENT = $new_component;

} else
{
ENV_sig( W => "No new Components created");
$new_component = "";
}
return $new_component;
}




sub check_component($$$)
{
my ($values_ref,
$default_ref,
$existing_comps_ref,
) = @_;
my $error_txt = '';

my $comp = $values_ref->[0];
if (grep( $comp eq $_, @{$existing_comps_ref}))
{
$error_txt = "Component '$comp' already exists";
} elsif (grep( $comp eq $_, qw( ALL RES EXPORT)))
{
$error_txt = "'ALL', 'RES' and 'EXPORT' are reserved names";
} else
{
;
}

return $error_txt;
}




sub main_new_target($)
{
my ($spec_new_target) = @_;	    # '' == ASK
my $new_target;

VALIDATE_root();
VALIDATE_administrator();

GBSSCM_preset( 0);

$new_target = PLUGINMAINT_new_target( $spec_new_target);

if ($new_target)
{
$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
@GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
}

return $new_target;
}




sub main_remove_target($)
{
my ($default_target) = @_;
my $target_to_remove;

VALIDATE_root();
VALIDATE_administrator();

GBSSCM_preset( 0);

$target_to_remove = PLUGINMAINT_remove_target( $default_target);

if ($target_to_remove)
{
$GBS_TARGET = ENV_getenv( 'GBS_TARGET');
@GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
}

return $target_to_remove;
}




sub main_new_audit($)
{
my ($spec_new_audit) = @_;	    # '' == ASK
my $new_audit;

VALIDATE_root();
VALIDATE_administrator();

GBSSCM_preset( 0);

$new_audit = PLUGINMAINT_new_audit( $spec_new_audit);

if ($new_audit)
{
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
@GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');
}

return $new_audit;
}




sub main_remove_audit($)
{
my ($default_audit) = @_;
my $audit_to_remove;

VALIDATE_root();
VALIDATE_administrator();

GBSSCM_preset( 0);

$audit_to_remove = PLUGINMAINT_remove_audit( $default_audit);

if ($audit_to_remove)
{
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
@GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');
}

return $audit_to_remove;
}




sub select_via_menu()
{
my $again = 1;
do
{
my @menu_refs = (
[ 'Exit',					    undef ],
[ 'Create a new Root/System',		    \&main_new_system ],
[ 'Adjust existing Root for this Platform',	    \&main_new_other_system ],
);
push @menu_refs,
[ 'Create a new SubSys',			    \&main_new_subsys ]
if ($GBS_ROOT_PATH ne '');
push @menu_refs,
[ 'Create a new Component',			    \&main_new_component ]
if ($GBS_SUBSYS ne '' && GBSGLO_subsystem_is_full_gbs( $GBS_SUBSYS));
push @menu_refs,
[ 'Create a new Target',			    \&main_new_target ]
if ($GBS_ROOT_PATH ne '');
push @menu_refs,
[ 'Remove a Target',			    \&main_remove_target ]
if ($GBS_ROOT_PATH ne '');
push @menu_refs,
[ 'Create a new Audit',			    \&main_new_audit ]
if ($GBS_ROOT_PATH ne '');
push @menu_refs,
[ 'Remove an Audit',			    \&main_remove_audit ]
if ($GBS_ROOT_PATH ne '');

my $function = ASK_value_from_menu( 'Select Function', 0, undef, [ @menu_refs ]);
if (defined $function)
{
$RESULT = &$function( '');
} else
{
$again = 0;
}
} while ($again);
}



