#===============================================================
#
#   gbssetup.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright © 2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSETUP @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use 5.016_003;
use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use glo::ask;
use glo::list;
use glo::slurp;
use glo::file;
use glo::format;
use glo::shell;
use gbsenv;
use profile;
use settings;
use setup;
use gbscmd;
use bootstrap;




sub check_essentials();
sub main_profile($$$);
sub store_settings_and_exit();
sub do_settings($);
sub do_change($);
sub handle_scripts_path($$$$);
sub handle_rel($$$$);
sub handle_site($$$$);
sub handle_log_root($$$$);
sub handle_beeps($$$$);
sub handle_proc($$$$);
sub handle_perms($$$$);
sub show_changes($);
sub get_profile_values();
sub get_changed_profile_names();
sub store_changed_profile_values($);
sub check_gbs_root($$$);
sub find_gbs_rels($$);
sub check_command_arg($$$);
sub get_default(@);
sub menu_startup_settings($$$);
sub menu_move_base($$$);
sub check_base_path($$$);
sub write_setup_file();




my $IS_WIN32 = (ENV_is_win32());
my $IS_LUNIX = ! $IS_WIN32;
my $IS_DEVELOPMENT = ENV_is_development();
my $CWD = ENV_cwd();

my $GBS_INITIAL_SETUP = ENV_getenv_bool( 'GBS_INITIAL_SETUP');	# Called from _gbssetup.bat/sh
my $GBS_BOOT_PATH = ENV_getenv_perl_path( 'GBS_BOOT_PATH');	# %APPDATA%/.gbs or ~/.gbs
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	# %APPDATA%/.gbs/base or ~/.gbs/base or other
ENV_sig( F => "GBS_BASE_PATH not defined")
if ($GBS_BASE_PATH eq '');
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();




my %MAN = (
m => 'Mandatory',
o => 'Optional',
);

my %ENV_DEFS;




my @ENVS_ORDER;

{
my @env_def_refs = (



[ GBS_SCRIPTS_ROOT	=> [ 'm', 2, 'Scripts Root',	\&handle_scripts_path ] ],
[ GBS_SCRIPTS_REL	=> [ 'm', 2, 'Version',		\&handle_rel ] ],
[ GBS_SITE        	=> [ 'm', 1, 'Site',		\&handle_site ] ],
[ GBS_LOG_ROOT   	=> [ 'm', 1, 'LogRoot',		\&handle_log_root ] ],
[ GBS_EDITOR     	=> [ 'o', 0, 'Editor',		\&handle_proc ] ],
[ GBS_BROWSER     	=> [ 'o', 0, 'Browser',		\&handle_proc ] ],
[ GBS_VIEWER     	=> [ 'o', 0, 'Vieuwer',		\&handle_proc ] ],
[ GBS_BEEPS     	=> [ 'o', 0, 'Beeps',		\&handle_beeps ] ],
[ GBS_ADMINISTRATOR     => [ 'o', 0, 'Administrator',	\&handle_perms ] ],
[ GBS_INTEGRATOR        => [ 'o', 0, 'Integrator',	\&handle_perms ] ],
);
%ENV_DEFS = map { $_->[0] => $_->[1] } @env_def_refs;
@ENVS_ORDER = map { $_->[0] } @env_def_refs;
}

my @BASIC_SETTINGS = qw( GBS_SITE GBS_LOG_ROOT );
my @GENERAL_SETTINGS = qw( GBS_BEEPS GBS_EDITOR GBS_BROWSER GBS_VIEWER );
my @SPECIAL_SETTINGS = qw( GBS_ADMINISTRATOR GBS_INTEGRATOR );
my @SWITCH_SETTINGS = qw( GBS_SCRIPTS_REL );
my @ADVANCED_SETTINGS = qw( GBS_SCRIPTS_ROOT GBS_SCRIPTS_REL );




my %ENV_DATA = map { $_ => [ undef, undef, 0 ] } @ENVS_ORDER;


my @NEW_SETUP_REFS;

my $HIGHEST_RESTART_LEVEL = 0;







$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>', 'menu_entry_1', "isor1..2", 0, "Primary Menu entry for immediate execution" ],
[ '<2>', 'menu_entry_2', "iso>0",    0, "Secondary Menu entry for single immediate execution" ],
);
GENOPT_set_optdefs( 'gbssetup', \@genopts,
"Setup/Maintain Basic GBS Profile settings (EnvVars).",
undef);
GENOPT_parse();
my @MENU_ENTRIES = grep( $_ != 0, map { scalar GENOPT_get( "menu_entry_$_") } 1..2);




{



SETUP_check_first_invocation();	# check SITE and LOG_ROOT




get_profile_values();




check_essentials();	# ROOT, REL, SITE, LOG_ROOT

if ($GBS_INITIAL_SETUP)
{
store_settings_and_exit();
} else
{




my @main_menu_items = (
[ 'Change Profile Settings',			\&main_profile ],
[ "Fix Startup Settings (Shortcuts)",		\&menu_startup_settings ],
[ "Move GBS_BASE_PATH",				\&menu_move_base ],
);
ASK_menu( 'Select Primary function to perform', \@main_menu_items, [ @MENU_ENTRIES ]);

}

PROFILE_close( 0);	# $must_write
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 1);
}





sub check_essentials()
{
my @missing_essentials;
do
{
@missing_essentials = ();
foreach my $env_name (@ENVS_ORDER)
{
my $man = $ENV_DEFS{$env_name}->[0];
if ($man eq 'm')
{
my $new_value = $ENV_DATA{$env_name}->[1];
if ($new_value eq '')
{
push @missing_essentials, $env_name
if ($env_name eq 'GBS_SCRIPTS_REL' && ENV_getenv( 'GBS_SCRIPTS_REL' ne 'beta'));
}
}
}

if (@missing_essentials)
{
ENV_beep();
ENV_say( 1, "Some essential data is missing. You need to supply that first:",
'');
do_settings( \@missing_essentials);
store_changed_profile_values( [ get_changed_profile_names() ]);
}
} while (@missing_essentials);
}




sub main_profile($$$)
{
my ($entry_nr,
$args_ref,
$entries_ref,
) = @_;

my @profile_menu_items = (
[ 'Basic Settings (Site, LogDir)',		sub { do_settings( \@BASIC_SETTINGS) } ],
[ 'General Settings (Beeps, Editor, ...)',	sub { do_settings( \@GENERAL_SETTINGS) } ],
[ 'Special Settings (Admin, Intgrtr)',		sub { do_settings( \@SPECIAL_SETTINGS) } ],
[ 'Switch GBS Version',				sub { do_settings( \@SWITCH_SETTINGS) } ],
[ 'Advanced Settings (Scripts, Rel)',		sub { do_settings( \@ADVANCED_SETTINGS) } ],
[ 'All Settings',				sub { do_settings( \@ENVS_ORDER) } ],
);
ASK_menu( 'Select function to perform', \@profile_menu_items, $entries_ref);

store_settings_and_exit();
}




sub store_settings_and_exit()
{



check_essentials();	# ROOT, REL, SITE, LOGROOT
my @changed_names = get_changed_profile_names();




if (@changed_names)
{
ENV_say( 1, @changed_names . ' item(s) changed');
show_changes( \@changed_names);
if (ASK_YN( '*** Implement Changes?', 'N') eq 'Y')
{
store_changed_profile_values( \@changed_names);
} else
{
@changed_names = ();
}
}

if (@changed_names)
{



if ($ENV_DATA{GBS_SCRIPTS_ROOT}->[1] ne $ENV_DATA{GBS_SCRIPTS_ROOT}->[0] ||	    # $new_value, $old_value
$ENV_DATA{GBS_SCRIPTS_REL}->[1]  ne $ENV_DATA{GBS_SCRIPTS_REL}->[0] )	    # $new_value, $old_value
{
SETTINGS_set_check( $ENV_DATA{GBS_SCRIPTS_ROOT}->[1], $ENV_DATA{GBS_SCRIPTS_REL}->[1]); # new_value
}




write_setup_file();
ENV_exit();
} else
{
ENV_say( 1, "Nothing changed");
}
}




sub do_settings($)
{
my ($envvar_names_ref) = @_;




my @refs;
foreach my $name (@{$envvar_names_ref})
{
my $stored_value = $ENV_DATA{$name}->[0];

{
my $man = $MAN{ $ENV_DEFS{$name}->[0] };
push @refs, [ $man, $name, $stored_value ];
}
}
ENV_say( 1, "Values: ");
ENV_say( 0, FORMAT_table( 0, 2, '  ', undef, @refs));




ENV_say( 1, "Change:");
foreach my $name (@{$envvar_names_ref})
{
my $continue = do_change( $name);
last
if (!$continue);
}
}




sub do_change($)
{
my ($name) = @_;	    # GBS_...
my $continue = 1;	    # FALSE if $ans was 'E'


my ($man, undef, $nice_name, $function_ref) = @{$ENV_DEFS{$name}};
my ($stored_value, $new_value) = @{$ENV_DATA{$name}};
my $env_value = ENV_getenv( $name);

ENV_say( 1, "$name    ($MAN{$man})");
ENV_say( 0, "  EnvVar : $env_value")
if ($env_value ne '');
ENV_say( 0, "  Current: $stored_value");
ENV_say( 0, "  New    : $new_value");

my $default;
my $ans;
if ($man eq 'o')	# optional
{
$default = 'N';
$ans = ASK_YNEQ( 'Change?', $default, 0);
} else			# mandatory
{
$default = ($stored_value eq '' && $new_value eq '') ? 'Y' : 'N';
$ans = ASK_YNQ( 'Change?', $default, 0);
}
if ($ans eq 'Y' || $ans eq 'N')
{
my $again;
do
{
$again = 0;
if ($ans eq 'Y')
{



$new_value = $function_ref->( $name, $stored_value, $new_value, $env_value);
$ENV_DATA{$name}->[1] = $new_value;
}
if ($man ne 'o' && $new_value eq '')
{
ENV_beep();
ENV_say( 1, "** Value required!");
$again = 1;
}
} while ($again);
} elsif ($ans eq 'E')
{
$continue = 0;
} else	# $ans eq 'Q'
{
ENV_say( 1, "Terminated - Nothing changed");
ENV_exit();
}

return $continue;
}




sub handle_scripts_path($$$$)
{
my ($name,
$stored_value,
$new_value,
$env_value,
) = @_;
my $new_scripts_path;

my $default = ENV_parent_path( $GBS_SCRIPTS_PATH);

$new_scripts_path = ASK_path( $name, $default, 1, -1, [ \&check_gbs_root, undef ] );

if (!$stored_value || $new_scripts_path ne $stored_value)
{
my $rel_part = ($IS_WIN32) ? '%GBS_SCRIPTS_REL%' : '$GBS_SCRIPTS_REL';
my $gbs_scripts = ENV_os_path( "$new_scripts_path/$rel_part");
$ENV_DATA{GBS_SCRIPTS_PATH}->[0] = ''
if (!defined $ENV_DATA{GBS_SCRIPTS_PATH}->[0]);	    # $stored_value
$ENV_DATA{GBS_SCRIPTS_PATH}->[1] = $gbs_scripts;	    # $new_value
ENV_say( 1, "GBS_SCRIPTS_PATH set to '$gbs_scripts'");
}

return ENV_os_path( $new_scripts_path);
}




sub handle_rel($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $new_rel;

my $gbs_scripts_root = $ENV_DATA{GBS_SCRIPTS_ROOT}->[1];	# $new_value
my @gbs_rels = find_gbs_rels( $gbs_scripts_root, 0);	# $include_beta = 0;
if (@gbs_rels == 0)
{
ENV_sig( F => "No GBS Release directories found in $gbs_scripts_root");
} elsif (@gbs_rels == 1)
{
$new_rel = $gbs_rels[0];
ENV_say( 1, "Single GBS Release directory '$new_rel' taken");
} else
{
my $default_i = -1;
$default_i = LIST_firstidx_str( $new_value, \@gbs_rels) if ($new_value);
$default_i = LIST_firstidx_str( $env_value, \@gbs_rels) if ($env_value);
$default_i = LIST_firstidx_str( $stored_value, \@gbs_rels) if ($stored_value);
my $new_rel_i = ASK_index_from_menu( $name, $default_i, undef, [ @gbs_rels ]);
$new_rel = $gbs_rels[$new_rel_i];
}

return $new_rel;
}




sub handle_site($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;


my $default = get_default( $new_value, $stored_value, $env_value);

return SETUP_ask_site( $default);
}




sub handle_log_root($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;


my $default = get_default( $new_value, $stored_value, $env_value);

return SETUP_ask_log_root( $default);
}




sub handle_beeps($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $default = get_default( $new_value, $stored_value, $env_value, 'OFF');
$default = uc $default;
$default = ($default eq 'YES' || $default eq 'ON') ? 'N' : 'Y';
my $ans = ASK_YN( 'Enable Beeps?', $default);
$newer_value = ($ans eq 'Y') ? 'YES' : 'NO';

return $newer_value ;
}




sub handle_proc($$$$)
{
my ($name,		    # ignored
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $prompt_name = ucfirst( lc( substr( $name, 4)));




my $default = get_default( $new_value, $stored_value, $env_value);
my $default_exec = '';
my @default_args = ();
($default_exec, @default_args) = ENV_split_quoted_space( $default) if ($default);
my $default_args = "@default_args";

my $exec = ASK_filespec( "Enter exec path for $prompt_name", $default_exec, 1, 0);
$exec = ENV_os_path( $exec);
if ($exec ne '')
{
$exec = "\"$exec\"" if ($exec =~ /\s/);

my $again;
my $param = ($IS_WIN32) ? '%1%' : '$1';
$default_args = $param if (!$default_args);
my $arg = ASK_text( "Argument(s) with '$param' to specify file", $default_args,
-2, [ \&check_command_arg, $param ]);
$newer_value = "$exec $arg";
} else
{
$newer_value = '';
}

return $newer_value;
}




sub handle_perms($$$$)
{
my ($name,		    # GBS_INTEGRATOR or GBS_ADMINISTRATOR
$stored_value,
$new_value,
$env_value,
) = @_;
my $newer_value;

my $prompt = ($name eq 'GBS_INTEGRATOR') ? 'Integrator' : 'Administrator';
my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
my $default = get_default( $new_value, $stored_value, $env_value);
my @perms = split( ',', $default);
if (grep $gbs_system_name eq $_, @perms)
{
ENV_say( 1, "You are already $prompt");
my $ans = ASK_YN( 'Disable?', 'N');
if ($ans eq 'Y')
{
@perms = grep $gbs_system_name ne $_, @perms;
}
} else
{
ENV_say( 1, "You are not $prompt");
my $ans = ASK_YN( 'Enable?', 'N');
if ($ans eq 'Y')
{
push @perms, $gbs_system_name;
}
}
$newer_value = join( ',', @perms);

return $newer_value ;
}




sub show_changes($)
{
my ($changed_names_ref,
) = @_;

my @line_refs;
foreach my $name (@{$changed_names_ref})
{
push @line_refs, [ $name, $ENV_DATA{$name}->[1] ];	# $new_value
}

ENV_say( 1, "Changes: ");
ENV_say( 0, FORMAT_table( 0, 2, ' => ', undef, @line_refs));
}




sub get_profile_values()
{



ENV_say( 1, "Read Profile...");
my $exists = PROFILE_open( $GBS_BASE_PATH);
foreach my $name (@ENVS_ORDER)
{
my $value = PROFILE_get( $name);
$ENV_DATA{$name}->[0] = $value;     # $stored_value
$ENV_DATA{$name}->[1] = $value;     # $new_value

}




if ($GBS_INITIAL_SETUP)
{
$GBS_SCRIPTS_PATH = $CWD;
my $gbs_scripts_root = ENV_perl_path_nospace( ENV_parent_path( $CWD));
my $gbs_scripts_rel = ENV_parent_dir( $CWD, -1);
if ($ENV_DATA{GBS_SCRIPTS_ROOT}->[1] ne $gbs_scripts_root)
{
$ENV_DATA{GBS_SCRIPTS_ROOT}->[1] = $gbs_scripts_root;		# $new_value
ENV_say( 1, "GBS_SCRIPTS_ROOT set to $gbs_scripts_root");
}
if ($ENV_DATA{GBS_SCRIPTS_REL}->[1] ne $gbs_scripts_rel)
{
$ENV_DATA{GBS_SCRIPTS_REL}->[1] = $gbs_scripts_rel;			# $new_value
ENV_say( 1, "GBS_SCRIPTS_REL  set to $gbs_scripts_rel");
} else
{
ENV_say( 1, "GBS_SCRIPTS_REL already $gbs_scripts_rel");
}
}
}





sub get_changed_profile_names()
{
my @names;


foreach my $name (@ENVS_ORDER)
{
push @names, $name
if ($ENV_DATA{$name}->[0] ne $ENV_DATA{$name}->[1]);    # $stored_value , $new_value;
}

return @names;
}




sub store_changed_profile_values($)
{
my ($changed_names_ref,
) = @_;

if (@{$changed_names_ref})
{
ENV_say( 1, "Store Profile Changes...");




foreach my $name (@{$changed_names_ref})
{
my $data_ref = $ENV_DATA{$name};
my ($stored_value, $new_value) = @{$data_ref};
PROFILE_set( $name => $new_value);
$data_ref->[1] = $new_value;	# $stored_value
my $restart_level = $ENV_DEFS{$name}->[1];
$HIGHEST_RESTART_LEVEL = $restart_level
if ($restart_level > $HIGHEST_RESTART_LEVEL);
push @NEW_SETUP_REFS, [ $name, $new_value ];
}

PROFILE_write();
}
}




sub check_gbs_root($$$)
{
my ($values_ref,
$default_ref,
$args_ref,
) = @_;
my $error_txt = '';

my $path = $values_ref->[0];

my @gbs_rels = find_gbs_rels( $path, 1);	# $include_beta = 1
$error_txt = "'$path' is not a valid GBS_SCRIPTS_PATH"
if (!@gbs_rels);

return $error_txt;
}




sub find_gbs_rels($$)
{
my ($path,
$include_beta,
) = @_;
my @gbs_rels;

my @rels = glob( "$path/*/gbsswr.pl");
@gbs_rels = map { ENV_parent_dir( $_, -2) } @rels;

my @beta_rels = grep( /^beta/, @gbs_rels);
if (@beta_rels)
{
if (!$include_beta && ! $IS_DEVELOPMENT)
{
ENV_say( 2, "@beta_rels ignored");
@gbs_rels = grep( $_ !~ /^beta/, @gbs_rels);
}
}

return @gbs_rels;
}




sub check_command_arg($$$)
{
my ($values_ref,
$default_ref,
$param,
) = @_;
my $error_txt = '';

my $arg = $values_ref->[0];
my $q_param = quotemeta $param;
$error_txt = "'$param' missing"
if ($arg !~ /$q_param/);

return $error_txt;
}





sub get_default(@)
{
my @values = @_;
my $default = '';

foreach my $value (@values)
{
if ($value)
{
$default = $value;
last;
}
}

return $default;
}




sub menu_startup_settings($$$)
{
my ($entry_nr,
$args_ref,
$entries_ref,
) = @_;


SETTINGS_set_check( $ENV_DATA{GBS_SCRIPTS_ROOT}->[1], $ENV_DATA{GBS_SCRIPTS_REL}->[1]); # new_value
}




sub menu_move_base($$$)
{
my ($entry_nr,
$args_ref,
$entries_ref,
) = @_;

ENV_say( 1, "Current GBS_BASE_PATH: $GBS_BASE_PATH");




my $new_location;
if ($IS_WIN32)
{
my $mydocuments_path = ENV_get_user_path( 0);
my $short_mydocuments_path = ENV_perl_paths( ENV_short_path_name( $mydocuments_path));
if ("$short_mydocuments_path/.gbs/base" ne $GBS_BASE_PATH)
{
my @locations = (
"My Documents ($mydocuments_path/.gbs/base)",
'<other>',
);
my $index = ASK_index_from_menu( 'Specify Location to move GBS_BASE_PATH to', -1, undef, [ @locations ]);
if ($index == 0)
{
$new_location = $short_mydocuments_path;
}
}
}
if (!defined $new_location)
{
$new_location = ASK_path( "Enter new location (without '/.gbs/base')", '', 0, 0, [ \&check_base_path ]);

}




if ($new_location ne '')
{
my $new_base_path = "$new_location/.gbs/base";
ENV_say( 1, "New GBS_BASE_PATH: $new_base_path");




if (!-e $new_base_path)
{
if (ASK_YNEQ( 'Location does not exist. Create?', 'Y', 1) eq 'Y')
{
ENV_say( 2, 'Created')
if (ENV_mkpath( $new_base_path, undef, 'EE'));
}
}




if (-e $new_base_path)
{




my %exclude_files = map { $_ => 1 } SETTINGS_get_setup_files( "$GBS_SCRIPTS_PATH/setup");
my $bootstrap_file = BOOTSTRAP_file();  # bootstrap.bat or bootstrap.sh
$exclude_files{$bootstrap_file} = 1;

my $nr_copied = 0;
my @from_filespecs;
foreach my $base_file (SLURP_dir_files( $GBS_BASE_PATH, 1))
{
if (!exists $exclude_files{$base_file})
{
push @from_filespecs, "$GBS_BASE_PATH/$base_file";
my $replicated = FILE_replicate( "$GBS_BASE_PATH/$base_file", "$new_base_path/$base_file");
if ($replicated == 1)
{
ENV_say( 2, "Copied $base_file");
$nr_copied++;
}
}
}
unlink @from_filespecs;
ENV_say( 1, "Moved $nr_copied files");




$new_base_path = BOOTSTRAP_change( $new_base_path);
push @NEW_SETUP_REFS, [ GBS_BASE_PATH => $new_base_path ];





if (SLURP_dir_all( $GBS_BASE_PATH, 0) == 0 &&
ASK_YN( "Delete original location ($GBS_BASE_PATH)", 'Y') eq 'Y')
{
FILE_del_tree( $GBS_BASE_PATH, undef, 5);	    # $verbose_level
if ($GBS_BASE_PATH =~ m!/\.gbs/base$!)
{
FILE_del_dirs( $GBS_BASE_PATH);			# .../.gbs/base
ENV_say( 2, "Deleted $GBS_BASE_PATH");
my $gbs_top_path = $GBS_BASE_PATH;
$gbs_top_path =~ s!/base$!!;




if (!SLURP_dir_all( $gbs_top_path, 1))	    # $include_dot_files
{
FILE_del_dirs( $gbs_top_path);			# .../.gbs
ENV_say( 2, "Deleted $gbs_top_path")
} else
{
ENV_say( 2, "$gbs_top_path not empty: not deleted")
}
}
}
$GBS_BASE_PATH = $new_base_path;




write_setup_file();
ENV_exit();
} else
{
ENV_say( 1, 'Nothing moved');
}
}
}




sub check_base_path($$$)
{
my ($values_ref,	# Value may be changed
$default_ref,
$app_data_ref,	# [ @display_texts_or_refs ]
) = @_;
my $error_txt = '';

$values_ref->[0] =~ s!/\.gbs/base$!!i;
$error_txt = '.gbs not allowed in path'
if ($values_ref->[0] =~ m!\.gbs!i);

return $error_txt;
}




sub write_setup_file()
{
ENV_say( 1, "Set Env. Changes...");



my @lines;
if ($HIGHEST_RESTART_LEVEL == 2)
{



if (GBSENV_gbs_is_active())
{
push @lines, SHELL_echo( "*** GBS Exiting... ***");
push @lines, GBSCMD_get_gbs_command( 'gbsexit');
push @lines, SHELL_echo( "*** GBS Restarting... ***");
push @lines, GBSCMD_get_base_command( 'gbsinit');
}
} else
{



foreach my $ref (@NEW_SETUP_REFS)
{
my ($name, $new_value) = @{$ref};
push @lines, SHELL_setenv( $name => $new_value);
ENV_say( 2, "$name => $new_value");
}
if ($HIGHEST_RESTART_LEVEL == 1)
{



if (ENV_getenv( 'GBS_ROOT_PATH') ne '') # rsw active?
{
push @lines, SHELL_echo( "*** Restarting the current Root...");
push @lines, GBSCMD_get_gbs_command( 'gbsswr', '.');
}
}
}

GBSENV_write_result_script( \@lines, 0);
}


