#==============================================================
#
#   gbsshow.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSHOW @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::genopt;
use glo::format;
use glo::build;
use glo::check;
use system;
use gbsenv;




sub show_currencies();
sub sanity_check();




my $BRIEF;

my $GBS_ROOT_PATH = GBSENV_gbs_root_path();






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ 'brief', 'brief',	   'bso',   0, "Brief (short) listing" ],
);
GENOPT_set_optdefs( [ 'gbsshow', 'gbs' ], \@genopts,
'Show GBS currencies',
undef);
GENOPT_parse();
$BRIEF = GENOPT_get( 'brief');

{



ENV_say( 1, '');
ENV_say( 0, show_currencies());




sanity_check();
}




ENV_exit( $RC);




END
{
ENV_print_end_msg( 0);
}




sub show_currencies()
{
my @lines;




if (!$BRIEF)
{
push @lines, "  Settings:";
my @row_refs;




my $gbs_show_version = BUILD_get_show_version();
my $gbs_platform = ENV_getenv( 'GBS_PLATFORM');
push @row_refs, [ 'GBS        ', ': ', "Version: $gbs_show_version on $gbs_platform" ];




my ($lics, $licn, $lice, $licd) = BUILD_get_lic();
push @row_refs, [ 'License', ': ', "$licn ($lics)" ];
push @row_refs, [ '',        ': ', "Expires: $lice. Remaining days: $licd" ];
if ($licd < 0)
{
push @row_refs, [ '',        ': ', "** Expired **" ];
ENV_sig( W => 'License expired!');
}




my $GBS_SITE = ENV_getenv( 'GBS_SITE');
push @row_refs, [ 'Site', ': ', $GBS_SITE ];




my @roles;
push @roles, 'Integrator' if (GBSENV_is_integrator());
push @roles, 'Administrator' if (GBSENV_is_administrator());
push @row_refs, [ 'Role(s)', ': ', "@roles" ];




if ($GBS_ROOT_PATH ne '')
{
my $gbs_scms = ENV_getenv( 'GBS_SCMS');
my $gbs_scms_repository = ENV_getenv("GBS_SCMS_REPOSITORY");
my $gbs_scms_data = ENV_getenv("GBS_SCMS_DATA");
my $text = $gbs_scms;
$text .= ": $gbs_scms_repository" if ($gbs_scms_repository ne '');
$text .= "($gbs_scms_data)" if ($gbs_scms_data ne '');
push @row_refs, [ 'SCMS', ': ', $text ];
}

push @lines, FORMAT_table( 0, 4, '', undef, @row_refs);
}




my @row_refs;
push @lines, "  Currencies:";

if ($GBS_ROOT_PATH ne '')
{



my $gbs_system_name = ENV_getenv( 'GBS_SYSTEM_NAME');
if ($BRIEF)
{
push @row_refs, [ 'System Name', ': ', "$gbs_system_name" ];
} else
{
my $build_start = SYSTEM_get( $GBS_ROOT_PATH, 'build_start');
my $build_end = SYSTEM_get( $GBS_ROOT_PATH, 'build_end');
push @row_refs, [ 'System Name', ': ', "$gbs_system_name [$build_start - $build_end]" ];
}




if ($BRIEF)
{
push @row_refs, [ 'Root', ': ', "$GBS_ROOT_PATH" ];
} else
{
my $gbs_root_version = ENV_getenv( 'GBS_ROOT_VERSION');
push @row_refs, [ 'Root', ': ', "$GBS_ROOT_PATH [$gbs_root_version]" ];
}

{
my @item_refs;

my $longest_length = 0;




my $gbs_subsys = ENV_getenv( 'GBS_SUBSYS');
my $ss_type = ENV_getenv( 'GBS_SSTYPE');
if ($gbs_subsys ne '')
{
my $both_length = length( $gbs_subsys) + length( $ss_type);
$longest_length = $both_length
if ($both_length > $longest_length);
push @item_refs, [ SubSystem => $gbs_subsys, $ss_type, $both_length ];
} else
{
push @item_refs, [ SubSystem => '' ];
}




if ($ss_type eq 'GBS')
{
my $gbs_component = ENV_getenv( 'GBS_COMPONENT');
push @item_refs, [ Component => $gbs_component ];
}




foreach my $plugin_type (qw( Target Audit Tool))
{
my $uc_plugin_type = uc $plugin_type;
my $gbs_plugin_type = ENV_getenv( "GBS_$uc_plugin_type");
if ($gbs_plugin_type ne '')
{
my $gbs_plugin_type_plugin = ENV_getenv( "GBS_${uc_plugin_type}_PLUGIN");
my $both_length = length( $gbs_plugin_type) + length( $gbs_plugin_type_plugin);
$longest_length = $both_length
if ($both_length > $longest_length);
push @item_refs, [$plugin_type => $gbs_plugin_type, $gbs_plugin_type_plugin, $both_length ];
} else
{
push @item_refs, [$plugin_type => '' ]
if (!$BRIEF || $plugin_type eq 'Target');
}
}




foreach my $ref (@item_refs)
{
my ($title, $name, $data, $both_length) = @{$ref};
if (defined $data)
{
my $string = "$name         " . (' ' x ($longest_length - $both_length)) . "[$data]";
push @row_refs, [ $title, ': ', $string ];
} else
{
push @row_refs, [ $title, ': ', $name ];
}
}
}
} else
{



push @row_refs, [ 'System Name', ': ', '*** No Current Root ***' ];
}




push @row_refs, [ 'Currenty in', ': ', ENV_cwd() ];

push @lines, FORMAT_table( 0, 4, '', undef, @row_refs);

return @lines;
}




sub sanity_check()
{
my $nr_errors_org = ENV_get_nr_errors();




{



my $gbs_log_root = ENV_getenv_perl_path( 'GBS_LOG_ROOT');
my $error_text = CHECK_path( 'Path', $gbs_log_root, 1);
if ($error_text eq '')
{
$error_text = "Path $gbs_log_root is not writable!"
if (! -w $gbs_log_root);
}
if ($error_text ne '')
{
ENV_sig( E => "GBS_LOG_ROOT ($gbs_log_root)", "- $error_text");
}




my $gbs_site = ENV_getenv_perl_path( 'GBS_SITE');
ENV_sig( E => "'GBS_SITE' not set")
if ($gbs_site eq '');
}

if (ENV_get_nr_errors() > $nr_errors_org)
{
ENV_say( 1, '***');
ENV_say( 1, '*** Before you proceed you MUST run gbssetup and correct these error(s) ***');
ENV_say( 1, '***');
}
}


