#==============================================================
#
#   gbsswc.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWC @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::shell;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swa;
use swb;
use swc;
use swglo;




sub ask_component();




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'component',		'sso',	'', "Component: '' == ASK, '-' == none, '.' == current" ],
[ 'target',	'target',		'sso',	'', "Target: '-' == none, '.' == current" ],
[ 'audit',	'audit',	        'sso',  '', "Audit. '-' == none, '.' == current" ],
[ 'new',	'create_new_component', 'bso',	 0, "Create a new Component" ],
[ 'nocd',	'no_cd',		'bso',   0, "Do not 'cd' to the selected directories "],
);
my @genconflicts = (
[ [ new => 1], '=', '<1>', '=', 'target' ],
[ [ '<1>' => '' ],  '=', 'target', '=', 'audit' ],
);
GENOPT_set_optdefs( [ 'gbsswc', 'swc' ], \@genopts,
'Set and/or Create new Current Component',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $COMPONENT = GENOPT_get( 'component');
my $TARGET = GENOPT_get( 'target');
my $AUDIT = GENOPT_get( 'audit');
my $NEW = GENOPT_get( 'create_new_component');
my $NO_CD = GENOPT_get( 'no_cd');




VALIDATE_subsys_full_gbs();

my $GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";
my @ALL_COMPONENTS = GBSGLO_components( $GBS_SUBSYS);




if (!$NEW && @ALL_COMPONENTS == 0)
{
if (ASK_YN( "No Component(s) defined for this SubSystem. Create now?", 'Y') eq 'Y')
{
$NEW = 1;
}
}




if ($NEW)
{
$COMPONENT = RUN_gbsnew( 'new_component', $COMPONENT);
if ($COMPONENT eq '')
{
$COMPONENT = '.';
$CANCELLED = 1;
}
}

if (!$CANCELLED)
{



if ($COMPONENT eq '')
{
$COMPONENT = ask_component();
}
my $new_component = SWGLO_validate_component( $GBS_SUBSYS, $COMPONENT);
my $new_target = SWGLO_validate_target( $GBS_SUBSYS, $new_component, $TARGET);
my $new_audit = SWGLO_validate_audit( $GBS_SUBSYS, $new_component, $new_target, $AUDIT);





if ($new_component ne $GBS_COMPONENT)
{
my $must_write = ($new_component ne '' || $COMPONENT eq '-');
SWC_set( $GBS_SUBSYS, $new_component, $must_write);
}

if ($new_target ne $GBS_TARGET)
{
SWB_set( $new_target, 1);
$GBS_TARGET = $new_target;
}
if ($new_audit ne $GBS_AUDIT)
{
SWA_set( $new_audit, 1);
$GBS_AUDIT = $new_audit;
}

ENV_sig( W => "No current Component")
if ($new_component eq '' && $COMPONENT ne '-');
ENV_sig( W => "No current Target")
if ($new_target eq '' && $TARGET ne '-');








{
my $new_cwd = ($NO_CD) ? $CWD : SWGLO_new_cwd( $GBS_ROOT_PATH, $GBS_SUBSYS, $new_component);

my @lines;
if (ENV_get_changed_envs() || $new_cwd ne $CWD)
{



push @lines, GBSENV_changed_setenv_commands( 0);




push @lines, SWGLO_set_title( $GBS_ROOT_PATH, $GBS_SUBSYS, $new_component);




push @lines, SHELL_cd( $new_cwd);
}

push @lines, GBSCMD_get_gbs_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




END
{
ENV_print_end_msg( 0);
}




sub ask_component()
{
my $component;




if (scalar @ALL_COMPONENTS == 0)
{
ENV_sig( EE => "No Component(s) defined - use 'swc --new' to define Components");
} elsif (scalar @ALL_COMPONENTS == 1)
{
$component = $ALL_COMPONENTS[0];
ENV_say( 1, "Single Component '$component' taken by default");
} else
{



my @menu_refs = ( [ '', '-' ] );
foreach my $component (@ALL_COMPONENTS)
{
my $is_target = (GBSGLO_component_does_build( $GBS_SUBSYS, $component, $GBS_TARGET)) ? '+' : '-';
my $is_audit = (GBSGLO_component_does_audit( $GBS_SUBSYS, $component, $GBS_AUDIT, $GBS_TARGET)) ? '+' : '-';
my $display_text_ref = [ $component,  "[$is_target$is_audit]" ];
push @menu_refs, [ $display_text_ref, $component ];
}
my $default_component = ENV_getenv( 'GBS_COMPONENT');
my $prompt = "Select Component [$GBS_TARGET,$GBS_AUDIT]";
my $help =   '[<t><a>] -> t = Active for current Target, a = Active for current Audit';
$component = ASK_value_from_menu( [ $prompt, $help ], \$default_component, undef, [ @menu_refs ]);
$component = '-'
if ($component eq '');
}

return $component;
}


