#===============================================================
#
#   gbssysbg.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbssysbg;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSSYSBG_execute
GBSSYSBG_print_errors
);
}




use glo::env;
use glo::slurp;
use glo::banner;
use gbsenv;
use gbsglo;
use exec;
use cleanup;
use sws;




sub GBSSYSBG_execute($$$$$);
sub GBSSYSBG_print_errors();

sub do_sub_build($$$);
sub do_sub_audit($$);
sub do_tool($$);
sub do_audit_sum($);
sub do_command($);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');




my $IGNORE_ERRORS;	    # bool
my $FILES_REF;		    # may be undef




my $FOR_GEN = 0;
my $FOR_MAKE = 0;
my $FOR_AUDIT = 0;
my $FOR_TOOL = 0;
my $ACTION_TEXT;
my $GBS_AUDIT;
my $GBS_TOOL;

my $AUDIT_COUNT;





sub GBSSYSBG_execute($$$$$)
{
my ($jobtype,			# 'gen', 'make', 'audit' or 'tool'
$ignore_errors,			# bool
$files_ref,			# --files= may be undef
$steps_filespec,
$options_ref,
) = @_;
my $rc = 0;

$IGNORE_ERRORS = $ignore_errors;
$FILES_REF = $files_ref;

my @command_options = @{$options_ref};  # --i, --mm, --export, --summary,  etc




$AUDIT_COUNT = 1;
if ($jobtype eq 'gen')
{
$FOR_GEN = 1;
$ACTION_TEXT = 'Generating';
} elsif ($jobtype eq 'make')
{
$FOR_MAKE = 1;
$ACTION_TEXT = 'Making';
} elsif ($jobtype eq 'audit')
{
$FOR_AUDIT = 1;
$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
$ACTION_TEXT = "Auditing $GBS_AUDIT";
} elsif ($jobtype eq 'tool')
{
$FOR_TOOL = 1;
$GBS_TOOL = ENV_getenv( 'GBS_TOOL');
$ACTION_TEXT = "Executing $GBS_TOOL";
} else
{
ENV_sig( F => "Invalid jobtype '$jobtype'");
}
ENV_setenv( GBS_IGNORE_ERRORS => $IGNORE_ERRORS);




my @step_refs = map { [ split( "\t", $_) ] } SLURP_file( $steps_filespec);

unlink $steps_filespec;




{
my @steps = map { $_->[0] } @step_refs;
BANNER_print( 'SYSTEM', "ALL_STEPS: @steps");
}

while (@step_refs)
{



my $step_ref = shift @step_refs;
my ($step_name, $subsys, $tool, $must_force_execute, @commands) = @{$step_ref};

my $must_execute = ($IGNORE_ERRORS || $must_force_execute);
if ($rc == 0 || $IGNORE_ERRORS || $must_force_execute)
{
my $banner_name = ($must_force_execute) ? 'EXPLICIT STEP' : 'STEP';
my $done_text;
my $this_rc;
if ($subsys ne '')
{



BANNER_print( 'SUBSYS', [
"$banner_name: $step_name - SubSystem: $subsys",
"Silent switch to SubSystem: $subsys" ]);
SWS_set( $subsys, 0);
my $subsys_path = "$GBS_ROOT_PATH/dev/$subsys";
ENV_chdir( $subsys_path);




if ($FOR_AUDIT)
{
$this_rc = do_sub_audit( $subsys, \@command_options);
$done_text = "SubSystem $step_name - Target=$GBS_TARGET - Audit=$GBS_AUDIT";
} else
{
$this_rc = do_sub_build( $jobtype, $subsys, \@command_options);
$done_text = "SubSystem $step_name - Target=$GBS_TARGET";
}
} else
{
ENV_chdir( $GBS_ROOT_PATH);
if (@commands)
{
my $nr_commands = @commands;
BANNER_print( 'SUBSYS', "$banner_name: $step_name - $nr_commands commands");
foreach my $command (@commands)
{
$this_rc = do_command( $command);
last
if ($this_rc != 0);
}
$done_text = "Command $step_name";
} else # Tool
{
BANNER_print( 'SUBSYS', "$banner_name: $step_name - Tool: $tool");
$this_rc = do_tool( $tool, \@command_options);
$done_text = "Tool $step_name";
}
}
$rc = $this_rc if ($this_rc > $rc);

if ($this_rc == 0)
{
BANNER_print( 'NOTE', "DONE: $done_text");
} else
{
BANNER_print( 'FAIL', "FAILED: $done_text - RC=$this_rc");
}
if ($this_rc != 0 && !$IGNORE_ERRORS)
{
if (@step_refs)
{
my $nr_steps = @step_refs;
BANNER_print( 'NOTE', "Remaining STEPs ($nr_steps) will be skipped");
} else
{
BANNER_print( 'NOTE', 'No Remaining STEPs');
}
last;
}
}

}




if ($FOR_AUDIT)
{
my $this_rc = do_audit_sum( \@command_options);
$rc = $this_rc if ($this_rc > $rc);
}

return $rc;
}





sub do_sub_build($$$)
{
my ($jobtype,	    # 'gen' or 'make'
$subsys,
$gbs_build_opts_ref,
) = @_;
my $rc = 0;




if ($FOR_GEN)
{
BANNER_print( 'NOTE', "Implicit: Cleanup the bld directories for SubSys: $subsys");
CLEANUP_subsys_bld( $subsys, $GBS_TARGET);
}




if (GBSGLO_subsystem_is_full_gbs( $subsys))
{



my @gbs_build_opts = @{$gbs_build_opts_ref};
ENV_say( 1, "$ACTION_TEXT Full GBS SubSystem: $subsys (@gbs_build_opts)...");
$rc = EXEC_run( "gbs$jobtype.pl", [ @{$FILES_REF}, @gbs_build_opts ]);
} else
{



ENV_say( 1, "$ACTION_TEXT Non-Full GBS SubSystem: $subsys...");




$rc = EXEC_run( 'gbssub.pl', [ $jobtype, $IGNORE_ERRORS ]);
}

return $rc;
}




sub do_sub_audit($$)
{
my ($subsys,
$gbsaudit_opts_ref,
) = @_;
my $rc = 0;




if (GBSGLO_subsystem_is_full_gbs( $subsys))
{



my @gbsaudit_opts = @{$gbsaudit_opts_ref};
ENV_say( 1, "$ACTION_TEXT Full GBS SubSystem: $subsys (@gbsaudit_opts)...");
my @args = ( @{$FILES_REF}, @gbsaudit_opts, "--call=$AUDIT_COUNT");

$rc = EXEC_run( "gbsaudit.pl", \@args);
$AUDIT_COUNT++;
} else
{



ENV_say( 1, "$ACTION_TEXT Non-Full GBS SubSystem: $subsys...");
$rc = EXEC_run( 'gbssub.pl', [ 'audit', $IGNORE_ERRORS ]);
}

return $rc;
}




sub do_audit_sum($)
{
my ($gbsaudit_opts_ref) = @_;
my $rc = 0;





if ($AUDIT_COUNT > 1)
{
ENV_chdir( $GBS_ROOT_PATH);
my @gbsaudit_opts = @{$gbsaudit_opts_ref};
BANNER_print( 'SUBSYS', "IMPLICIT STEP: Create Audit Summary (@gbsaudit_opts)" );

my @args = ('summary', @gbsaudit_opts, "--call=-$AUDIT_COUNT");
$rc = EXEC_batch_run( ENV_command_ref( 'gbsaudit.pl', \@args));
if ($rc == 0)
{
BANNER_print( 'NOTE', "AUDIT SUMMARY: Target=$GBS_TARGET Audit=$GBS_AUDIT");
} else
{
BANNER_print( 'FAIL', "AUDIT SUMMARY FAILED: Target=$GBS_TARGET Audit=$GBS_AUDIT - RC=$rc");
}
}
$AUDIT_COUNT = 1;

return $rc;
}




sub do_tool($$)
{
my ($tool,
$gbstool_opts_ref,
) = @_;
my $rc = 0;

my @gbstool_opts = @{$gbstool_opts_ref};
ENV_say( 1, "$ACTION_TEXT $tool (@gbstool_opts)...");
my @args = ( $tool, @gbstool_opts );

$rc = EXEC_run( "gbstool.pl", \@args);

return $rc;
}




sub do_command($)
{
my ($command_line) = @_;
my $rc = 0;

my ($command, $args) = split( / /, $command_line, 2);
$args = ''
if (!defined $args);
if ($command eq 'ECHO')
{
BANNER_print( 'NOTE', "Execute ECHO");
my $line = ENV_expand_envs( $args);
ENV_say( 0, $line);
} elsif ($command eq 'SET')
{
BANNER_print( 'NOTE', "Execute $command_line");
my ($name, $value) = split( / /, $args, 2);

$value = ENV_expand_envs( $value);
ENV_setenv( $name => $value);
} else
{
BANNER_print( 'NOTE', "Executing Command: $command_line...");
my @args = split( ' ', $args);
$rc = EXEC_batch_run( ENV_command_ref( $command, \@args));
}

return $rc;
}





sub GBSSYSBG_print_errors()
{
my $gbs_logfile = ENV_getenv( 'GBS_LOGFILE');
if ($gbs_logfile && $gbs_logfile ne '-')
{
my $fail_filespec = "$gbs_logfile.err";
if (-e $fail_filespec)
{
BANNER_print( 'FAIL', [
'Error Log:',
'',
SLURP_file( $fail_filespec) ]);
}
}
}

1;


