#=================================================
#
#   proc.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::proc;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PROC_at
PROC_at_show
PROC_spawn_detached
PROC_spawn_xterm
PROC_spawn
PROC_wait
PROC_waitpid
PROC_waitpid_test
PROC_kill
);
}




use POSIX qw(:errno_h :sys_wait_h);
BEGIN
{
if ($^O eq 'MSWin32')
{
require glo::proc_win32;
import  glo::proc_win32;
}
}

use glo::env;
use glo::shell;
use glo::file;




sub PROC_at($$$$);
sub PROC_at_show();
sub PROC_spawn_detached($);
sub PROC_spawn_xterm($$$$$$$);
sub PROC_spawn($$);
sub PROC_wait();
sub PROC_waitpid($);
sub PROC_waitpid_test($);
sub PROC_kill($);

sub background_command($);




my $WNOHANG = 1;
my $IS_WIN32 = ENV_is_win32();

my $APP_SCRIPTS_PATH;	# E.g.: GBS_SCRIPTS_PATH




INIT
{
$APP_SCRIPTS_PATH = ENV_get_application_scripts_path();	# E.g.: GBS_SCRIPTS_PATH
}




sub PROC_at($$$$)
{
my ($command,
$name,			# jobname
$delay,			# now | [=+]hh:mm
$must_mail		# bool
) = @_;

my $os_command = SHELL_at( $command, $name, $delay, $must_mail);
if ($IS_WIN32)
{
$os_command = "start \"$name\" /I /MIN $os_command";
} else
{

}
ENV_system( $os_command, 0 );
}





sub PROC_at_show()
{
if ($IS_WIN32)
{

} else
{
my $rc = ENV_system( SHELL_at_show(), [0, 1]);
ENV_sig( W => "'\\atq' returned rc=$rc. This seems to happen sometimes without known reason...")
if ($rc != 0);
}
}





sub PROC_spawn_detached($)
{
my ($command_ref,		# ADT
) = @_;

ENV_debug( 1, 'PROC_spawn_detached:');

my $command_line = ENV_command_line( $command_ref);

if ($IS_WIN32)
{
my ($type, $prefix, $os_exec, @args) = ENV_get_command_ref_data( $command_ref);
if ($prefix eq 'perl')
{
my $command_spec = ENV_get_perl_cmd();
if ($command_spec =~ /^perl(\s|$)/)
{
$command_spec = FILE_which( ENV_perl_path_nospace( 'perl'), 'E');
ENV_sig( F => "Cannot find 'perl'")
if (!defined $command_spec);
$command_spec =~ tr%/%\\%;	# os_filespec
}
my ($pid, $proc) = PROC_WIN32_create_detached( $command_spec, $command_line, '.');
} else
{
$command_line = 'start ' . $command_line
if ($command_line !~ /^start /);

ENV_system( $command_line, 0);
}
} else #assume LUNIX
{
my $full_command = background_command( $command_line);

ENV_system( $full_command, 0);
}
}





sub PROC_spawn_xterm($$$$$$$)
{
my ($command_ref,		# ADT
$xtitle,
$fg,			# may be undef. Default = blue
$bg,			# may be undef. Default = white
$cols,			# may be undef. Default = 100
$rows,			# may be undef. Default = 60
$icon			# filespec
) = @_;

ENV_debug( 1, "PROC_spawn_xterm:");

my ($fgc, $bgc) = SHELL_get_term_color_codes( $fg, $bg);
($cols, $rows) = SHELL_get_geometry( $cols, $rows);

my $full_command;
if ($IS_WIN32)
{
my $command_line;
my ($type, $prefix, $os_exec, @args) = ENV_get_command_ref_data( $command_ref);
$prefix = ''
if ($prefix eq 'start');
$command_line = ENV_command_line( [ $type, $prefix, $os_exec, @args ]);
$command_line = "start \"$xtitle\" $APP_SCRIPTS_PATH\\glo\\xterm.bat \"$xtitle\" ${bgc}${fgc} $cols $rows $command_line";
ENV_debug( 2, "command_line=$command_line");
ENV_system( $command_line, 0);

} else #assume LUNIX
{
my $command_line = ENV_command_line( $command_ref);
$command_line = "xterm -T \"$xtitle\" -fg $fgc -bg $bgc -geometry ${cols}x${rows} -e $command_line";
my $full_command = background_command( $command_line);
ENV_system( $full_command, 0);
}
}





sub background_command($)
{
my ($command_line,
) = @_;
my $full_command;

my $logfile = (ENV_is_debug()) ? ENV_get_tmp_spec( 'nohup.out') : undef;    # undef = /dev/nul
$full_command = join( ' ', (
"nohup $command_line",
SHELL_redirect_stdout_stderr( $logfile, 1),
SHELL_redirect_stdin( undef),		    # undef = /dev/nul
SHELL_run_background(),
));


return $full_command;
}






sub PROC_spawn($$)
{
my ($command_ref,		# ADT
$log_file,		# undef = do redirect
) = @_;
my $pid = 0;

ENV_debug( 1, "PROC_spawn:");

do
{
$pid = fork;
if (defined $pid)
{
if ($pid == 0)
{



ENV_debug( 1, "PROC_spawn: fork child");
ENV_exec( $command_ref, $log_file);
} else
{



ENV_debug( 1, "PROC_spawn: fork parent: child_pid=$pid");
}
} else
{
if ($! == EAGAIN)
{



my $command_line = ENV_command_line( $command_ref);
ENV_sig( W => "fork 'exec $command_line' failed",
"- $!",
"- Retry after 5 seconds");
sleep( 5);
} else
{
ENV_sig( F => "Unable to fork: $!");
}
}
} while (!defined $pid);

return $pid;
}






sub PROC_wait()
{
my ($r_pid, $rc);

ENV_debug( 1, "PROC_wait");
$r_pid = wait();
ENV_debug( 1, "- r_pid=$r_pid");

if ($r_pid != -1)
{
$rc = ENV_rc_status( $?, 'wait');
} else
{
$rc = -1;
}

return ($r_pid, $rc);
}






sub PROC_waitpid($)
{
my ($pid) = @_;	    # -1 == wait for any (is equal to PROC_Wait)
my $rc = 0;

ENV_debug( 1, "PROC_waitpid pid=$pid");
my $r_pid = waitpid( $pid, 0);
ENV_debug( 1, "- r_pid=$r_pid");

if ($r_pid != -1)
{
$rc = ENV_rc_status( $?, "$r_pid=waitpid $pid, 0");
} else
{
$rc = -1;
}

return (wantarray ? ($r_pid, $rc) : $rc);
}







sub PROC_waitpid_test($)
{
my ($pid) = @_;	# -1 == wait for any.
my $rc = 0;

ENV_debug( 1, "PROC_waitpid_test pid=$pid");
my $r_pid = waitpid( $pid, WNOHANG);
ENV_debug( 1, "- r_pid=$r_pid");

if ($r_pid != -1 && $r_pid != 0)
{
$rc = ENV_rc_status( $?, "waitpid_test $r_pid, " . WNOHANG);
} else
{
$rc = -1;
}

return (wantarray ? ($r_pid, $rc) : $rc);
}





sub PROC_kill($)
{
my ($pid) = @_;
my $killed = 0;	    # bool

ENV_debug( 1, "$$ PROC_kill pid=$pid");

if ($IS_WIN32)
{
my $command = "TASKKILL /F /T /PID $pid";





my ($rc, $stdout) = ENV_backtick( $command, undef, 1);
if ($rc == 0 || $stdout =~ /^SUCCESS.*with PID $pid .*/m)
{
$killed = 1;
}
ENV_debug( 1, "PROC_kill rc = $rc", $stdout, "killed=$killed");
} else  # $IS_LINUX
{
my $command = "kill -TERM $pid";
my $rc = ENV_system( $command, undef);
if ($rc == 0)
{
$killed = 1;
}
ENV_debug( 1, "PROC_kill rc=$rc, killed=$killed");
}

return $killed;
}

1;
