#=================================================
#
#   time.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::time;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
TIME_num2time
TIME_time2num
TIME_time2numdate
TIME_time2num_packed
TIME_date2num_packed
TIME_unpack
TIME_unpack_date
TIME_deltatime2num
TIME_check_numdate
TIME_check_numdatetime
);
}




use Time::Local;
use glo::env;




sub TIME_num2time($);
sub TIME_time2num(;$);
sub TIME_time2numdate(;$);
sub TIME_time2num_packed(;$);
sub TIME_date2num_packed(;$);
sub TIME_unpack($);
sub TIME_unpack_date($);
sub TIME_deltatime2num($);
sub TIME_check_numdate($);
sub TIME_check_numdatetime($);







































sub TIME_num2time($)
{
my ($num) = @_;
my $time;

if ($num =~ /^0000-00-00/)
{
$time = 0;
} else
{
my ($year, $month, $day, $hour, $min, $sec) = (0,0,0,0,0,0);
if (length( $num) == 10)
{
($year, $month, $day) = ($num =~ /(\d\d\d\d)-(\d\d)-(\d\d)/);
} else
{
($year, $month, $day, $hour, $min, $sec) = ($num =~ /(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)/);
}
$time = timelocal( $sec, $min, $hour, $day, $month - 1, $year);
}

return $time;
}








sub TIME_time2num(;$)
{
my ($time) = @_;
my @thistime = (defined $time) ? localtime( $time) : localtime;

return sprintf "%4d-%02d-%02d %02d:%02d:%02d",
$thistime[5] + 1900, $thistime[4] + 1,$thistime[3],
$thistime[2], $thistime[1], $thistime[0];
}








sub TIME_time2numdate(;$)
{
my ($time) = @_;
my @thistime = (defined $time) ? localtime( $time) : localtime;

return sprintf "%4d-%02d-%02d",
$thistime[5] + 1900, $thistime[4] + 1,$thistime[3];
}








sub TIME_time2num_packed(;$)
{
my ($time) = @_;
my @thistime = (defined $time) ? localtime( $time) : localtime;

return sprintf "%04d%02d%02d-%02d%02d%02d",
$thistime[5] + 1900, $thistime[4] + 1,$thistime[3],
$thistime[2], $thistime[1], $thistime[0];
}








sub TIME_date2num_packed(;$)
{
my ($time) = @_;
my @thistime = (defined $time) ? localtime( $time) : localtime;

return sprintf "%04d%02d%02d",
$thistime[5] + 1900, $thistime[4] + 1,$thistime[3];
}














sub TIME_unpack($)
{
my ($packed_time) = @_;


my $length = length( $packed_time);

if ($length == 6)
{
my ($hh, $mm, $ss) = $packed_time =~ /(\d\d)(\d\d)(\d\d)/;

return sprintf "%02d:%02d:%02d",
$hh, $mm, $ss;
} elsif ($length == 8)
{
my ($YYYY, $MM, $DD) = $packed_time =~ /(\d\d\d\d)(\d\d)(\d\d)/;

return sprintf "%4d-%02d-%02d",
$YYYY, $MM, $DD;
} elsif ($length == 13)
{
my ($YY, $MM, $DD, $hh, $mm, $ss) = $packed_time =~ /(\d\d)(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)/;

return sprintf "%4d-%02d-%02d %02d:%02d:%02d",
$YY + 2000, $MM, $DD, $hh, $mm, $ss;
} elsif ($length == 15)
{
my ($YYYY, $MM, $DD, $hh, $mm, $ss) = $packed_time =~ /(\d\d\d\d)(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)/;

return sprintf "%4d-%02d-%02d %02d:%02d:%02d",
$YYYY, $MM, $DD, $hh, $mm, $ss;
} else
{
ENV_sig( F => "Invalid date/time length for '$packed_time'");
}
}











sub TIME_unpack_date($)
{
my ($packed_time) = @_;


my $length = length( $packed_time);

if ($length == 6)	# hhmmss
{
ENV_sig( F => "Cannot unpack date for time ($packed_time)");
} else
{
return substr( TIME_unpack( $packed_time), 0, 10);
}
}







sub TIME_deltatime2num($)
{
my ($diff_time) = @_;	    # nr-seconds

my $sec = $diff_time % 60;
my $min = $diff_time / 60;
my $hours = $min / 60;
$min = $min % 60;

return sprintf "%03d:%02d:%02d", $hours, $min, $sec;
}







sub TIME_check_numdate($)
{
my ($numdate) = @_;
my $error_txt = '';

my ($YYYY, $MM, $DD) = $numdate =~ /^(\d{4})-(\d\d)-(\d\d)$/;
if (defined $YYYY)
{
if ($MM > 12)
{
$error_txt = "Invalid Month '$MM'";
} elsif ($MM eq '02')
{
if ($DD > 29)
{
$error_txt = 'February never has more than 29 days';
} elsif ($DD == 29)
{




my $is_leap_year = (($YYYY % 4 == 0 and $YYYY % 100 != 0) || $YYYY % 400 == 0) ? 1 : 0;
$error_txt = 'Not a leap year'
if (!$is_leap_year);
}
} elsif (grep( $MM eq $_, qw( 04 06 09 11)) >= 0)
{
$error_txt = 'This month only has 30 days'
if ($DD > 30);
} else
{
$error_txt= 'No month has more than 31 days'
if ($DD > 31);
}
$error_txt = 'Perl dates are limited to 1970-01-01 till 2038-01-19'
if ($numdate lt '1970-01-01' || $numdate gt '2038-01-19');
} else
{
$error_txt = 'Format must be YYYY-MM-DD';
}

return $error_txt;
}







sub TIME_check_numdatetime($)
{
my ($numdatetime) = @_;
my $error_txt = '';

return $error_txt;
}

1;
