#=================================================
#
#   plugin.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package plugin;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PLUGIN_reset
PLUGIN_read
PLUGIN_setup
PLUGIN_set_init_envs
PLUGIN_get_main_envs
PLUGIN_check_envs
PLUGIN_att2plugin
PLUGIN_get_gbs_path
PLUGIN_get_all_plugin_names
PLUGIN_get_att_plugin_name
PLUGIN_get_att_names
PLUGIN_get_all_att_names
PLUGIN_get_platform
);
}




use glo::env;
use gbsenv;
use audit;
use fspec;
use target;
use tool;
use system;
use gbssfile;




sub PLUGIN_reset();
sub PLUGIN_read($);
sub PLUGIN_setup($);
sub PLUGIN_set_init_envs($);
sub PLUGIN_get_main_envs($);
sub PLUGIN_check_envs($$);
sub PLUGIN_att2plugin($);
sub PLUGIN_get_gbs_path($$);
sub PLUGIN_get_att_plugin_name($);
sub PLUGIN_get_all_plugin_names($);
sub PLUGIN_get_att_names($);
sub PLUGIN_get_all_att_names($);
sub PLUGIN_get_platform($);

sub get_plugin_name_type($);
sub get_plugin_name_att($);
sub init_from_system();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $IS_WIN32 = ENV_is_win32();




my %READ_FUNCS = (

audit   => \&AUDIT_read,
build   => \&TARGET_read,
tools   => \&TOOL_read,
);

my %PLUGIN2NAMES = (

audit   => [ qw( audit  AUDIT  Audit ) ],
build   => [ qw( target TARGET Target ) ],
tools   => [ qw( tool   TOOL   Tool) ],
);

my %ATT2PLUGIN = (

audit  => 'audit',
target => 'build',
tool   => 'tools',
);

my %SYSTEM2PLUGIN = (

audits  => 'audit',
targets => 'build',
tools   => 'tools',
);
my @SYSTEM_TYPES = sort keys %SYSTEM2PLUGIN;

my @ALL_PLATFORMS = qw( MSWin32 linux);




my %PLUGIN_ENVS;


my %ALL_PLUGIN_TYPES;


my %ALL_PLUGINS;


my %ALL_ATT_NAMES;


my %ALL_PLUGIN_NAMES_REFS;



my %ATT_NAMES_REFS;



my %ALL_ATT_NAMES_REFS;



my %ALL_ATT_PLATFORMS;





sub PLUGIN_reset()
{
%PLUGIN_ENVS = ();
%ALL_PLUGIN_TYPES = ();
%ALL_PLUGINS = ();
%ALL_ATT_NAMES = ();
%ALL_PLUGIN_NAMES_REFS = ();

%ATT_NAMES_REFS = ();
%ALL_ATT_NAMES_REFS = ();

%ALL_ATT_PLATFORMS = ();
}




sub PLUGIN_read($)
{
my ($plugin_name,
) = @_;
my $struct_id;

my $plugin_type = get_plugin_name_type( $plugin_name);
my $att_name = get_plugin_name_att( $plugin_name);

$struct_id = $READ_FUNCS{$plugin_type}->($att_name);

return $struct_id;
}




sub PLUGIN_setup($)
{
my ($plugin_type,    # audit build tools
) = @_;

my ($att_type, $uc_att_type, $fuc_att_type) = @{$PLUGIN2NAMES{$plugin_type}};

my $gbs_att_type = "GBS_$uc_att_type";		    # E.g.: GBS_AUDIT
my $att = ENV_getenv( $gbs_att_type);	    # $GBS_AUDIT
my @atts = ENV_getenv( $gbs_att_type . 'S');    # @GBS_AUDITS



if (grep( $_ eq $att, @atts))
{



PLUGIN_set_init_envs( $att);




my $gbs_att_plugin = ENV_getenv( "GBS_${uc_att_type}_PLUGIN");	    # GBS_AUDIT_PLUGIN
my $gbsext_plugin_build;
{
my $uc_gbs_att_plugin = uc $gbs_att_plugin;
my $plugin_path = ENV_getenv( "GBSEXT_${uc_gbs_att_plugin}_PATH");
ENV_setenv_os_path( "GBS_${uc_att_type}_PLUGIN_PATH" => $plugin_path);

my $plugin_bin_path = ENV_getenv( "GBSEXT_${uc_gbs_att_plugin}_BIN_PATH");
ENV_setenv_os_path( "GBS_${uc_att_type}_PLUGIN_BIN_PATH" => $plugin_bin_path);

$gbsext_plugin_build = ENV_getenv( "GBSEXT_${uc_gbs_att_plugin}_BUILD");
}




{
my $plugin_rel = PLUGIN_get_main_envs( $gbs_att_plugin);
my $build_text = ($gbsext_plugin_build eq '') ? '' : "($gbsext_plugin_build)";
my $sca_text = '';
my $src_types_text = '';
if ($plugin_type eq 'audit')
{
my @src_types = AUDIT_get_src_types( $att);
$src_types_text = " (@src_types)";
$sca_text = (-e FSPEC_scadef( $gbs_att_plugin)) ? ' SCA' : '';
} elsif ($plugin_type eq 'build')
{
my @src_types = TARGET_get_src_types( $att);
$src_types_text = " (@src_types)";
} else  # ($plugin_type eq 'tool')
{

}
ENV_whisper( 1, "$att [$gbs_att_plugin ($plugin_rel$build_text)$sca_text]$src_types_text");
}




{
my @command_data_refs;

if ($plugin_type eq 'audit')
{
@command_data_refs = AUDIT_get_section_commands_data_refs( $att, 'INIT');	# E.g.: cpptestinit.pl
} elsif ($plugin_type eq 'build')
{
@command_data_refs = TARGET_get_section_commands_data_refs( $att, 'INIT');
} else  # ($plugin_type eq 'tool')
{

}
if (@command_data_refs)
{
ENV_say( 1, "Executing $fuc_att_type INIT...");
foreach my $command_data_ref (@command_data_refs)
{

my $command_line = $command_data_ref->[1];   # $command_line

my ($command, $args_ref) = ENV_command_split( $command_line);
my $command_ref = ENV_command_ref( $command, $args_ref);
my $rc = ENV_system( $command_ref, undef);    # no RC check, no logfile
ENV_sig( EE => "$att INIT failed")
if ($rc != 0);
}
}
}
} else
{
ENV_say( 1, "$fuc_att_type: '$att' does not $fuc_att_type for this platform\n");
}
}





sub PLUGIN_set_init_envs($)
{
my ($att_name,
) = @_;

my $plugin_name = PLUGIN_get_att_plugin_name( $att_name);
my $uc_plugin_name = uc $plugin_name;
my $plugin_type = $ALL_PLUGIN_TYPES{$plugin_name};
my $struct_id = $READ_FUNCS{$plugin_type}->($att_name);


{



my @keys = ($IS_WIN32) ? qw( SET_W SET) : qw( SET_X SET);
foreach my $key (@keys)
{
my $set_refs_ref = GBSSFILE_get_section_items( $struct_id, INIT => $key);
if (defined $set_refs_ref)
{
foreach my $ref (@{$set_refs_ref})
{
my ($name, $value) = @{$ref};

$value = ENV_expand_envs( $value);
ENV_sig( EE => "No match for EnvVar in $key $name = $value")
if ($value =~ /%|\$/);
$value = ENV_os_path( ENV_perl_path_nospace( $value));
ENV_setenv( $name => $value );

}
}
}
}

{



my $setpath_refs_ref = GBSSFILE_get_section_items( $struct_id, INIT => 'SETPATH');
if (defined $setpath_refs_ref)
{
foreach my $ref (@{$setpath_refs_ref})
{
my ($name, @values) = @{$ref};

foreach my $value (@values)
{

if ($value eq $name || $value eq "%$name%" || $value eq "\$$name")
{

$value = ENV_os_path( ENV_getenv( $name));
} else
{
$value = ENV_expand_envs( $value);
ENV_sig( EE => "No match for EnvVar '$value'", "in SETPATH $name = @values")
if ($value =~ /%|\$/);
$value = ENV_perl_path_nospace( $value);
$value = ENV_os_path( $value);
}

}
my $new_value = ENV_join_path( \@values);	# ';' or ':'
ENV_setenv( $name => $new_value );

}
}
}




{
my $bin_dir = GBSSFILE_get_section_items( $struct_id, INIT => 'BIN_DIR');
my $plugin_path = ENV_getenv_perl_path( "GBSEXT_${uc_plugin_name}_PATH");
my $plugin_bin_path = ($bin_dir eq '') ? $plugin_path : "$plugin_path/$bin_dir";
ENV_setenv_os_path( "GBSEXT_${uc_plugin_name}_BIN_PATH" => $plugin_bin_path);

}
}




sub PLUGIN_get_main_envs($)
{
my ($plugin_name) = @_;
my ($plugin_rel, $plugin_path) = ();

my $check_ok;
my $ref = $PLUGIN_ENVS{$plugin_name};
if (defined $ref)
{
($plugin_rel, $plugin_path, $check_ok) = @{$ref};
} else
{
($plugin_rel, $plugin_path) = GBSENV_get_plugin_envs( $plugin_name);
$check_ok = PLUGIN_check_envs( $plugin_name, 'E');
$PLUGIN_ENVS{$plugin_name} = [ $plugin_rel, $plugin_path, $check_ok ];
}
ENV_sig( EE => "Essential EnvVars missing for Plugin $plugin_name")
if (!$check_ok);

if (wantarray)
{
return ($plugin_rel, $plugin_path);
} else
{
return $plugin_rel;
}
}




sub PLUGIN_check_envs($$)
{
my ($plugin_name,
$sig_on_error,		# '', 'E', 'EE', etc
) = @_;
my $check_ok;



my $ref = $PLUGIN_ENVS{$plugin_name};
if (defined $ref)
{
(undef, undef, $check_ok) = @{$ref};

} else
{
$check_ok = 1;
my ($plugin_rel, $plugin_path) = GBSENV_get_plugin_envs( $plugin_name);




my $plugin_env_name = uc $plugin_name;
my @errors;
if ($plugin_rel eq '')
{
push @errors, "- 'GBSEXT_${plugin_env_name}_REL' not defined in switch.gbs"
}
if ($plugin_path eq '')
{
push @errors, "- 'GBSEXT_${plugin_env_name}_PATH' not defined in switch.gbs"
} else
{
push @errors, "- 'GBSEXT_${plugin_env_name}_PATH)' ($plugin_path) does not exist"
if (! -e $plugin_path);
}




my $struct_id = ENV_try( W => sub { PLUGIN_read( $plugin_name) } );
if (defined $struct_id)
{
my @requires = GBSSFILE_get_section_items( $struct_id, SETUP => 'REQUIRE');
foreach my $envvar (@requires)
{
push @errors, "- '$envvar' not defined in switch.gbs"
if (ENV_getenv( $envvar) eq '');
}
}




if (@errors)
{
ENV_sig( $sig_on_error => "Plugin: $plugin_name:", @errors);
$check_ok = 0;
}
$PLUGIN_ENVS{$plugin_name} = [ $plugin_rel, $plugin_path, $check_ok ];
}

return $check_ok;
}




sub PLUGIN_get_gbs_path($$)
{
my ($plugin_type,	# audit, build or tools
$plugin_name,
) = @_;
my $gbs_plugin_path;

ENV_sig( F => "Invalid plugin_type ($plugin_type)")
if (!grep( $plugin_type eq $_, qw( audit build tools)));

my ($plugin_rel, $plugin_path) = PLUGIN_get_main_envs( $plugin_name);
$gbs_plugin_path = "$ENV{GBS_PLUGIN_ROOT}/$plugin_type/$plugin_name/$plugin_rel";
ENV_mkpath( $gbs_plugin_path);

return $gbs_plugin_path;
}




sub PLUGIN_att2plugin($)
{
my ($att_type,	# audit, target or tool
) = @_;
my $plugin_type;	# audit, build, tools

return $ATT2PLUGIN{$att_type};
}




sub PLUGIN_get_all_plugin_names($)
{
my ($plugin_type,	# audit, build or tools
) = @_;
my @plugin_names;

init_from_system()
if (!%ALL_PLUGIN_NAMES_REFS);

return @{$ALL_PLUGIN_NAMES_REFS{$plugin_type}};
}




sub PLUGIN_get_platform($)
{
my ($att_name,
) = @_;


init_from_system()
if (!%ALL_ATT_PLATFORMS);

return $ALL_ATT_PLATFORMS{$att_name};
}




sub get_plugin_name_type($)
{
my ($plugin_name,
) = @_;
my $plugin_type;	# audit, build or tools

init_from_system()
if (!%ALL_PLUGIN_TYPES);

$plugin_type = $ALL_PLUGIN_TYPES{$plugin_name};
ENV_sig( F => "Invalid Plugin '$plugin_name'")
if (!defined $plugin_type);

return $plugin_type;
}




sub PLUGIN_get_att_plugin_name($)
{
my ($att_name,	# e.g. target_name
) = @_;
my $plugin_name;

init_from_system()
if (!%ALL_PLUGINS);

return $ALL_PLUGINS{$att_name};
}




sub get_plugin_name_att($)
{
my ($plugin_name,
) = @_;
my $att_name;   # e.g. target_name

init_from_system()
if (!%ALL_ATT_NAMES);

return $ALL_ATT_NAMES{$plugin_name};
}





sub PLUGIN_get_att_names($)
{
my ($plugin_type,	# audit, build or tools
) = @_;
my @att_names;

if (!%ATT_NAMES_REFS)
{
foreach my $system_type (@SYSTEM_TYPES)	# (audits targets tools)
{
my @names = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '.', $system_type);
my $plugin_type = $SYSTEM2PLUGIN{$system_type};
$ATT_NAMES_REFS{$plugin_type} = [ @names ];
}

}

return @{$ATT_NAMES_REFS{$plugin_type}};
}





sub PLUGIN_get_all_att_names($)
{
my ($plugin_type,	# audit, build or tools
) = @_;
my @att_names;

if (!%ALL_ATT_NAMES_REFS)
{
foreach my $system_type (@SYSTEM_TYPES)	# (audits targets tools)
{
my @names = SYSTEM_get_os_ref_names( $GBS_ROOT_PATH, '*', $system_type);
my $plugin_type = $SYSTEM2PLUGIN{$system_type};
$ALL_ATT_NAMES_REFS{$plugin_type} = [ @names ];
}

}

return @{$ALL_ATT_NAMES_REFS{$plugin_type}};
}




sub init_from_system()
{
my ($plugin_name,
) = @_;

foreach my $system_type (@SYSTEM_TYPES)	# (audits targets tools)
{
my %plugin_names;

foreach my $platform (SYSTEM_get_osnames( $GBS_ROOT_PATH, $system_type))
{
my @refs = SYSTEM_get_os_refs( $GBS_ROOT_PATH, $platform, $system_type);

foreach my $ref (@refs)
{
my ($att_name, $plugin_name) = @{$ref};
$ALL_PLUGIN_TYPES{$plugin_name} = $SYSTEM2PLUGIN{$system_type};
$ALL_PLUGINS{$att_name} = $plugin_name;
$ALL_ATT_NAMES{$plugin_name} = $att_name;
$ALL_ATT_PLATFORMS{$att_name} = $platform;
$plugin_names{$plugin_name} = 1;
}
}
$ALL_PLUGIN_NAMES_REFS{$system_type} = [ sort keys %plugin_names ];
}





}

1;
