#===============================================================
#
#   cpptest.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Call:
#	arg_1	= file_name	(without type)
#	arg_2	= in_filespec	(relative file spec - file_name + type
#	arg_3	= out_file_base (relative file spec - with in file type)
#	arg_4	= out_file_dir	(relative file spec - directory only)
#	arg_5	= out_filetype
#	arg_6-N = command_flags (from command line)
#   Environment:
#	General:
#	    GBS_INCS		Include paths ( -I<dir> ... )
#	    GBS_SYSINCS 	System include paths ( -SI<dir> ... )
#	    GBS_AUDIT_FLAGS	Passed from session
#	    GBS_FLAGS		From flags_<type> files
#	    GBS_SYSFLAGS	From bld_flags_<type> file
#	    GBS_AUDIT_OPTIONS	($MUST_SHOW_STDOUT)
#	Application:
#	    GBS_TARGET
#	    GBS_AUDIT
#	    GBSEXT_CPPTEST_REL
#	    GBSEXT_CPPTEST_PATH
#	    GBS_COMPONENT
#============================================================================
package cpptest;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CPPTEST_init
CPPTEST_set
CPPTEST_pre
CPPTEST_run
CPPTEST_post
CPPTEST_load
);
}




use glo::env;
use glo::flock;
use gbsenv;
use scafile;
use scadef;
use scametrics;
use gbsbgglo;
use gbsbgscasub;
use XML::Simple qw(:strict);

use cpptestglo;
use cpptestgout;





sub CPPTEST_init($$$);
sub CPPTEST_set($$);
sub CPPTEST_pre($$);
sub CPPTEST_run($$$@);
sub CPPTEST_post($$);
sub CPPTEST_load();

sub handle_unkn_msg($$$);




my $GBS_SCRIPTS_PATH = ENV_getenv_perl_path( 'GBS_SCRIPTS_PATH');
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBSEXT_CPPTEST_BUILD = ENV_getenv( 'GBSEXT_CPPTEST_BUILD');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');

my $GBS_SILO_CPPTEST_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache/cpptest";
my $DATA_PATH = "$GBS_SILO_CPPTEST_PATH/$GBS_TARGET";








my $PLUGIN_NAME;	    # cpptest
my $GBSEXT_PLUGIN_PATH;
my $GBS_PLUGIN_PATH;	    # $GBS_PLUGIN_ROOT/$plugin/$plugin_rel

my @SRC_FILES;
my $CONFIG_SPEC;
my $CONFIG_NAME;




my @GBS_DEFAULTS;	    #
my @COMPILER_SETTINGS;	    #
my @GBS_SETTINGS;	    #




my $GOUT_FILE_SPEC;

my $LOCK_FILESPEC = "$GBS_BASE_PATH/cpptest.lck";
my $LOCK_FH;

my @LEVEL_TEXTS;






my %MET_DEFS = (

'METRICS-29'    => [ 'CYC', qr/value:\s+(\d+)$/ ],	    # Cyclomatic Complexity ... value: 1
'METRICS-33'    => [ 'M07', qr/value:\s+(\d+)$/ ],	    # Essential Complexity ... value: 3
'METRICS-43'    => [ 'HLB',	qr/is\s+(\d+\.\d+)$/ ],	    # Halstead's delivered bugs ... is 0.10
);






sub CPPTEST_init($$$)
{
(	$PLUGIN_NAME,		# cpptest
$GBSEXT_PLUGIN_PATH,
$GBS_PLUGIN_PATH,	# $GBS_PLUGIN_ROOT/$plugin/$plugin_rel
) = @_;





{
my $gbsext_use_stub = ENV_getenv_bool( "GBSEXT_CPPTEST_USE_STUB");
if ($gbsext_use_stub)
{
ENV_say( 1, "** Running in Development Mode from STUBS...");
my $gbsext_plugin_bin_path = "$GBS_SCRIPTS_PATH/STUBS/cpptest";
ENV_setenv( "GBSEXT_CPPTEST_BIN_PATH" => ENV_os_path( $gbsext_plugin_bin_path));
}
}




($CONFIG_SPEC, $CONFIG_NAME) = CPPTESTGLO_init( $GBS_PLUGIN_PATH);

my @plugin_msg_filespecs = CPPTESTGLO_get_msg_filespecs();	    # .../rules.xml and .../rulemap.xml




my @msg_level_refs = SCADEF_get_levels( $PLUGIN_NAME);



map { $LEVEL_TEXTS[$_->[0]] = $_->[2] } @msg_level_refs;	# $level $text

return (\&handle_unkn_msg, [ @plugin_msg_filespecs ], \&CPPTESTGLO_read_msg_file, 1);        # $help_file_required
}





sub CPPTEST_set($$)
{
my ($compiler_settings_ref,
$extra_gbs_defaults_ref,	# e.g.: quiet-options (none for CPPTEST)
) = @_;




@GBS_DEFAULTS = ();





@COMPILER_SETTINGS = @{$compiler_settings_ref};




push @GBS_SETTINGS, @{$extra_gbs_defaults_ref};

}





sub CPPTEST_pre($$)
{
my ($src_path,
$source_file,
) = @_;
my @printable_out_files;

my $out_file_path = "../aud/$GBS_AUDIT/$GBS_TARGET";
my $src_type = ENV_split_spec_t( $source_file);
$GOUT_FILE_SPEC = "$out_file_path/gout$src_type.xml";

my @flags = (
'-nobuild',
-config		=> qq{"$CONFIG_SPEC"},
-data		=> qq{"$DATA_PATH"},
-resource	=> "${GBS_SUBSYS}_$GBS_COMPONENT",
-localsettings	=> "$GBS_SCRIPTS_PATH/plugins/audit/cpptest/gbs.settings",
);
ENV_setenv( GBS_CPPTEST_FLAGS => "@flags");


SCAFILE_garg_write( '#', \@GBS_DEFAULTS, \@COMPILER_SETTINGS, \@GBS_SETTINGS);


return @printable_out_files;
}





sub CPPTEST_run($$$@)
{
my ($src_files_ref,		#
$must_print_commands,	# scripts should print their own command(s). '-' == undef
$out_files_ref,		#
@command_data_lines	# ($command_type, $command_line, $ok_values, $exec_condition), ...

) = @_;
my $rc = 0;

@SRC_FILES = @{$src_files_ref};




$LOCK_FH = FLOCK_ex_wait( $LOCK_FILESPEC);




$rc = GBSBGGLO_exec( \@SRC_FILES, $must_print_commands, "@{$out_files_ref}", @command_data_lines);




FLOCK_unlock( $LOCK_FH);

return $rc;
}





sub CPPTEST_post($$)
{
my ($src_files_ref,
$unix_style_search,
) = @_;
my $rc = 0;	    # 1 = contains FATAL warnings, 2 = Execution error

my $nr_checked_files = CPPTESTGOUT_init( $GOUT_FILE_SPEC);
my $nr_src_files = @{$src_files_ref};
ENV_sig( E => "Mismatch between requested nr src files ($nr_src_files) and cpptest checked nr files ($nr_checked_files)")
if ($nr_checked_files != $nr_src_files);





my %gout_warning_refs = map { $_->[0] => $_->[1] } CPPTESTGOUT_get_warnings();

my %gout_fatal_refs = map { $_->[0] => $_->[1] } CPPTESTGOUT_get_fatals();





my $total_warnings = 0;
foreach my $src_file (@SRC_FILES)
{
ENV_say(1, "C++Test Analysing $GBS_COMPONENT:$src_file...");




my @all_refs;
{
my $gout_warning_refs_ref = $gout_warning_refs{$src_file};
push @all_refs, @{$gout_warning_refs_ref}
if (defined $gout_warning_refs_ref);
my $gout_fatal_refs_ref = $gout_fatal_refs{$src_file};
push @all_refs, @{$gout_fatal_refs_ref}
if (defined $gout_fatal_refs_ref);
}

my @warning_refs;


my $total_refs_ref;


my $metric_refs_ref;


my %met_data;





{
my $nr_warnings = 0;
foreach my $ref (@all_refs)
{
my ($src_line_nr, $col_nr, $msg_level, $msg_id, $msg_text) = @{$ref};

my $os_filespec = '-';	    # file is source-file
my $line_nr = $src_line_nr;
my $level_text = $LEVEL_TEXTS[ $msg_level];
my $help_file_url = '-';
my @extra_src_lines;	    # none

push @warning_refs, [ $src_line_nr, $col_nr, $os_filespec, $line_nr, $msg_level, $level_text, $msg_id, $msg_text, $help_file_url, @extra_src_lines ];

$nr_warnings++;
$total_warnings++;




if (exists $MET_DEFS{$msg_id})
{
my ($metric_name, $regexp) = @{$MET_DEFS{$msg_id}};
my ($value) = $msg_text =~ $regexp;
{
no integer;

my $met_ref = $met_data{$metric_name};
if (defined $met_ref)
{

$met_ref->[0] = $value if ($value < $met_ref->[0]);	    # $min
$met_ref->[1] = $value if ($value > $met_ref->[1]);	    # $max
$met_ref->[2] += $value;				    # $total
} else
{
$met_data{$metric_name} = [ $value, $value, $value ];
}
use integer;
}
}
}
@warning_refs = sort { $a->[0] <=> $b->[0] } @warning_refs;	# sort on $src_line_nr
}




{
my $src_type = ENV_split_spec_t( $src_file);
my $is_cpp = ($src_type eq '.cpp') ? 1 : 0;
($total_refs_ref, $metric_refs_ref) = SCAMETRICS_get( $src_file, undef, $is_cpp);

foreach my $metric_name (keys %met_data)
{
push @{$metric_refs_ref}, [ $metric_name => @{$met_data{$metric_name}} ];
}
}




my $this_rc = GBSBGSCASUB_file_store( $src_file, $unix_style_search, \@warning_refs, $total_refs_ref, $metric_refs_ref);
$rc = $this_rc
if ($this_rc > $rc);
%met_data = ();
}

ENV_say( 1, "Total $total_warnings warnings (rc=$rc)");

return $rc;
}





sub handle_unkn_msg($$$)
{
my ($msg_id,		# defined
$msg_level,		# defined
$level_text,		# defined
) = @_;
my $help_file_url = '.';	# take the default

return ($msg_level, $level_text, $help_file_url);
}





sub CPPTEST_load()
{
ENV_say( 1, "Loaded");
}

1;



