#===============================================================
#
#   cpptestglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Generic package for cpptest
#===============================================================
package cpptestglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CPPTESTGLO_init
CPPTESTGLO_get_msg_filespecs
CPPTESTGLO_read_msg_file
CPPTESTGLO_get_config_paths
);
}




use glo::env;
use glo::slurp;
use glo::spit;
use glo::flock;
use glo::unzip;
use gbsenv;
use scadef;
use audit;
use XML::Simple qw(:strict);




sub CPPTESTGLO_init($);
sub CPPTESTGLO_get_msg_filespecs();
sub CPPTESTGLO_read_msg_file($);
sub CPPTESTGLO_get_config_paths();

sub add_msg_def($$$$$);




my $GBS_SCRIPTS_PATH = ENV_getenv_perl_path( 'GBS_SCRIPTS_PATH');
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBSEXT_PLUGIN_PATH = ENV_getenv_perl_path( 'GBSEXT_CPPTEST_PATH');
my $GBSEXT_CPPTEST_REL = ENV_getenv( 'GBSEXT_CPPTEST_REL');
my $GBSEXT_CPPTEST_BUILD = ENV_getenv( 'GBSEXT_CPPTEST_BUILD');
my $GBS_PLUGIN_ROOT = ENV_getenv_perl_path( 'GBS_PLUGIN_ROOT');

my $GBS_SILO_CPPTEST_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache/cpptest";

my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');




my $GBS_PLUGIN_PATH;		# $GBS_PLUGIN_ROOT/$plugin/$plugin_rel

my $GBS_CONFIG_PATH = "$GBS_PLUGIN_ROOT/audit/cpptest/$GBSEXT_CPPTEST_REL";
my $GBS_CONFIG_SPEC = "$GBS_CONFIG_PATH/gbs_rules.properties";

my $DEFAULT_CONFIG_NAME = 'gbs://gbs_rules';

my $CONFIG_BUILTIN_PATH = "$GBSEXT_PLUGIN_PATH/eclipse/plugins/com.parasoft.xtest.checkers.eclipse.core.cpp_$GBSEXT_CPPTEST_BUILD/configs";
my $CONFIG_USER_PATH = "$GBS_SILO_CPPTEST_PATH/$GBS_TARGET/.metadata/.plugins/com.parasoft.xtest.common.eclipse.core/com.parasoft.xtest.checkers.api/configs/c++test";

my $CPPTEST_RULES_PATH  = "$GBSEXT_PLUGIN_PATH/eclipse/plugins/com.parasoft.xtest.standards.eclipse.core.cpp_$GBSEXT_CPPTEST_BUILD/rules/en";
my $CPPTEST_DOCS_PATH   = "$GBSEXT_PLUGIN_PATH/eclipse/plugins/com.parasoft.xtest.checkers.eclipse.ui.cpp_$GBSEXT_CPPTEST_BUILD";




my @MSG_FILE_SPECS;	# .../rules.xml and .../rulemap.xml

my @LEVEL_TEXTS;





my $CONFIG_NAME;		# E.g.: 'builtin://MISRA C 2012'
my $CONFIG_LOC;			# E.g.: 'builtin:', 'user:', 'team:', 'http:'
my $STANDARD;			# E.g.: 'gbs_rules'
my $CONFIG_SPEC;




sub CPPTESTGLO_init($)
{
(	$GBS_PLUGIN_PATH,	# $GBS_PLUGIN_ROOT/$plugin/$plugin_rel
) = @_;





ENV_sig( F => "cpptest: GBSEXT_CPPTEST_PATH ($GBSEXT_PLUGIN_PATH): No such directory")
if (! -d $GBSEXT_PLUGIN_PATH);

$CONFIG_NAME = AUDIT_get_section_items( $GBS_AUDIT, CONFIG => 'NAME');
if (defined $CONFIG_NAME)
{
ENV_say( 1, "Configuration: $CONFIG_NAME");

} else
{
$CONFIG_NAME = $DEFAULT_CONFIG_NAME;	    # gbs://gbs_rules
ENV_say( 1, "Configuration: $CONFIG_NAME assumed");
}

my $config_path;
if ($CONFIG_NAME =~ m"://")
{
($CONFIG_LOC, $STANDARD) = split( "//", $CONFIG_NAME);
if ($CONFIG_LOC eq 'gbs:')
{
$config_path = "$GBS_SCRIPTS_PATH/plugins/audit/cpptest";
} elsif ($CONFIG_LOC eq 'builtin:')
{
$config_path = $CONFIG_BUILTIN_PATH;
} elsif ($CONFIG_LOC eq 'user:')
{
$config_path = $CONFIG_USER_PATH;
} elsif ($CONFIG_LOC eq 'team:')
{
ENV_sig( EE => "CONFIG_LOC 'team' not supported (yet)");
} elsif ($CONFIG_LOC eq 'http:')
{
ENV_sig( EE => "CONFIG_LOC 'http' not supported (yet)");
} else
{
ENV_sig( EE => "Unknown CONFIG_LOC '$CONFIG_LOC'",
"- $CONFIG_NAME");
}
$CONFIG_SPEC = "$config_path/$STANDARD.properties";
} else
{
($CONFIG_LOC, $STANDARD, my $type) = ENV_split_spec_dnt( $CONFIG_NAME);
$config_path = $CONFIG_LOC;
if ($type eq '')
{
$CONFIG_SPEC = "$config_path/$STANDARD.properties";
} else
{
$CONFIG_SPEC = "$config_path/$STANDARD$type";
}
}



if (!-e $CONFIG_SPEC)
{
my @files = map { "- $_" } ENV_readdir( $config_path);
ENV_sig( EE => "Config file '$CONFIG_NAME' not found in",
$config_path,
'Available files:',
@files);
}




{
@MSG_FILE_SPECS = ("$CPPTEST_RULES_PATH/rules.xml", "$CPPTEST_RULES_PATH/rulemap.xml");
}

return ($GBS_CONFIG_SPEC, $CONFIG_NAME);
}




sub CPPTESTGLO_get_msg_filespecs()
{
return @MSG_FILE_SPECS;
}




sub CPPTESTGLO_read_msg_file($)
{
my ($help_data_wanted) = @_;    # Bool
my @msg_def_refs;








if ($help_data_wanted)
{
my $zip_filespec = "$CPPTEST_DOCS_PATH/doc.zip";
ENV_whisper( 1, "Extracting helpfiles to '$GBS_PLUGIN_PATH'");
my $nr_files = UNZIP_all( $zip_filespec => $GBS_PLUGIN_PATH);
ENV_whisper( 0, "$nr_files help html files extracted");
}




my @msg_level_refs = SCADEF_get_levels( $GBS_AUDIT_PLUGIN);



map { $LEVEL_TEXTS[$_->[0]] = $_->[2] } @msg_level_refs;	# $level $text




ENV_whisper( 1, "Reading rules.xml...");
my $rules_ref = XMLin( $MSG_FILE_SPECS[0], KeyAttr => [ ], ForceArray => 0);
my $categories_ref = $rules_ref->{builtin}->{category};

my %rules;

foreach my $ref (@{$categories_ref})
{

my $description = $ref->{description};
my $name = $ref->{name};
my $sub_category_ref = $ref->{category};
if (defined $sub_category_ref)
{

my $sub_description = $sub_category_ref->{description};
my $sub_name = $sub_category_ref->{name};
my @sub_rule_refs = @{$sub_category_ref->{rule}};

foreach my $sub_rule_ref (@sub_rule_refs)
{
my $sub_header = $sub_rule_ref->{header};
my $sub_id = $sub_rule_ref->{id};
my $sub_severity = $sub_rule_ref->{severity};

my $sub_msg_id = "$name-$sub_name-$sub_id";
push @msg_def_refs, add_msg_def( $help_data_wanted, $sub_msg_id, $sub_severity, $sub_header, $sub_msg_id);
$rules{"$sub_name-$sub_id"} = [ $sub_severity, $sub_header ];
}

}
my @rule_refs = @{$ref->{rule}};

foreach my $rule_ref (@rule_refs)
{
my $header = $rule_ref->{header};
my $id = $rule_ref->{id};
my $severity = $rule_ref->{severity};

my $msg_id = "$name-$id";
push @msg_def_refs, add_msg_def( $help_data_wanted, $msg_id, $severity, $header, $msg_id);
$rules{$msg_id} = [ $severity, $header ];
}
}
$rules_ref = undef;	    # cleanup





ENV_whisper( 1, "Reading rulemaps.xml...");
my $rulemap_ref = XMLin( $MSG_FILE_SPECS[1], KeyAttr => [ ], ForceArray => 0);





my $category_ids_re;
{
my @category_refs = @{$rulemap_ref->{category}};

my @category_ids;
foreach my $ref (@category_refs)
{
my $description = $ref->{description};
my $category_id = $ref->{id};

push @category_ids, $category_id;
ENV_whisper( 2, "- $description");
}
ENV_sig( EE => "No categories found in Config '$CONFIG_NAME' not found")
if (@category_ids == 0);
$category_ids_re = join( '|', @category_ids);
}




{
push @msg_def_refs, [ 'COMPILE_ERROR',   6, $LEVEL_TEXTS[6], 'Compilation error', '-', 'COMPILE_ERROR' ];
push @msg_def_refs, [ 'COMPILE_FATAL',   6, $LEVEL_TEXTS[6], 'Fatal Compilation error - cannot proceed', '-', 'COMPILE_FATAL' ];
push @msg_def_refs, [ 'COMPILE_INCLUDE', 6, $LEVEL_TEXTS[6], 'Include file not found - cannot proceed', '-', 'COMPILE_INCLUDE' ];

my @clone_refs = @{$rulemap_ref->{clone}};


foreach my $ref (@clone_refs)
{

my $new_id = $ref->{newId};
if ($new_id =~ /^($category_ids_re)/) # starts with...
{


my $id = $ref->{id};
my $severity = $ref->{severity};
$severity = $rules{$id}->[0]
if (!defined $severity);
my $header = $ref->{header};
$header = $rules{$id}->[1]
if (!defined $header);
if (defined $severity && defined $header)
{
push @msg_def_refs, add_msg_def( $help_data_wanted, $new_id, $severity, $header, $id);
} else
{
ENV_sig( W => "Cannot determine severity and/or header of msg-id '$new_id'->'$id'");

}
}
}
}






if ($help_data_wanted)
{
ENV_say( 1, "Creating the gbs_rules config file from",
$CONFIG_SPEC);
unlink $GBS_CONFIG_SPEC;
my @lines;
@lines = SLURP_file( $CONFIG_SPEC);
foreach my $ref (@msg_def_refs)
{
my $msg_id = $ref->[0];
push @lines, "$msg_id=true";
}




my $fh = FLOCK_ex_wait( "$GBS_CONFIG_SPEC.lck");
SPIT_file_nl( $GBS_CONFIG_SPEC, \@lines);
FLOCK_unlock( $fh);
}

return @msg_def_refs;
}




sub add_msg_def($$$$$)
{
my ($help_data_wanted,
$msg_id,
$level,
$header,
$help_id,
) = @_;



my $level_text = $LEVEL_TEXTS[$level];	# $text
my $help_file_url = "rules/$help_id.html";

if ($help_data_wanted)
{
my @lines;
if (!-e "$GBS_PLUGIN_PATH/rules/$help_id.html")
{


push @lines, 'Help not found';
}
return [ $msg_id, $level, $level_text, $header, $help_file_url, $header, [ @lines ]];
} else
{
return [ $msg_id, $level, $level_text, $header, $help_file_url ];

}
}




sub CPPTESTGLO_get_config_paths()
{
return ( $CONFIG_BUILTIN_PATH, $CONFIG_USER_PATH);
}

1;


