#===============================================================
#
#   cpptestgout.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package cpptestgout;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
CPPTESTGOUT_init
CPPTESTGOUT_get_warnings
CPPTESTGOUT_get_fatals
CPPTESTGOUT_get_rules
);
}




use XML::Simple qw(:strict);
use glo::env;




sub CPPTESTGOUT_init($);
sub CPPTESTGOUT_get_warnings();
sub CPPTESTGOUT_get_fatals();
sub CPPTESTGOUT_get_rules();

sub un_escape_characters($);












my $GOUT_FILE_SPEC;
my $GOUT_REF;			# XML file





sub CPPTESTGOUT_init($)
{
(	$GOUT_FILE_SPEC,
) = @_;
my $nr_checked_files;

if (-e $GOUT_FILE_SPEC)
{
$GOUT_REF = XMLin( $GOUT_FILE_SPEC, KeyAttr => [ ], ForceArray => 1);




my $ScopeProjectInfo_ref = $GOUT_REF->{Scope}->[0]->{ProjectInformations}->[0]->{ScopeProjectInfo}->[0];

$nr_checked_files = $ScopeProjectInfo_ref->{totFiles};
ENV_sig( F => "'$GOUT_FILE_SPEC' does not seem to have the expected format")
if (! defined $nr_checked_files);
} else
{
ENV_sig( E => "$GOUT_FILE_SPEC not found");
}

return $nr_checked_files;
}




sub CPPTESTGOUT_get_warnings()
{
my @source_warnings_refs;



my $StdViol_ref = $GOUT_REF->{CodingStandards}->[0]->{StdViols}->[0]->{StdViol};
if (defined $StdViol_ref)
{


my @warning_refs;

my $cur_src_file = '';
foreach my $ref (@{$StdViol_ref})
{

my $filespec = $ref->{locFile};		    # /$project/$component/src/$file

my ($src_file) = $filespec =~ m!.*/(.*)$!;
if ($src_file ne $cur_src_file)
{
push @source_warnings_refs, [ $cur_src_file, [ @warning_refs ] ]
if ($cur_src_file ne '');
$cur_src_file = $src_file;
@warning_refs = ();
}
my $src_line_nr = $ref->{ln};
my $col_nr = $ref->{locStartPos};
my $msg_level = $ref->{sev};
my $msg_id = $ref->{rule};
my $msg_text = un_escape_characters( $ref->{msg});

push @warning_refs, [ $src_line_nr, $col_nr, $msg_level, $msg_id, $msg_text ];

}
push @source_warnings_refs, [ $cur_src_file, [ @warning_refs ] ]
if ($cur_src_file ne '');
}


return @source_warnings_refs;
}




sub CPPTESTGOUT_get_fatals()
{
my @source_fatals_refs;



my $CppAnalysisProblem_ref = $GOUT_REF->{CodingStandards}->[0]->{TestingProcessProblems}->[0]->{CppAnalysisProblem};
if (defined $CppAnalysisProblem_ref)
{

foreach my $ref (@{$CppAnalysisProblem_ref})
{

my $filespec = $ref->{tested};		    # /$project/src/$file

my ($src_file) = $filespec =~ m!.*/(.*)$!;

my @fatal_refs;

foreach my $error_list_ref (@{$ref->{ErrorList}})
{

foreach my $error_ref (@{$error_list_ref->{Error}})
{

my @lines = split( "\r\n", $error_ref->{val});






















my $i = 1;
my $nr_lines = @lines - 1;	# Error during parsing file:...
$lines[-1] = '';
while ($i < $nr_lines)
{
my @msg_lines;
my $again;
do
{
$again = 1;
my $line = $lines[$i++];
if ($line eq '')
{
$again = 0;
} elsif ($line =~ /^\s*\^$/)
{
$again = 0;
push @msg_lines, $line;
$i++
if ($lines[$i] eq '');
} else
{
push @msg_lines, $line;
}
} while ($again);




my $first_line = $msg_lines[0];
for (my $i = 1; $i < @msg_lines - 2; $i++)
{
$first_line .= $msg_lines[$i];
}
$first_line =~ s/\s+/ /g;

my $src_line_nr;
my $col_nr;
my $msg_level;
my $msg_id;
my $msg_text;
my $msg_type;
($src_line_nr, $msg_type, $msg_text) = $first_line =~ / line (\d+): (fatal error|error|note): (.*)/;
($src_line_nr, $msg_type, $msg_text) = $first_line =~ /.*:(\d+):\d+: (fatal error|error|note): (.*)/
if (!defined $msg_text);
if (defined $msg_text)
{
if ($msg_type eq 'fatal error')
{
$col_nr = length( $msg_lines[-1]) - 1;
$msg_level = 6;
$msg_id = ($msg_lines[0] =~ / No such file or directory/) ? 'COMPILE_INCLUDE' : 'COMPILE_FATAL';
} elsif ($msg_type eq 'error')
{
$col_nr = length( $msg_lines[-1]) - 3;
$msg_level = 6;
$msg_id = 'COMPILE_ERROR';
} elsif ($msg_type eq 'note')
{
$col_nr = length( $msg_lines[-1]) - 3;
$msg_level = 6;
$msg_id = 'COMPILE_ERROR';
} else
{
ENV_sig( E => "Unknown message format (not 'fatal error' or 'error' or 'note')", @msg_lines, $first_line);
}
$msg_text = un_escape_characters( $msg_text);
push @fatal_refs, [ $src_line_nr, $col_nr, $msg_level, $msg_id, $msg_text ];

} else
{
if ($msg_lines[0] =~ /error/)
{
ENV_sig( E => "$src_file:", @msg_lines);
push @fatal_refs, [ 0, -1, 6, 'COMPILE_ERROR', $msg_lines[0] ];
} else
{
ENV_say( 1, "$src_file:", @msg_lines);
}
}
}
}
}
push @source_fatals_refs, [ $src_file, \@fatal_refs ];
}
}


return @source_fatals_refs;
}





sub CPPTESTGOUT_get_rules()
{
my @msg_def_refs;


my $Rule_ref = $GOUT_REF->{CodingStandards}->[0]->{Rules}->[0]->{RulesList}->[0]->{Rule};
if (defined $Rule_ref)
{


foreach my $ref (@{$Rule_ref})
{
my $msg_id = $ref->{id};
my $level = $ref->{sev};
my $msg_text = $ref->{desc};
push @msg_def_refs, [ $msg_id, $level, $msg_text ];
}
} else
{
ENV_sig( E => "No Rules found in '$GOUT_FILE_SPEC'");
}


return @msg_def_refs;
}




sub un_escape_characters($)
{
my ($line) = @_;

$line =~ s/&lt;/</g;
$line =~ s/&gt;/>/g;
$line =~ s/&amp;/&/g;
$line =~ s/&quot;/"/g;

return $line;
}

1;



