#===============================================================
#
#   prqa.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Generic package for qac and qac++
#===============================================================
#
#   Call:
#	arg_1	= file_name	(without type)
#	arg_2	= in_filespec	(relative file spec - file_name + type
#	arg_3	= out_file_base (relative file spec - with in file type)
#	arg_4	= out_file_dir	(relative file spec - directory only)
#	arg_5	= out_filetype
#	arg_6-N = command_flags (from command line)
#   Environment:
#	General:
#	    GBS_INCS		Include paths ( -I<dir> ... )
#	    GBS_SYSINCS 	System include paths ( -SI<dir> ... )
#	    GBS_AUDIT_FLAGS	Passed from session
#	    GBS_FLAGS		From flags_<type> files
#	    GBS_SYSFLAGS	From bld_flags_<type> file
#	    GBS_AUDIT_OPTIONS	($MUST_SHOW_STDOUT)
#	Application:
#	    GBS_TARGET
#	    GBS_AUDIT
#	    GBSEXT_tool_REL
#	    GBSEXT_tool_PATH
#	    GBS_COMPONENT
#	    toolHELPFILES   (optional)
#============================================================================
package prqa;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PRQA_init
PRQA_set
PRQA_pre
PRQA_run
PRQA_post
PRQA_load
);
}




use glo::env;
use glo::slurp;
use glo::parse_c;
use gbsenv;
use scadef;
use scafile;

use prqaglo;
use gbsbgscasub;




sub PRQA_init($$$);
sub PRQA_set($$);
sub PRQA_pre($$);
sub PRQA_run($$$@);
sub PRQA_post($$);
sub PRQA_load();

sub handle_unkn_msg($$$);
sub do_metrics();
sub handle_st_data($$);




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');
my $GBS_AUDIT_PLUGIN = ENV_getenv( 'GBS_AUDIT_PLUGIN');




my $PLUGIN_NAME;	    # qac or qacpp
my $GBSEXT_PLUGIN_PATH;
my $GBS_PLUGIN_PATH;	    # $GBS_PLUGIN_ROOT/$plugin/$plugin_rel

my $FOR_QAC;			# bool
my $FOR_QACPP;			# bool
my $PLUGIN_NAME_UC;		# QAC or QACPP
my $GBS_SYSAUDIT_AUDIT_PATH;
my $PLUGIN_HELPFILES_PATH;
my $GBSEXT_PLUGIN_BIN_PATH;	# depends on GBSEXT_PLUGIN_USE_STUB




my @GBS_DEFAULTS;	    # @QAC_SETTINGS / @QACPP_SETTINGS, @ANA_SETTINGS
my @COMPILER_SETTINGS;	    #
my @GBS_SETTINGS;	    # @ERRDISP_SETTINGS




my $GOUT_FILE_SPEC;

my $SOURCE_FILE;
my $SOURCE_FILE_SPEC;
my $OS_SOURCE_FILE_SPEC;
my $MET_FILE_SPEC;	    # qac/qacpp output file

my $USR_MSG_FILESPEC;	    # if undef: file does not exist




my %MET_DATA;

my @MET_DATA_ORDER;




my $USR_MSG_FILE_TYPE = '.gbs';

my @QAC_SETTINGS = (		# QAC
'-comment a',
);

my @QACPP_SETTINGS = (		# QACPP
'-nomsg 1599,1596',		# disable: 'Included from here' and 'Previously declared here'
);

my @ANA_SETTINGS = (		# QAC & QACPP (-D && -Q)	TBS: Move to option-file??
'-D PRQA',
);

my @PAL_SETTINGS = (		# PAL
);

my @ERRDSP_SETTINGS = (		# ERRDSP
'-maxcount 50',		# Limit the number of times that any particular warning message is displayed
'-references+',		# The optional verbose/reference text specified in the message file is displayed
'-messagesonly+',		# -m+ == diaglist, -m- == annsrc
'-format \n*GBS*%F*%l*%C*%u*%g*%n*%t\n--%v\n--%B',


'-nomsg 1599,1596',		# disable: 'Included from here' and 'Previously declared here'
);

my %IGNORE_METRICS = map { $_ => 1 } PRQAGLO_get_ignore_metrics();






sub PRQA_init($$$)
{
(	$PLUGIN_NAME,		# qac or qacpp
$GBSEXT_PLUGIN_PATH,
$GBS_PLUGIN_PATH,	# $GBS_PLUGIN_ROOT/$plugin/$plugin_rel
) = @_;


$FOR_QAC = ($PLUGIN_NAME eq 'qac') ? 1 : 0;
$FOR_QACPP = ($PLUGIN_NAME eq 'qacpp') ? 1 : 0;
$PLUGIN_NAME_UC = uc $PLUGIN_NAME;
$GBS_SYSAUDIT_AUDIT_PATH = "$GBS_ROOT_PATH/sysaudit/$GBS_AUDIT";




{
my $gbsext_use_stub = ENV_getenv( "GBSEXT_${PLUGIN_NAME}_USE_STUB");
$gbsext_use_stub = ($gbsext_use_stub =~ /^(1|Y|y)/) ? 1 : 0;
if ($gbsext_use_stub)
{
ENV_say( 1, "** Running in Development Mode from STUBS...");
$GBSEXT_PLUGIN_BIN_PATH = "$GBS_SCRIPTS_PATH/STUBS/qac/bin";
ENV_setenv( "GBSEXT_${PLUGIN_NAME_UC}_BIN_PATH" => ENV_os_path( $GBSEXT_PLUGIN_BIN_PATH));
}
}




PRQAGLO_init( $PLUGIN_NAME);




$PLUGIN_HELPFILES_PATH = ENV_getenv( "${PLUGIN_NAME_UC}HELPFILES");
$PLUGIN_HELPFILES_PATH = "$GBSEXT_PLUGIN_PATH/help"
if ($PLUGIN_HELPFILES_PATH eq '' && -d "$GBSEXT_PLUGIN_PATH/help");

$ENV{"${PLUGIN_NAME_UC}BIN"} = ENV_os_path( "$GBSEXT_PLUGIN_PATH/bin");
$ENV{"${PLUGIN_NAME_UC}TEMPFILES"} = $ENV{GBS_TMP_PATH};
$ENV{"${PLUGIN_NAME_UC}HELPFILES"} = $PLUGIN_HELPFILES_PATH;

my @plugin_msg_filespecs = PRQAGLO_get_msg_filespecs();

return ( \&handle_unkn_msg, [ @plugin_msg_filespecs ], \&PRQAGLO_read_msg_file, 0);	    # $help_file_required = 0
}





sub PRQA_set($$)
{
my ($compiler_settings_ref,
$extra_gbs_defaults_ref,	# e.g. -quiet options
) = @_;




@COMPILER_SETTINGS = @{$compiler_settings_ref};




$USR_MSG_FILESPEC = "$GBS_SYSAUDIT_AUDIT_PATH/$PLUGIN_NAME.usr$USR_MSG_FILE_TYPE";	# .gbs
if (-e $USR_MSG_FILESPEC)
{
push @ERRDSP_SETTINGS, "-usrfile $USR_MSG_FILE_TYPE";	    # .gbs
push @ERRDSP_SETTINGS, "-usrpath $GBS_SYSAUDIT_AUDIT_PATH";
} else
{
$USR_MSG_FILESPEC = undef;
}
@GBS_SETTINGS = (@ERRDSP_SETTINGS);





push @ANA_SETTINGS, @{$extra_gbs_defaults_ref};

@GBS_DEFAULTS = (($FOR_QAC) ? @QAC_SETTINGS : @QACPP_SETTINGS, @ANA_SETTINGS);


}





sub PRQA_pre($$)
{
my ($src_path,
$source_file,
) = @_;
my @printable_out_files;

$SOURCE_FILE = $source_file;
$SOURCE_FILE_SPEC = "$src_path/$source_file";
$OS_SOURCE_FILE_SPEC = ENV_os_path( $SOURCE_FILE_SPEC);
my $out_file_path = "../aud/$GBS_AUDIT/$GBS_TARGET";
my $out_file_base = "$out_file_path/$source_file";
$GOUT_FILE_SPEC = "$out_file_base.gout";




$ENV{"${PLUGIN_NAME_UC}OUTPATH"} = ENV_os_path( $out_file_path);

$MET_FILE_SPEC = "$out_file_base.met";

SCAFILE_garg_write( '* ', \@GBS_DEFAULTS, \@COMPILER_SETTINGS, \@GBS_SETTINGS);

if ($FOR_QAC)
{
ENV_setenv( "GBS_QAC_FLAGS" => "@QAC_SETTINGS @ANA_SETTINGS @COMPILER_SETTINGS");
} else
{
ENV_setenv( "GBS_QACPP_FLAGS" => "@QACPP_SETTINGS @ANA_SETTINGS @COMPILER_SETTINGS");
}

my $style_filespec = ENV_shortest_path( undef, "$GBS_ROOT_PATH/sysaudit/$GBS_AUDIT/style_ALL.nrf");
ENV_setenv( GBS_PAL_FLAGS => "@PAL_SETTINGS -nrf $style_filespec");

ENV_setenv( GBS_ERRDSP_FLAGS => "@ERRDSP_SETTINGS");

@printable_out_files = ( $GOUT_FILE_SPEC );

return @printable_out_files;
}






sub PRQA_run($$$@)
{
my ($src_files_ref,		#
$must_print_commands,	# scripts should print their own command(s). '-' == undef
$out_files_ref,		#
@command_data_lines	# ($command_type, $command_line, $ok_values, $exec_condition), ...

) = @_;
my $rc = 0;

ENV_sig( F => "The PRQA_run function is obsolete");

return $rc;
}





sub PRQA_post($$)
{
my ($src_files_ref,	# one file only
$unix_style_search,
) = @_;
my $rc = 0;	    # 1 = contains FATAL warnings, 2 = Execution error


my @warning_refs;


my @total_refs;


my @metric_refs;





{
























my @lines = SLURP_file( $GOUT_FILE_SPEC);

my $nr_lines = @lines;
my $i = 0;
while ($i < $nr_lines)
{



my $line1 = $lines[$i++];		# *GBS*...
next if ($line1 eq '');
my $line2 = $lines[$i++];		# --<Extra message>
my $line3 = $lines[$i++];		# --<file>
my $line4 = $lines[$i++];		# 	--<internal message>

if ($line1 !~ /^\*GBS\*/ ||
$line2 !~ /^--/ ||
$line3 !~ /^--/ )
{
ENV_say( 0, "=== $GOUT_FILE_SPEC");
map { ENV_say( 0, sprintf( "%7d $lines[$_]", $_+1)) } $i-4..$i-1;
ENV_say( 0, "===");
ENV_sig( EE => "Invalid .gout file format. ");
}
my (undef, $gbs_text, $os_filespec, $line_nr, $col_nr, $not_used_msg_depth, $msg_level, $msg_nr, $msg_text) = split( '\*', $line1, 9);

my $src_line_nr;
my $level_text;
my $help_file_url = '';	# filled by GBSBGSCA
my @extra_src_lines;



my $os_filespec_nospace;
if (ENV_paths_equal( $os_filespec, $OS_SOURCE_FILE_SPEC))   # In Source
{
$os_filespec_nospace = '-';
$src_line_nr = $line_nr;
} else					    # In include
{
$os_filespec_nospace = ENV_os_path( ENV_perl_path_nospace( $os_filespec));
$src_line_nr = -1;
}

$level_text = ($msg_level == 0) ? 'INFO' :
($msg_level == 9) ? 'ERROR' :
($msg_level  > 9) ? 'SEVERE ERROR' :
'WARNING';

my $msg_id = substr( "    $msg_nr", -4);    # Add leading spaces

my $extra_msg_line = substr( $line2, 2);	# Skip '--'
if ($extra_msg_line ne '')
{
@extra_src_lines = ($extra_msg_line);
}

push @warning_refs, [ $src_line_nr, $col_nr, $os_filespec_nospace, $line_nr, $msg_level, $level_text, $msg_id, $msg_text, $help_file_url, @extra_src_lines ];


}
}




my ($total_refs_ref, $metric_refs_ref) = do_metrics();




$rc = GBSBGSCASUB_file_store( $SOURCE_FILE, $unix_style_search, \@warning_refs, $total_refs_ref, $metric_refs_ref);

return $rc;
}




sub do_metrics()
{

my @total_refs;


my @metric_refs;


my $cur_filespec = '';
my $cur_st_size;
my $cur_st_name;
my $cur_st_type;    # 1 = Function, 2 = Class, 3 = File
my @cur_st_refs;    # [ $st_name, $st_value ]

my $s_nr_lines = undef;
my $t_nr_lines = undef;
my $s_nr_funcs = undef;
my $t_nr_funcs = undef;
my $s_nr_class = undef;
my $t_nr_class = undef;
my $s_nr_const = undef;
my $t_nr_const = undef;

%MET_DATA = ();
@MET_DATA_ORDER = ();




$s_nr_const = PARSE_C_get_nr_distinct_words( PARSE_C_new( $SOURCE_FILE_SPEC), 'const', 1);	# $cleanup
$t_nr_const = -1;




foreach my $line (SLURP_file( $MET_FILE_SPEC), '<S>')
{
next if ($line eq '');
next if (substr( $line, 0, 3) ne '<S>');
if (substr( $line, 0, 5) eq '<S>ST')
{



my ($full_key, $value) = split( ' ', $line, 2);
my $metric_name = substr( $full_key, 5);

if ($metric_name eq 'FIL')          # <S>STFIL
{
$cur_filespec = ENV_perl_path_nospace( $value);

} elsif ($metric_name eq 'NAM')     # <S>STNAM  (function or class)
{
my $name = ENV_perl_path_nospace( $value);


if ($name eq $SOURCE_FILE_SPEC)
{

$cur_st_type = 3;   # File
} else
{
$cur_st_name = $name;
if (substr( $name , 0, 2) eq '::' && substr( $name, -1, 1) ne ')' )
{

$cur_st_type = 2;   # Class

$s_nr_class++
if ($cur_filespec eq $SOURCE_FILE_SPEC);
} else
{

$cur_st_type = 2;   # Func

$s_nr_funcs++
if ($cur_filespec eq $SOURCE_FILE_SPEC);
}
}
} elsif ($metric_name eq 'XLN')     # <S>STXLN  (Number of Executable Statements)
{
$cur_st_size = $value;
push @cur_st_refs, [ $metric_name, $value ];
} elsif (SCADEF_metric_is_enabled( $GBS_AUDIT_PLUGIN, $metric_name))
{
my ($first_value) = split( ' ', $value);
push @cur_st_refs, [ $metric_name, $first_value ];
} elsif (exists $IGNORE_METRICS{$metric_name})
{

} else
{
ENV_sig( W => "Unknown met-key '$full_key'");
}
} elsif ($line eq '<S>')
{



if (@cur_st_refs)
{
if ($cur_st_type == 1)
{

foreach my $ref (@cur_st_refs)
{
handle_st_data( $ref->[0], $ref->[1])
if ($cur_st_size > 0);
}
} elsif ($cur_st_type == 2)
{

foreach my $ref (@cur_st_refs)
{
handle_st_data( $ref->[0], $ref->[1]);
}
} else
{

foreach my $ref (@cur_st_refs)
{
handle_st_data( $ref->[0], $ref->[1]);
}
}
$cur_st_size = 0;
$cur_st_name = '';
$cur_st_type = 0;
@cur_st_refs = ();      # [ $st_name, $st_value ]
}
} else
{

}
}

$MET_DATA{'TPP'} = [ 0, 0, 0 ] if (!exists $MET_DATA{'TPP'});   # Total source lines before pre-processing
$MET_DATA{'TLN'} = [ 0, 0, 0 ] if (!exists $MET_DATA{'TLN'});   # Total source lines after pre-processing

$s_nr_lines = $MET_DATA{'TPP'}->[0];    # Total source lines before pre-processing
$t_nr_lines = $MET_DATA{'TLN'}->[0];    # Total source lines after pre-processing

my $met_fnc = $MET_DATA{'FNC'}->[0];
if (defined $met_fnc && $s_nr_funcs != $met_fnc)
{
ENV_sig( E => "s_nr_funcs=$s_nr_funcs != met_fuc=$met_fnc");
}




@total_refs = (
[ 0, $s_nr_lines, $t_nr_lines ],	# Lines
[ 1, $s_nr_funcs, $t_nr_funcs ],	# Funcs
[ 2, $s_nr_class, $t_nr_class ],	# Class
[ 3, $s_nr_const, $t_nr_const ],	# Const
);
foreach my $ref (@total_refs)
{
$ref->[1] = -1
if (! defined $ref->[1]);	# $s_nr_lines/funcs/class/const
$ref->[2] = -1
if (! defined $ref->[2]);	# $t_nr_lines/funcs/class/const
}




foreach my $metric_name (@MET_DATA_ORDER)
{
my ($min, $max, $total) = @{$MET_DATA{$metric_name}};
push @metric_refs, [ $metric_name => $min, $max, $total ];
}

return ( \@total_refs, \@metric_refs );
}





sub handle_st_data($$)
{
my ($metric_name,
$value,
) = @_;

no integer;

if (exists $MET_DATA{$metric_name})
{
my ($min, $max, $total) = @{$MET_DATA{$metric_name}};
$min = $value if ($value < $min);
$max = $value if ($value > $max);
$total += $value;
$MET_DATA{$metric_name} = [ $min, $max, $total ];
} else
{
$MET_DATA{$metric_name} = [ $value, $value, $value ];
push @MET_DATA_ORDER, $metric_name;
}

use integer;
}





sub handle_unkn_msg($$$)
{
my ($msg_id,		# defined
$msg_level,		# defined
$level_text,		# defined
) = @_;
my $help_file_url = '';


$help_file_url = "$msg_id.html";		# Create a 'msg does not exist.html' file?  TBS

return ($msg_level, $level_text, $help_file_url);
}





sub PRQA_load()
{
ENV_say( 1, "Loaded");
}

1;



