#===============================================================
#
#   prqaglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#   Generic package for qac and qac++
#===============================================================
package prqaglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PRQAGLO_init
PRQAGLO_get_msg_filespecs
PRQAGLO_read_msg_file
PRQAGLO_get_ignore_metrics
);
}




use glo::env;
use glo::slurp;
use gbsenv;




sub PRQAGLO_init($);
sub PRQAGLO_get_msg_filespecs();
sub PRQAGLO_read_msg_file($);
sub PRQAGLO_get_ignore_metrics();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');




my $USR_MSG_FILE_TYPE = '.gbs';




my $PLUGIN_NAME;	    #   qac or qacpp

my $PLUGIN_NAME_UC;	    #   QAC or QACPP
my $GBS_AUDIT;
my $GBSEXT_PLUGIN_PATH;
my $GBSEXT_PLUGIN_REL;

my $GBS_SYSAUDIT_AUDIT_PATH;	# "$GBS_ROOT_PATH/sysaudit/$GBS_AUDIT";

my $MSG_FILE_SPEC;		# "$GBSEXT_PLUGIN_PATH/bin/$PLUGIN_NAME.msg";		# must exist
my $MSG_USR_FILE_SPEC;		# "$GBS_SYSAUDIT_AUDIT_PATH/$PLUGIN_NAME.usr.gbs";	# may exist




sub PRQAGLO_init($)
{
($PLUGIN_NAME,		# qac or qacpp
) = @_;

ENV_sig( F => "plugin_name ($PLUGIN_NAME) mut be qac or qacpp")
if ($PLUGIN_NAME ne 'qac' && $PLUGIN_NAME ne 'qacpp');

$PLUGIN_NAME_UC = uc $PLUGIN_NAME;

$GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');

$GBSEXT_PLUGIN_REL = ENV_getenv( "GBSEXT_${PLUGIN_NAME}_REL");
my @tool_rel = split( /\./, $GBSEXT_PLUGIN_REL);
if ($PLUGIN_NAME eq 'qac')
{
ENV_sig( EE => "QAC Release ($GBSEXT_PLUGIN_REL) not supported. Minimum required is 6.0")
if ($tool_rel[0] < 6);
} else  # $PLUGIN_NAME eq 'qacpp'
{
ENV_sig( EE => "QAC++ Release ($GBSEXT_PLUGIN_REL) not supported. Minimum required is 2.0")
if ($tool_rel[0] < 2);
}

$GBS_SYSAUDIT_AUDIT_PATH = "$GBS_ROOT_PATH/sysaudit/$GBS_AUDIT";

$GBSEXT_PLUGIN_PATH = ENV_getenv_perl_path( "GBSEXT_${PLUGIN_NAME_UC}_PATH");
ENV_sig( F => "$PLUGIN_NAME: GBSEXT_${PLUGIN_NAME_UC}_PATH ($GBSEXT_PLUGIN_PATH): No such directory")
if (! -d $GBSEXT_PLUGIN_PATH);

$MSG_FILE_SPEC     = "$GBSEXT_PLUGIN_PATH/bin/$PLUGIN_NAME.msg";		# must exist
$MSG_USR_FILE_SPEC = "$GBS_SYSAUDIT_AUDIT_PATH/$PLUGIN_NAME.usr.gbs";	# may exist
}




sub PRQAGLO_get_msg_filespecs()
{
return ($MSG_FILE_SPEC, $MSG_USR_FILE_SPEC);
}




sub PRQAGLO_read_msg_file($)
{
my ($help_data_wanted,	# bool. Always 0. Not used
) = @_;
my @msg_def_refs;






my %levels;



my @lines = SLURP_file( $MSG_FILE_SPEC);
if (-e $MSG_USR_FILE_SPEC)
{
ENV_say( 1, "Reading '$MSG_USR_FILE_SPEC'...");
push @lines, SLURP_file( $MSG_USR_FILE_SPEC)
}
while (@lines)
{









my $line = shift @lines;

next if ($line eq '');
my $fc = substr( $line, 0, 1);
next if ($fc eq '*');
if ($fc eq '#')
{

if (substr($line,0,10) eq '#levelname')
{

my (undef, $level_number, $level_name) = split( ' ', $line);
} elsif (substr($line,0,7) eq '#define')
{

my (undef, $name, $level_number, $comment) = split( ' ', $line, 4);
$levels{$name} = $level_number;

} else
{
ENV_say( 1, "MSG-file ERROR: Unknown # directive",
$line);
}
} elsif ($fc ge '0' && $fc le '9')
{

my ($msg_nr, $level_text, $rest) = split( ' ', $line, 3);
$msg_nr *= 1;		# get rid of leading zeros
my $level = $levels{$level_text};
if (!defined $level)
{
ENV_say( 1, "Unknown level '$level_text' in line:");
ENV_say( 0, $line);
$level = 0;
}

$line = $rest;
my @msg_texts;
my @verbose_texts;
my $verbose = 0;
my $again = 1;
do
{
if (substr( $line, -1, 1) eq '\\')
{
if (substr( $line, -2, 1) eq '\\')
{

push @msg_texts, substr( $line, 0, -2);
$verbose = 1;
} else
{

if ($verbose)
{
push @verbose_texts, substr( $line, 1, -1);
} else
{
push @msg_texts, substr( $line, 1, -1);
}
}
$line = shift @lines;

} else
{
if ($verbose)
{
push @verbose_texts, $line;
} else
{
push @msg_texts, $line;
}
$again = 0;
}
} while ($again);

my $msg_text = "@msg_texts";
$msg_text = substr( $msg_text, 0, 77) . '...'
if (length $msg_text > 80);
my $help_file_url = sprintf( '%04d.html', $msg_nr);

my $msg_id = substr( "    $msg_nr", -4);
push @msg_def_refs, [ $msg_id, $level, $level_text, $msg_text, $help_file_url, "@verbose_texts" ];

} else
{
ENV_say( 1, "MSG-file ERROR: Unknown format",
$line);
}
}
return @msg_def_refs;
}




sub PRQAGLO_get_ignore_metrics()
{
return (
'BCS',	# Header: BASIC COCOMO STATISTICS
'COM',  # Header: COMMENT BASED STATISTICS

'FIL',  # File
'NAM',  # Name (File, Function or Class)
'NTB',  # Header: NON-TOKEN BASED STATISTICS
'TKB',  # Header: TOKEN BASED STATISTICS
'MT' ,  # ?????
);
}

1;


