#===============================================================
#
#   doxygen.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
#
#   Call:
#	arg_1	= $action = 'full' or 'api'
#	arg_2	= $subsys_def_list
#	arg_3	= $out_filespec
#	arg_4	= $out_file_path
#============================================================================
use strict;
use warnings;
use integer;
print "*DOXYGEN @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::file;
use glo::spit;
use gbsenv;




sub run_doxygen();
sub get_command_arguments();




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SYSTEM_NAME = ENV_getenv( 'GBS_SYSTEM_NAME');
my $GBS_ROOT_PARENT = ENV_getenv( 'GBS_ROOT_PARENT');
my $GBS_TOOL = ENV_getenv( 'GBS_TOOL');

my $GBSEXT_DOXYGEN_PATH = ENV_getenv_perl_path( 'GBSEXT_DOXYGEN_PATH');
my $GBSEXT_DOXYGEN_BIN_PATH = ENV_getenv_perl_path( 'GBSEXT_DOXYGEN_BIN_PATH');
my $GBSEXT_DOXYGEN_REL = ENV_getenv( 'GBSEXT_DOXYGEN_REL');




my $ACTION;		    # 'full' or 'api'
my $SUBSYS_DEF_LIST;	    # comma-list ($subsys:$ss_type)
my $OUT_FILE_SPEC;	    # GBS_FILE_PATH/GBS_TOOL.html
my $OUT_FILE_PATH;	    # GBS_SILO_TOOLS_PATH/GBS_TOOL




my $OUT_FILE_PARENT_PATH;






$| = 1;
my $RC = 0;
GBSENV_init();

get_command_arguments();
$OUT_FILE_PARENT_PATH = ENV_parent_path( $OUT_FILE_PATH);

ENV_sig( EE => "No GBS_ROOT_PATH defined")
if (!$GBS_ROOT_PATH);




run_doxygen();




ENV_exit( $RC);




END
{
ENV_print_end_msg(1);
}




sub run_doxygen()
{


my $inputs_filespec = "$OUT_FILE_PATH/doxygen_inputs_${ACTION}_$$.txt";




ENV_say( 1, "Cleanup directories...");
FILE_del_tree( $OUT_FILE_PATH, undef, 0);




ENV_say( 1, "Collecting directories...");
ENV_chdir( "$GBS_ROOT_PATH/dev");

my @subsys_defs = split( ',', $SUBSYS_DEF_LIST);

my @dirs;
my $doxygen_recursive = 'NO';
my @doxygen_file_patterns;
foreach my $subsys_def (@subsys_defs)
{
my ($subsys, $ss_type) = split( ':', $subsys_def);
if ($ss_type eq 'GBS')
{
push @dirs, glob( "./$subsys/comp/*/src")
unless ($ACTION eq 'api');
push @dirs, glob( "./$subsys/comp/*/inc");
push @dirs, glob( "./$subsys/comp/*/loc")
unless ($ACTION eq 'api');
} else
{
push @dirs, "./$subsys/app";
$doxygen_recursive = 'YES';
}
}

ENV_sig( EE => "No valid directories found. <SubSystem>/tools/$GBS_TOOL directory missing?")
if (!@dirs);







if ($doxygen_recursive eq 'YES' && $ACTION eq 'api')
{
@doxygen_file_patterns = qw( *.h *.hh *.hxx *.hpp *.h++ *.inc);
}




{
my @lines;
push @lines, "INPUT = \\";
push @lines, map { "\t$_ \\" } @dirs;
push @lines, '';
SPIT_file_nl( $inputs_filespec, \@lines);
}




my $config_file = "$GBS_ROOT_PATH/systools/doxygen/doxygen.conf";
$config_file = "$GBS_SCRIPTS_PATH/plugins/tools/doxygen/doxygen.conf"
if (! -e $config_file);
ENV_sig( F => "doxygen.conf ($config_file) not found")
if (! -e $config_file);





ENV_say( 1, "Running $GBS_TOOL (doxygen [$GBSEXT_DOXYGEN_REL])...");

ENV_setenv( GBS_DOXYGEN_STRIP_FROM_PATH => $GBS_ROOT_PATH);
ENV_setenv( GBS_DOXYGEN_PROJECT_NAME => "$GBS_SYSTEM_NAME [$GBS_ROOT_PARENT] - $ACTION");
ENV_setenv( GBS_DOXYGEN_INCLUDE => $inputs_filespec);
ENV_setenv( GBS_DOXYGEN_FILE_PATTERNS => "@doxygen_file_patterns")
if (@doxygen_file_patterns);					    # do not set empty string ("")
ENV_setenv( GBS_DOXYGEN_RECURSIVE => $doxygen_recursive);
ENV_setenv( GBS_DOXYGEN_OUTPUT_DIRECTORY => $OUT_FILE_PARENT_PATH);
ENV_setenv( GBS_DOXYGEN_HTML_OUTPUT => $GBS_TOOL);
ENV_say( 1, 'Doxygen EnvVars:');
foreach my $env (sort keys %ENV)
{
if (substr( $env, 0, 11) eq 'GBS_DOXYGEN')
{
ENV_say( 2, "$env => " . ENV_getenv( $env));
}
}




my $command = ($GBSEXT_DOXYGEN_PATH) ? "$GBSEXT_DOXYGEN_BIN_PATH/doxygen" : 'doxygen';
my $command_ref = ENV_command_ref( $command, [ $config_file ]);
$RC = ENV_system( $command_ref, undef);
if ($RC == 0)
{
unlink( $inputs_filespec);
}
}




sub get_command_arguments()
{
( $ACTION,			    # 'full' or 'api'
$SUBSYS_DEF_LIST,		    # Commalist
$OUT_FILE_SPEC,
$OUT_FILE_PATH,
) = ARGS_get( [4,4], 'perl doxygen.pl [ full | api ] <subsys_def_list> <out_filespec> <out_file_path>', undef);

if (defined $ACTION)
{
if ($ACTION ne 'full' && $ACTION ne 'api')
{
ARGS_error_exit( "Action must be 'full' or 'api' ($ACTION)");
}
}

$OUT_FILE_SPEC = ENV_perl_paths( $OUT_FILE_SPEC);
$OUT_FILE_PATH = ENV_perl_paths( $OUT_FILE_PATH);

my $expected_spec = "$OUT_FILE_PATH/index.html";
ENV_sig( F => "<out_filespec> mismatch",
"Expected:        $expected_spec",
"<out_filespec>: $OUT_FILE_SPEC")
if ($OUT_FILE_SPEC ne $expected_spec);
}



