#===============================================================
#
#   settings.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package settings;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SETTINGS_set_check
SETTINGS_uninstall_version
SETTINGS_uninstall_gbs
SETTINGS_get_setup_files
);
}




BEGIN
{
if ($^O eq 'MSWin32')
{
require glo::winreg;
import  glo::winreg;
} else
{
require glo::bash;
import  glo::bash;
}
}
use glo::bash;
use glo::env;
use glo::file;
use glo::shortcut;
use gbsenv;




sub SETTINGS_set_check($$);
sub SETTINGS_uninstall_version($);
sub SETTINGS_uninstall_gbs();
sub SETTINGS_get_setup_files($);

sub win32_check_registry();
sub linux_check_login_files();
sub check_predefined_envvar($$$);
sub copy_setup_files();




my $IS_WIN32 = (ENV_is_win32());
my $IS_LINUX = ! $IS_WIN32;
my $IS_DEVELOPMENT = ENV_is_development();




my $GBS_BOOT_PATH = ENV_getenv_perl_path( 'GBS_BOOT_PATH');	# %APPDATA%/.gbs or ~/.gbs
my $GBS_BASE_PATH = ENV_getenv_perl_path( 'GBS_BASE_PATH');	# %MYDOCUMENTS%/.gbs/base or ~/.gbs/base or other
$GBS_BASE_PATH =~ s/GBS~\d$/.gbs/
if ($IS_WIN32);

my $SHELL_FILE_TYPE = ENV_shell_filetype();

my @LINUX_ALIAS_REFS = (

[ gbs	    => qq{. "$GBS_BOOT_PATH/gbsinit.sh" \$*} ],
[ gbshelp	    => qq{$GBS_BOOT_PATH/define.sh cur perl gbshelp.pl \$*} ],
[ gbssetup	    => qq{$GBS_BOOT_PATH/define.sh cur source gbssetup.sh \$*} ],
[ gbsuninstall  => qq{$GBS_BOOT_PATH/define.sh cur perl gbsuninstall.pl \$*} ],
);

my @LINUX_BASHRC_LINES = (
qq{if [[ -e "/tmp/\$USER/gbs_pre_define_\$PPID.sh" ]]},
qq{then},
qq{    \\source "/tmp/\$USER/gbs_pre_define_\$PPID.sh"},
qq{    \\rm "/tmp/\$USER/gbs_pre_define_\$PPID.sh"},
qq{fi},
);


my $STARTMENU_PATH = ENV_get_startmenu_path( 0);    # user_path



my $WIN32_SHORTCUT_ARGS_FORMAT = '/K %s';
my $LINUX_SHORTCUT_ARGS_FORMAT = '-title "GBS xterm" -geometry 100x40+100+100 -bg white -fg darkblue -sb -ls -e "bash %s; bash -login"';

my $SHORTCUT_ARGS_FORMAT = ($IS_WIN32) ? $WIN32_SHORTCUT_ARGS_FORMAT : $LINUX_SHORTCUT_ARGS_FORMAT;

my $SETUP_ACTION = ($IS_WIN32) ? 'exec gbssetup.bat' : 'source gbssetup.sh';
my $UNINSTALL_ACTION = 'perl gbsuninstall.pl';
my @SHORTCUT_REFS = (

[ 'GBS Startup [%s]',   'Application', 'gbs',	      [ 'DESKTOP', 'STARTMENU/GBS' ], 'Startup a GBS Command Window [%s]' ],
[ 'GBS Help [%s]',      'Link', 'doc/index.html',	      [ 'DESKTOP', 'STARTMENU/GBS' ], 'Startup the GBS Help [%s]' ],
[ 'GBS Setup [%s]',	    'Application', $SETUP_ACTION,     'STARTMENU/GBS', 'Startup the GBS Setup function [%s]' ],
[ 'GBS Uninstall [%s]', 'Application', $UNINSTALL_ACTION, 'STARTMENU/GBS', 'Startup the GBS Uninstall function [%s]' ],
);




my $THIS_GBS_REL;
my $THIS_GBS_SCRIPTS;




sub SETTINGS_set_check($$)
{
my ($new_scripts_root_path,
$new_rel,
) = @_;

ENV_say( 1, "Set/Check Startup Settings...");

$THIS_GBS_REL = $new_rel;					# may have new value
$THIS_GBS_SCRIPTS = "$new_scripts_root_path/$new_rel";	# may have new value





copy_setup_files();




if ($IS_WIN32)
{



win32_check_registry();
} else  # LINUX
{
linux_check_login_files();
}




if (ENV_get_desktop_path( 0) ne '')		# user_path
{




ENV_say( 1, "Checking Shortcuts...");
my $update_count = 0;
my $init_file = "$THIS_GBS_SCRIPTS/setup/shortcut";

my @shortcut_rels;
push @shortcut_rels, ($THIS_GBS_REL eq 'beta') ? 'beta' : 'cur';
push @shortcut_rels, 'cur'
if ($IS_DEVELOPMENT);
my $define_file = ($IS_WIN32) ? 'define.bat' : 'pre_define.sh';
my $os_define_spec = ENV_os_path( "$GBS_BOOT_PATH/$define_file");    # Note: shortcuts do no expand EnvVars

foreach my $shortcut_rel (@shortcut_rels)	# 'beta' and/or 'cur'
{
my $icon_name = ($shortcut_rel eq 'beta') ? 'gbs_test' : 'gbs';
my $icon_spec = "$GBS_BOOT_PATH/$icon_name";
my $os_define_command = "$os_define_spec $shortcut_rel";
foreach my $ref (@SHORTCUT_REFS)
{
my ($shortcut_name, $shortcut_type, $action, $locations_ref, $comment) = @{$ref};
my $command_spec;
my $command_args;
if ($shortcut_type eq 'Application')
{
$command_spec = undef;
$command_args = sprintf( $SHORTCUT_ARGS_FORMAT, "$os_define_command $action");
} else	# $shortcut_type eq 'Link'
{
$command_spec = "$THIS_GBS_SCRIPTS/$action";
$command_args = '';
}
$update_count += SHORTCUT_update( $locations_ref,
$init_file,
sprintf( $shortcut_name, $shortcut_rel),
$shortcut_type,	    # $type: Application or Link
sprintf( $comment, $shortcut_rel),

$icon_spec,		    # $icon_spec
$command_spec,	    # $command_spec
$command_args,	    # $command_args
undef,		    # $working_directory
undef);		    # show: 0 = NORMAL, 1 = Maximised, -1 = Minimised
}
}




if ($update_count > 0)
{
ENV_sig( I => "$update_count standard GBS Shortcut(s) on the DeskTop and StartMenu were updated.",
"If you have your own GBS Shortcuts, please update them accordingly.");
}
} else
{
my $dtp = ENV_get_desktop_path( 0);
ENV_say( 1, "Desktop directory ( $dtp) not found.",
"No checking or creation of Shortcuts");
}
}








sub SETTINGS_uninstall_version($)
{
my ($version,
) = @_;




my $gbs_scripts_rel = ENV_getenv( 'GBS_SCRIPTS_REL');
if ($version eq $gbs_scripts_rel || $version =~ /^beta/)
{
ENV_say( 2, 'Removing Desktop and Startmenu shortcuts...');

my $shortcut_rel = ($version =~ /^beta/) ? 'beta' : 'cur';
foreach my $ref (@SHORTCUT_REFS)
{
my ($shortcut_name, $shortcut_type, $action, $locations_ref, $comment) = @{$ref};
SHORTCUT_delete( $locations_ref, sprintf( $shortcut_name, $shortcut_rel));
}
}
}




sub SETTINGS_uninstall_gbs()
{



{
ENV_say( 2, 'Removing Startmenu directory...');
my $gbs_startmenu_path = "$STARTMENU_PATH/GBS";
if ($IS_DEVELOPMENT)
{
ENV_say( 2, "- Simulate Removing $gbs_startmenu_path");
} else
{
ENV_rmtree( $gbs_startmenu_path, 1, 1, 'E');
}
}




ENV_say( 2, 'Removing .gbs/base directory...');
ENV_say( 3, "Removing $GBS_BASE_PATH");
if ($IS_DEVELOPMENT)
{
ENV_say( 3, "- Simulate Removing $GBS_BASE_PATH");
} else
{
ENV_rmtree( $GBS_BASE_PATH, 1, 1, 'E');
}




ENV_say( 2, 'Removing .gbs directory...');
ENV_say( 3, "Removing $GBS_BOOT_PATH");
if ($IS_DEVELOPMENT)
{
ENV_say( 3, "- Simulate Removing $GBS_BOOT_PATH");
} else
{
ENV_rmtree( $GBS_BOOT_PATH, 1, 1, 'E');
}




if ($IS_LINUX)
{
ENV_say( 2, 'Removing GBS alias entries...');

my $login_filespec = BASH_parse( 'EE');

my %env_file_specs;
foreach my $ref (@LINUX_ALIAS_REFS)
{
my ($alias) = @{$ref};
my ($value, $filespec, $is_login, $line_nr) = BASH_get_alias( $alias);
$env_file_specs{$filespec} = 1
if (defined $value);
}
foreach my $filespec (keys %env_file_specs)
{
BASH_remove_aliases( $filespec, 1, 'GBS', @LINUX_ALIAS_REFS);
}

ENV_say( 2, 'Removing GBS startup Script...');
BASH_replace_lines( $login_filespec, 1, 'GBS Startup', undef);

BASH_cleanup();
}
}




sub win32_check_registry()
{



ENV_say( 1, "Checking Registry...");
foreach my $envvar_name (WINREG_get_userenv_names())
{
if (substr( uc $envvar_name, 0, 3) eq 'GBS')
{
my $value = WINREG_get_userenv( $envvar_name);
$value = '' if (!defined $value);
check_predefined_envvar( $envvar_name, $value, 'Registry')
}
}
}




sub linux_check_login_files()
{



ENV_say( 1, "Checking Login-files...");

my $login_filespec = BASH_parse( 'EE');

foreach my $envvar_name (BASH_get_envvar_names( 1))  # export_variables only
{
my ($value, $is_export, $is_login, $filespec, $line_nr) = BASH_get_envvar( $envvar_name);

ENV_sig( W => "- It is unnecessary and unwise to set EnvVars in '$filespec($line_nr)'")
if (!$is_login);
if (substr( $envvar_name, 0, 3) eq 'GBS')
{
check_predefined_envvar( $envvar_name, $value, "$filespec($line_nr)");
}
}




BASH_replace_lines( $login_filespec, 1, 'GBS Startup', \@LINUX_BASHRC_LINES);




my %env_file_specs;
foreach my $ref (@LINUX_ALIAS_REFS)
{
my ($alias) = @{$ref};
my ($value, $filespec, $is_login, $line_nr) = BASH_get_alias( $alias);
$env_file_specs{$filespec} = 1
if (defined $value);
}
if (%env_file_specs)
{
ENV_say( 2, 'Removing GBS alias entries...');

foreach my $filespec (keys %env_file_specs)
{
BASH_remove_aliases( $filespec, 1, 'GBS', @LINUX_ALIAS_REFS);
}
}

BASH_cleanup();
}






sub check_predefined_envvar($$$)
{
my ($name,		    # Must start with 'GBS'. Mixed case ok.
$value,
$source,	    # For errormessage. Win32: 'Registry. Linux: "$filespec($line_nr)"
) = @_;

my $warn_line;
my $uc_name = uc $name;
if (substr( $uc_name, 0, 4) eq 'GBS_')
{
if (!GBSENV_is_predefined_env( $uc_name))	# E.g: GBS_PERL_PATH, GBS_BASE_PATH
{
$warn_line = "** This GBS EnvVar may not be defined here! **";
}
} elsif (substr( $uc_name, 0, 7) eq 'GBSEXT_')
{
$warn_line = "** GBSEXT_ EnvVars must be defined in the switch.gbs$SHELL_FILE_TYPE file! **";
} elsif (substr( $uc_name, 0, 7) eq 'GBSALL_')
{
$warn_line = "** GBSALL_ EnvVars must be defined in the gbsall$SHELL_FILE_TYPE file! **";
}

if ($warn_line)
{
ENV_sig( W => "User EnvVar $name=$value",
$warn_line,
"- $source");
}
}





sub copy_setup_files()
{
my $nr_copied = 0;

my $from_dir = "$THIS_GBS_SCRIPTS/setup";

foreach my $file (SETTINGS_get_setup_files( $from_dir))
{
my $filespec = "$from_dir/$file";
my $replicated = FILE_replicate( $filespec, "$GBS_BOOT_PATH/$file");
if ($replicated == 1)
{
ENV_say( 1, "Copied $file to $GBS_BOOT_PATH");
if ($IS_LINUX && $file =~ /\.sh$/)
{
ENV_say( 2, "- chmod u+x");
ENV_chmod( 'u+x', "$GBS_BOOT_PATH/$file");
}
$nr_copied++;
}
}

return $nr_copied;
}




sub SETTINGS_get_setup_files($)
{
my ($setup_path,		# e.g.: $GBS_SCRIPTS_PATH/setup
) = @_;
my @files;


my @wild_files = ($IS_WIN32) ? ( '*.bat', '*.ico') : ( '*.sh', '*.png');	# scripts and icons
foreach my $wild_file (@wild_files)
{
foreach my $filespec (ENV_glob( "$setup_path/$wild_file"))
{
push @files, ENV_split_spec_f( $filespec);
}
}

return @files;
}

1;

