#==============================================================
#
#   swrglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swrglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWRGLO_validate
SWRGLO_split_spec
SWRGLO_find_root_path
);
}




use glo::env;
use glo::build;
use gbsenv;
use gbsrc;
use system;
use roots;
use gbsglo;
use gbscheck;




sub SWRGLO_validate($$$$$$);
sub SWRGLO_split_spec($);
sub SWRGLO_find_root_path($);




my $CWD = ENV_cwd();
my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');

my ($THIS_CMD_EXT, $OTHR_CMD_EXT) = ENV_shell_filetype();




sub SWRGLO_validate($$$$$$)
{
my ($root_path,	    # '', '-', '.', '!!', '!<os_spec>', <number>, <perl_path_nospace>
$target,	    # '', '-', '.', <target>
$subsys,	    # '', '-', '.', <subsys>
$component,	    # '', '-', '.', <component>
$audit,		    # '', '-', '.', <audit>
$tool,		    # '', '-', '.', <tool>
) = @_;
my ($new_root_path, $new_target, $new_subsys, $new_component, $new_audit, $new_tool);

if (0)
{
CORE::say( "** ROOT_PATH = $root_path");
CORE::say( "** SUBSYS    = $subsys");
CORE::say( "** COMP      = $component");
CORE::say( "** TARGET    = $target");
CORE::say( "** AUDIT     = $audit");
CORE::say( "** TOOL      = $tool");
CORE::say( "**");
}












{

if ($root_path eq '-')
{
$new_root_path = '';
} else
{



my $this_root_path;
if (substr( $root_path, 0, 1) eq '!')
{
if ($root_path eq '!!')
{
$this_root_path = $CWD;
} else
{
my $this_path = ($root_path eq '!') ? $CWD : substr( $root_path, 1);
($this_root_path, my $found_subsys, my $found_component) = SWRGLO_split_spec( $this_path);
if ($this_root_path ne '')
{
if ($found_subsys ne '')
{
$subsys = $found_subsys
if ($subsys eq '.' || $subsys eq '');
if ($found_component ne '')
{
$component = $found_component
if ($component eq '.' || $component eq '');
} else
{
$component = '-';
}
} else
{
$subsys = '-';
$component = '-';
}
ENV_whisper( 1, "Found: $this_root_path / $found_subsys / $found_component");
} else
{
ENV_sig( EE => "No GBS-Root found in '$this_path'");
}
}
} else	# $root_path eq '', '.', <number> or <spec>
{
$this_root_path = $root_path;
}

if ($this_root_path ne '')
{



if ($this_root_path !~ /^(\.|\d+)$/)	# Not a '.' or number
{
$this_root_path = ENV_perl_path_nospace( $this_root_path);
ENV_sig( EE => "Root '$this_root_path' is not a valid GBS Root")
if (!-e "$this_root_path/system.gbs");
}
my $default_root_path = GBSRC_get_base( 'root');
my @all_roots = ROOTS_get_all();
$new_root_path = GBSCHECK_root( $this_root_path, $default_root_path, \@all_roots);




if ($new_root_path ne '')
{
my $build_now = ENV_getenv( 'GBS_BUILD');
my $build_start = SYSTEM_get( $new_root_path, 'build_start');
my $build_end = SYSTEM_get( $new_root_path, 'build_end');

if ($build_now < $build_start ||
($build_end && $build_now > $build_end))
{
my $build_now_print = BUILD_build_2_version( $build_now);
my $build_start_print = BUILD_build_2_version( $build_start);
my $build_end_print = BUILD_build_2_version( $build_end);
if ($build_now < $build_start)
{
ENV_sig( EE => "Current GBS version: $build_now_print ($build_now) too old.",
"Need at least GBS version $build_start_print ($build_start).",
"Range: [$build_start-$build_end]");

} else
{
ENV_sig( EE => "Current GBS version: $build_now_print ($build_now) too new.",
"Need latest GBS version $build_end_print ($build_end).",
"Range: [$build_start-$build_end]");
}
$new_root_path = '';
}
}

if ($new_root_path ne '')
{



if (! ROOTS_exists( $new_root_path))
{
ENV_sig( W => "'$new_root_path' is not listed in Roots List",
"Use 'swr --add' to add the Root to the Roots List",
"Continuing anyway...");
}
}
}
}

if ($new_root_path eq '')
{
$subsys = '-';
$component = '-';
$target = '-';
$audit = '-';
$tool = '-';
} else
{
my $new_gbs_root_version = SYSTEM_get( $new_root_path, 'root_version');
my $gbs_version = ENV_getenv( 'GBS_VERSION');
if ($new_gbs_root_version lt $gbs_version)
{
ENV_sig( W => "GBS ROOT-VERSION ($new_gbs_root_version) mismatch ($gbs_version)",
"Please run gbsmaint 7 9 (Upgrade)",
"Currencies are reset");
$subsys = '-';
$component = '-';
$target = '-';
$audit = '-';
$tool = '-';
}
}
}









{

if ($subsys eq '-')
{
$new_subsys = '';
$component = '-';
} else # $subsys eq '', '.' or <name>
{
my $default_subsys = GBSRC_get_root( $new_root_path, 'subsys');
$new_subsys = GBSCHECK_subsys( $new_root_path, $subsys, $default_subsys);
$component = '-'
if ($new_subsys eq '');
}
}









{

if ($component eq '-')
{
$new_component = '';
} else # $component eq '', '.' or <name>
{
if (GBSGLO_subsystem_is_full_gbs( $new_subsys, $new_root_path))
{
my $default_component = GBSRC_get_subsys( $new_root_path, $new_subsys, 'component');
$new_component = GBSCHECK_component( $new_root_path, $new_subsys, $component, $default_component);
} else
{
if ($component eq '')
{
$new_component = '';
} else
{
ENV_sig( EE => "Cannot specify a component ($component) for Non GBS SubSystem");
}
}
}
}









{

if ($target eq '-')
{
$new_target = '';
} else # $target eq '.' or <name>
{
my $default_target = GBSRC_get_root( $new_root_path, 'target');
my @targets = map { $_->[0] } SYSTEM_get_os_refs( $new_root_path, '.', 'targets');   # [ $target, $plugin ]
$new_target = GBSCHECK_target( $new_root_path, $new_subsys, $new_component, $target, $default_target, \@targets);
}
}





if ($target ne '-')
{

if ($audit eq '-')
{
$new_audit = '';
} else # $audit eq '.' or <name>
{
my @audits = map { $_->[0] } SYSTEM_get_os_refs( $new_root_path, '.', 'audits');   # [ $audit, $plugin ]
if (@audits)
{
my $default_audit = GBSRC_get_root( $new_root_path, 'audit');
$new_audit = GBSCHECK_audit( $new_root_path, $new_subsys, $new_component,
$new_target, $audit, $default_audit, \@audits);
}
}
} else
{
$new_audit = '';
}




{

if ($tool eq '-')
{
$new_tool = '';
} else # $tool eq '.' or <name>
{
my @tools = map { $_->[0] } SYSTEM_get_os_refs( $new_root_path, '.', 'tools');   # [ $tool, $plugin ]
if (@tools)
{
my $default_tool = GBSRC_get_root( $new_root_path, 'tool');
$new_tool = GBSCHECK_tool( $new_root_path, $new_subsys, $new_component, $tool, $default_tool, \@tools);
}
}
}

if (0)
{
CORE::say( "** NEW_ROOT_PATH = $new_root_path");
CORE::say( "** NEW_SUBSYS    = $new_subsys");
CORE::say( "** NEW_COMP      = $new_component");
CORE::say( "** NEW_TARGET    = $new_target");
CORE::say( "** NEW_AUDIT     = $new_audit");
CORE::say( "** NEW_TOOL      = $new_tool");
CORE::say( "**");
}


return ($new_root_path, $new_target, $new_subsys, $new_component, $new_audit, $new_tool);
}





sub SWRGLO_find_root_path($)
{
my ($filespec) = @_;	# Abs dir or file spec
my $root_path = '';

my @dirs = split( '/', $filespec);
pop @dirs if (-f $filespec);
while (@dirs)
{
my $possible_root_path = join( '/', @dirs);
if (GBSCHECK_system( $possible_root_path))
{
$root_path = $possible_root_path;
last;
}
pop @dirs;
}

return $root_path;
}




sub SWRGLO_split_spec($)
{
my ($filespec) = @_;
my ($root_path, $subsys, $component, $remain) = ( '', '', '', '' );

my $gbs_root_path = GBSENV_gbs_root_path();
if ($gbs_root_path ne '' && ENV_is_in_path( $filespec, $gbs_root_path))
{
$root_path = $gbs_root_path;
} else
{
$root_path = SWRGLO_find_root_path( $filespec);
}

if ($root_path ne '')
{
if ($filespec =~ m!$root_path/dev/([^/]+)!)
{
$subsys = $1;
if ($filespec =~ m!$root_path/dev/$subsys/comp/([^/]+)!)
{
$component = $1;
$remain = substr( $filespec, length( "$root_path/dev/$subsys/comp/$component"));
} else
{
$remain = substr( $filespec, length( "$root_path/dev/$subsys"));
}
} else
{
$remain = substr( $filespec, length( $root_path));
}
$remain = substr( $remain, 1) if ($remain =~ m!^/!);
}

ENV_whisper( 1,  "Found: $root_path, $subsys, $component, $remain");
return ($root_path, $subsys, $component, $remain);
}

1;


