#===============================================================
#
#   upgrade.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package upgrade;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
UPGRADE_main
);
}




use glo::env;
use glo::ask;
use glo::scm;
use glo::slurp;
use gbsenv;
use plugin;
use system;
use gbsscm;
use fix;
use gbsglo;
use audit;
use target;
use tool;




sub UPGRADE_main($$$);

sub upgrade_301_400();
sub update_system_root_version();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_VERSION = ENV_getenv( 'GBS_VERSION');		    # format: "R.VV"
my $GBS_ROOT_VERSION = ENV_getenv( 'GBS_ROOT_VERSION');	    # format: "R.VV"
my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
my @GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');

my ($THIS_CMD_EXT, $OTHR_CMD_EXT) = ENV_shell_filetype();

my @UPGRADE_REFS = (
[ qw( 2.00 3.00) ],
[ qw( 3.00 3.01) ],

);




sub UPGRADE_main($$$)
{
my ($menu_entry,
$data_ref,
$entries_ref
) = @_;



ENV_say( 1, "This ROOT   version is $GBS_ROOT_VERSION",
"Current GBS version is $GBS_VERSION");
if ($GBS_ROOT_VERSION lt '2.00')  # format: "R.VV"
{
ENV_sig( E => "Root Version $GBS_ROOT_VERSION way too old. Cannot upgrade.",
'You will have to create a new Root/System and fill it with data from this Root/System');
} else
{
my $must_use_older_gbs = 0;
foreach my $ref (@UPGRADE_REFS)
{
my ($first_version, $last_version) = @{$ref};
if ($GBS_ROOT_VERSION lt $last_version)
{
ENV_say( 1, "Must upgrade $first_version to $last_version using GBS $last_version first",
'You can temporarily switch to another version of GBS by starting a new GBS Window',
"or 'gbsexit' the current and then start the wanted version of GBS with the command",
"'gbs --rel=<version>' or 'gbs --rel=?'");
$must_use_older_gbs = 1;
}
}

my $ans;
if ($must_use_older_gbs)
{
ENV_sig( EE => "Root Version $GBS_ROOT_VERSION is too old, use previous version of GBS to upgrade");
} else
{
if ($GBS_VERSION gt $GBS_ROOT_VERSION)
{
ENV_say( 1, "Version mismatch.",
"Please read the release notes for any actions to be performed.",
"You may wish to exit this function in order to fix things first.",
"If you proceed with the Upgrade, this function will:",
"- Make the changes in the directory/file structure",
"- Check and repair the GBS system file(s)",
"- Check and repair the directory structure (as a final check)",
"- Set the ROOT-version to the current GBS version.",
"Make sure you have an UNMODIFIED work-area.",
"Immediately execute 'swr .' after the upgrade!",
"Immediately Checkin/Commit/Consolidate the work-area after the upgrade!");
$ans = ASK_YN( 'Upgrade?', 'Y');
} elsif ($GBS_VERSION eq $GBS_ROOT_VERSION)
{
ENV_say( 1, "Root-Version is Up-to-date");
$ans = ASK_YN( 'Force last Upgrade again?', 'N');
} else
{
ENV_sig( EE => "This GBS version is too old for this Root version");
}
}

if ($ans eq 'Y')
{
GBSSCM_preset( 0);
GBSSCM_connect();




upgrade_301_400();




my $ok = FIX_structure();




if ($GBS_VERSION ne $GBS_ROOT_VERSION)
{
if ($ok)
{
if (ASK_YN( 'Set ROOT-version?', 'Y') eq 'Y')
{
update_system_root_version();
}
} else
{
ENV_sig( W => 'Cannot update Root-Version: Missing/Incomplete files/directories');
}
}
ENV_say( 1, "*** You MUST run 'swr .' before continuing with other GBS commands! ***");
}
}

return;
}




sub upgrade_301_400()
{
ENV_say( 1, "Upgrading 3.01 to 4.00...");




{
my @move_pair_refs;


push @move_pair_refs, [ "$GBS_ROOT_PATH/sysgen", "$GBS_ROOT_PATH/sysbuild" ]
if (-e  "$GBS_ROOT_PATH/sysgen");

SCM_move( \@move_pair_refs, 0, 0)		# $pre_validated, $verbose
if (@move_pair_refs);
}




{
my @move_pair_refs;


foreach my $subsys (GBSGLO_subsystems())
{
push @move_pair_refs, [ "$GBS_ROOT_PATH/dev/$subsys/gen", "$GBS_ROOT_PATH/dev/$subsys/build" ]
if (-e  "$GBS_ROOT_PATH/dev/$subsys/gen");
}

SCM_move( \@move_pair_refs, 0, 0)		# $pre_validated, $verbose
if (@move_pair_refs);
}




{
my @names_refs = (

[ sysaudit => 'audit',  '>qac/bin/qacsum.pl',   'SCA' ],
[ sysaudit => 'audit',  'qacmaint',		    'prqamaint' ],
[ sysbuild => 'target', 'gnu',		    'lgnu_cpp' ],
);
foreach my $ref (@names_refs)
{
my ($file_dir, $file_name, $old_name, $new_name) = @{$ref};
my $qm_old_name = quotemeta( $old_name);
foreach my $type (qw( .usr .gbs))
{
my @filespecs = ENV_glob( "$GBS_ROOT_PATH/$file_dir/*/$file_name$type");
foreach my $filespec (@filespecs)
{
my $lines = SLURP_file( $filespec);
if ($lines =~ s/$qm_old_name/$new_name/gs)
{
SCM_store_file( $filespec, $lines);
ENV_say( 2, "$filespec: $old_name => $new_name");
}
}
}
}
}




my %get_plugin_funcs = (

audits	=> \&AUDIT_get_plugin,
targets	=> \&TARGET_get_plugin,
tools	=> \&TOOL_get_plugin,
);
foreach my $item (qw( audits targets tools ))
{
my $get_plugin_func = $get_plugin_funcs{$item};
foreach my $osname (SYSTEM_get_osnames( $GBS_ROOT_PATH, $item))
{
my @refs = SYSTEM_get_os_refs( $GBS_ROOT_PATH, $osname, $item);
my $changed = 0;
foreach my $ref ( @refs)	# [ $item, $plugin ]
{
if (!defined $ref->[1])	    # $value (plugin)
{
my $name = $ref->[0];    # $name ($item)
my $value = eval { $get_plugin_func->( $name) };
if (defined $value)
{
$ref->[1] = $value;
$changed = 1;
my $item_name = ucfirst( substr( $item, 0, -1));  # remove the trailing 's'
ENV_say( 2, "$item $osname $name plugin set to $value");
}
}
}
SYSTEM_put_os_refs( $GBS_ROOT_PATH, $osname, $item, \@refs)
if ($changed);
}
}
SYSTEM_write ();	# Includes backup and SCM_checkout
PLUGIN_reset();
}




sub update_system_root_version()
{
ENV_say( 1, "Updating ROOT version...");
SYSTEM_put( $GBS_ROOT_PATH, root_version => $GBS_VERSION);
SYSTEM_write();    # Includes backup and SCM_checkout
$GBS_ROOT_VERSION = $GBS_VERSION;
ENV_setenv( GBS_ROOT_VERSION => $GBS_ROOT_VERSION);
}

1;
