#===============================================
#
#   validate.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package validate;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
VALIDATE_gbs
VALIDATE_root
VALIDATE_subsys
VALIDATE_subsys_full_gbs
VALIDATE_subsys_audit
VALIDATE_component
VALIDATE_target
VALIDATE_target_no_subsys
VALIDATE_targets
VALIDATE_audit
VALIDATE_audit_no_subsys
VALIDATE_audits
VALIDATE_tool
VALIDATE_tool_no_subsys
VALIDATE_tools
VALIDATE_integrator
VALIDATE_administrator
);
}




use glo::env;
use glo::list;
use glo::ask;
use gbsenv;
use gbsglo;




sub VALIDATE_gbs();
sub VALIDATE_root();
sub VALIDATE_subsys();
sub VALIDATE_subsys_full_gbs();
sub VALIDATE_subsys_audit();
sub VALIDATE_component();
sub VALIDATE_target();
sub VALIDATE_target_no_subsys();
sub VALIDATE_targets($);
sub VALIDATE_audit($);
sub VALIDATE_audit_no_subsys($);
sub VALIDATE_audits($);
sub VALIDATE_tool($);
sub VALIDATE_tool_no_subsys($);
sub VALIDATE_tools($);
sub VALIDATE_integrator();
sub VALIDATE_administrator();

sub validate_user($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = '';
my $GBS_SUBSYS_PATH = '';

my @GBS_ALL_TARGETS = ENV_getenv( 'GBS_ALL_TARGETS');
my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');

my @GBS_ALL_AUDITS = ENV_getenv( 'GBS_ALL_AUDITS');
my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');

my @GBS_ALL_TOOLS = ENV_getenv( 'GBS_ALL_TOOLS');
my @GBS_TOOLS = ENV_getenv( 'GBS_TOOLS');

my $ROOT_VALIDATED = 0;




sub VALIDATE_gbs()
{
ENV_sig( EE => "Unable to continue: GBS is not active")
if (!GBSENV_gbs_is_active());
}






sub VALIDATE_root()
{
if (!$ROOT_VALIDATED)
{
VALIDATE_gbs();
ENV_sig( EE => "Unable to continue: No current Root")
if ($GBS_ROOT_PATH eq "");
$ROOT_VALIDATED = 1;
}

return $GBS_ROOT_PATH;
}






sub VALIDATE_subsys()
{
VALIDATE_root();
$GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
ENV_sig( EE => "Unable to continue: No current SubSystem")
if ($GBS_SUBSYS eq "");
$GBS_SUBSYS_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS";

return ($GBS_ROOT_PATH, $GBS_SUBSYS);
}






sub VALIDATE_subsys_full_gbs()
{
VALIDATE_subsys();

ENV_sig( EE => "This command is only possible for full GBS Subsystems")
if (! GBSGLO_subsystem_is_full_gbs( $GBS_SUBSYS));

return ($GBS_ROOT_PATH, $GBS_SUBSYS);
}






sub VALIDATE_subsys_audit()
{
VALIDATE_subsys_full_gbs();

ENV_sig( EE => "No Audit(s) defined for SubSystem '$GBS_SUBSYS'")
if (!scalar glob "$GBS_SUBSYS_PATH/audit/*");

return ($GBS_ROOT_PATH, $GBS_SUBSYS);
}






sub VALIDATE_component()
{
my $component;

VALIDATE_subsys_full_gbs();

$component = ENV_getenv( 'GBS_COMPONENT');
ENV_sig( EE => "Unable to continue: No current Component")
if ($component eq "");

return ($GBS_ROOT_PATH, $GBS_SUBSYS, $component);
}




sub VALIDATE_target()
{
VALIDATE_target_no_subsys();

if ($GBS_SUBSYS ne '')
{
ENV_sig( EE => "SubSystem ($GBS_SUBSYS) does not generate for Target '$GBS_TARGET'")
if (! GBSGLO_subsystem_does_build( $GBS_SUBSYS, $GBS_TARGET));
}
}




sub VALIDATE_target_no_subsys()
{
ENV_sig( EE => "Unable to continue: No current Target")
if ($GBS_TARGET eq "");
ENV_sig( EE => "No such Target '$GBS_TARGET' (@GBS_TARGETS)")
if (! -d "$GBS_ROOT_PATH/sysbuild/$GBS_TARGET");
}






sub VALIDATE_targets($)
{
my ($in_targets_ref) = @_;		# (possibly wild) Targets, '.' == current, '-' = None
my @out_targets;

ENV_sig( EE => "No Targets defined")
if (!@GBS_ALL_TARGETS);
ENV_sig( EE => "No Targets defined for this Platform ($^O)")
if (!@GBS_TARGETS);

foreach my $target (@{$in_targets_ref})
{
if ($target eq '.')
{
ENV_sig( EE => "No current Target")
if ($GBS_TARGET eq '');
push @out_targets, $GBS_TARGET;
} elsif ($target eq '-')
{
push @out_targets, $target;
} else
{
my @exp_targets = ENV_wildcard( $target, [ @GBS_TARGETS ]);
if (!@exp_targets)
{
@exp_targets = ENV_wildcard( $target, [ @GBS_ALL_TARGETS ]);
if (@exp_targets)
{
ENV_sig( EE => "No such Target '$target' for this Platform ($^O)");
} else
{
ENV_sig( EE => "No such Target '$target' (@GBS_TARGETS)");
}
}
push @out_targets, @exp_targets;
}
}

if (@out_targets)
{
LIST_unique( \@out_targets);
} else
{
ENV_sig( EE => "No current Target")
if ($GBS_TARGET eq '');
@out_targets = ($GBS_TARGET);
}

return @out_targets;
}






sub VALIDATE_audit($)
{
my ($audit) = @_;

VALIDATE_audit_no_subsys( $audit);
ENV_sig( EE => "SubSystem ($GBS_SUBSYS) does not audit for Audit '$audit' and Target '$GBS_TARGET'")
if (! GBSGLO_subsystem_does_audit( $GBS_SUBSYS, $audit, $GBS_TARGET));

return $audit;
}






sub VALIDATE_audit_no_subsys($)
{
my ($audit) = @_;

ENV_sig( EE => "No such Audit '$audit' (@GBS_ALL_AUDITS)")
if (! -d "$GBS_ROOT_PATH/sysaudit/$audit");

return $audit;
}






sub VALIDATE_audits($)
{
my ($audits_ref) = @_;		# (possibly wild) Audits, '.' == current, '-' = None
my @new_audits;

ENV_sig( EE => "No Audits defined")
if (!@GBS_ALL_AUDITS);
ENV_sig( EE => "No Audits defined for this Platform ($^O)")
if (!@GBS_AUDITS);

foreach my $audit (@{$audits_ref})
{
if ($audit eq '.')
{
my $gbs_audit = ENV_getenv( 'GBS_AUDIT');
ENV_sig( EE => 'No current Audit')
if ($gbs_audit eq '');
push @new_audits, $gbs_audit;
} elsif ($audit eq '-')
{
push @new_audits, $audit;
} else
{
my @exp_audits = ENV_wildcard( $audit, \@GBS_ALL_AUDITS );
ENV_sig( EE => "No such Audit '$audit' (@GBS_ALL_AUDITS)")
if (!@exp_audits);
push @new_audits, @exp_audits;
}
}

if (@new_audits)
{
LIST_unique( \@new_audits);
}

return @new_audits;
}






sub VALIDATE_tool($)
{
my ($tool) = @_;

VALIDATE_tool_no_subsys( $tool);
ENV_sig( EE => "SubSystem ($GBS_SUBSYS) does not execute Tool '$tool'")
if (! GBSGLO_subsystem_does_tool( $GBS_SUBSYS, $tool));

return $tool;
}






sub VALIDATE_tool_no_subsys($)
{
my ($tool) = @_;

ENV_sig( EE => "No such TOOL '$tool' (@GBS_ALL_TOOLS)")
if (! -d "$GBS_ROOT_PATH/systool/$tool");

return $tool;
}






sub VALIDATE_tools($)
{
my ($tools_ref) = @_;		# (possibly wild) Tools, '.' == current, '-' = None
my @new_tools;

ENV_sig( EE => "No Tools defined")
if (!@GBS_ALL_TOOLS);
ENV_sig( EE => "No Tools defined for this Platform ($^O)")
if (!@GBS_TOOLS);

foreach my $tool (@{$tools_ref})
{
if ($tool eq '.')
{
my $gbs_tool = ENV_getenv( 'GBS_TOOL');
ENV_sig( EE => 'No current Tool')
if ($gbs_tool eq '');
push @new_tools, $gbs_tool;
} elsif ($tool eq '-')
{
push @new_tools, $tool;
} else
{
my @exp_tools = ENV_wildcard( $tool, \@GBS_ALL_TOOLS );
ENV_sig( EE => "No such Tool '$tool' (@GBS_ALL_TOOLS)")
if (!@exp_tools);
push @new_tools, @exp_tools;
}
}

if (@new_tools)
{
LIST_unique( \@new_tools);
}

return @new_tools;
}




sub VALIDATE_integrator()
{
return validate_user( GBSENV_is_integrator(), 'Integrator');
}




sub VALIDATE_administrator()
{
return validate_user( GBSENV_is_administrator(), 'Administrator');
}




sub validate_user($$)
{
my ($is_ok,
$user_type, 	# 'Administrator', 'Integrator'
) = @_;


if (!$is_ok)
{
ENV_beep();
ENV_say( 1, "**");
ENV_say( 1, "** Warning: You are not an $user_type for this System");
ENV_say( 1, "**");
$is_ok = (ASK_YN( "Continue anyway?", 'N') eq 'Y');

ENV_sig( WE =>) if (!$is_ok && !defined wantarray)
}

return $is_ok;
}

1;

